@@ -1,4 +1,3 @@
-# coding: utf-8
 # Copyright (c) Pymatgen Development Team.
 # Distributed under the terms of the MIT License.
 
@@ -8,20 +7,24 @@ TransformedStructure is a structure that has been modified by undergoing a
 series of transformations.
 """
 
+from __future__ import annotations
+
 import datetime
 import json
 import os
 import re
+from typing import Any
 from warnings import warn
 
-from monty.json import MontyDecoder, MSONable, jsanitize
+from monty.json import MSONable, jsanitize
 
+from pymatgen.alchemy.filters import AbstractStructureFilter
 from pymatgen.core.structure import Structure
 from pymatgen.io.cif import CifParser
 from pymatgen.io.vasp.inputs import Poscar
-from pymatgen.io.vasp.sets import MPRelaxSet
-
-dec = MontyDecoder()
+from pymatgen.io.vasp.sets import MPRelaxSet, VaspInputSet
+from pymatgen.transformations.transformation_abc import AbstractTransformation
+from pymatgen.util.provenance import StructureNL
 
 
 class TransformedStructure(MSONable):
@@ -33,27 +36,33 @@ class TransformedStructure(MSONable):
     associated transformation history.
     """
 
-    def __init__(self, structure, transformations=None, history=None, other_parameters=None):
+    def __init__(
+        self,
+        structure: Structure,
+        transformations: list[AbstractTransformation] = None,
+        history: list[AbstractTransformation | dict[str, Any]] = None,
+        other_parameters: dict[str, Any] = None,
+    ) -> None:
         """
         Initializes a transformed structure from a structure.
 
         Args:
             structure (Structure): Input structure
-            transformations ([Transformations]): List of transformations to
+            transformations (list[Transformation]): List of transformations to
                 apply.
-            history (list): Previous history.
+            history (list[Transformation]): Previous history.
             other_parameters (dict): Additional parameters to be added.
         """
         self.final_structure = structure
         self.history = history or []
         self.other_parameters = other_parameters or {}
-        self._undone = []
+        self._undone: list[tuple[AbstractTransformation | dict[str, Any], Structure]] = []
 
         transformations = transformations or []
         for t in transformations:
             self.append_transformation(t)
 
-    def undo_last_change(self):
+    def undo_last_change(self) -> None:
         """
         Undo the last change in the TransformedStructure.
 
@@ -63,7 +72,7 @@ class TransformedStructure(MSONable):
         if len(self.history) == 0:
             raise IndexError("Can't undo. Already at oldest change.")
         if "input_structure" not in self.history[-1]:
-            raise IndexError("Can't undo. Latest history has no " "input_structure")
+            raise IndexError("Can't undo. Latest history has no input_structure")
         h = self.history.pop()
         self._undone.append((h, self.final_structure))
         s = h["input_structure"]
@@ -71,7 +80,7 @@ class TransformedStructure(MSONable):
             s = Structure.from_dict(s)
         self.final_structure = s
 
-    def redo_next_change(self):
+    def redo_next_change(self) -> None:
         """
         Redo the last undone change in the TransformedStructure.
 
@@ -84,11 +93,11 @@ class TransformedStructure(MSONable):
         self.history.append(h)
         self.final_structure = s
 
-    def __getattr__(self, name):
+    def __getattr__(self, name) -> Any:
         s = object.__getattribute__(self, "final_structure")
         return getattr(s, name)
 
-    def __len__(self):
+    def __len__(self) -> int:
         return len(self.history)
 
     def append_transformation(self, transformation, return_alternatives=False, clear_redo=True):
@@ -147,20 +156,21 @@ class TransformedStructure(MSONable):
         self.final_structure = s
         return None
 
-    def append_filter(self, structure_filter):
+    def append_filter(self, structure_filter: AbstractStructureFilter) -> None:
         """
         Adds a filter.
 
         Args:
-            structure_filter (StructureFilter): A filter implementating the
-                AbstractStructureFilter API. Tells transmuter waht structures
-                to retain.
+            structure_filter (StructureFilter): A filter implementing the
+                AbstractStructureFilter API. Tells transmuter what structures to retain.
         """
         hdict = structure_filter.as_dict()
         hdict["input_structure"] = self.final_structure.as_dict()
         self.history.append(hdict)
 
-    def extend_transformations(self, transformations, return_alternatives=False):
+    def extend_transformations(
+        self, transformations: list[AbstractTransformation], return_alternatives: bool = False
+    ) -> None:
         """
         Extends a sequence of transformations to the TransformedStructure.
 
@@ -174,7 +184,7 @@ class TransformedStructure(MSONable):
         for t in transformations:
             self.append_transformation(t, return_alternatives=return_alternatives)
 
-    def get_vasp_input(self, vasp_input_set=MPRelaxSet, **kwargs):
+    def get_vasp_input(self, vasp_input_set: type[VaspInputSet] = MPRelaxSet, **kwargs) -> dict[str, Any]:
         """
         Returns VASP input as a dict of vasp objects.
 
@@ -186,8 +196,14 @@ class TransformedStructure(MSONable):
         d["transformations.json"] = json.dumps(self.as_dict())
         return d
 
-    def write_vasp_input(self, vasp_input_set=MPRelaxSet, output_dir=".", create_directory=True, **kwargs):
-        r"""
+    def write_vasp_input(
+        self,
+        vasp_input_set: type[VaspInputSet] = MPRelaxSet,
+        output_dir: str = ".",
+        create_directory: bool = True,
+        **kwargs,
+    ) -> None:
+        """
         Writes VASP input to an output_dir.
 
         Args:
@@ -203,7 +219,7 @@ class TransformedStructure(MSONable):
         with open(os.path.join(output_dir, "transformations.json"), "w") as fp:
             json.dump(self.as_dict(), fp)
 
-    def __str__(self):
+    def __str__(self) -> str:
         output = [
             "Current structure",
             "------------",
@@ -219,7 +235,7 @@ class TransformedStructure(MSONable):
         output.append(str(self.other_parameters))
         return "\n".join(output)
 
-    def set_parameter(self, key, value):
+    def set_parameter(self, key: str, value: Any) -> None:
         """
         Set a parameter
 
@@ -229,7 +245,7 @@ class TransformedStructure(MSONable):
         self.other_parameters[key] = value
 
     @property
-    def was_modified(self):
+    def was_modified(self) -> bool:
         """
         Boolean describing whether the last transformation on the structure
         made any alterations to it one example of when this would return false
@@ -239,7 +255,7 @@ class TransformedStructure(MSONable):
         return not self.final_structure == self.structures[-2]
 
     @property
-    def structures(self):
+    def structures(self) -> list[Structure]:
         """
         Copy of all structures in the TransformedStructure. A
         structure is stored after every single transformation.
@@ -248,7 +264,12 @@ class TransformedStructure(MSONable):
         return hstructs + [self.final_structure]
 
     @staticmethod
-    def from_cif_string(cif_string, transformations=None, primitive=True, occupancy_tolerance=1.0):
+    def from_cif_string(
+        cif_string: str,
+        transformations: list[AbstractTransformation] = None,
+        primitive: bool = True,
+        occupancy_tolerance: float = 1.0,
+    ) -> TransformedStructure:
         """
         Generates TransformedStructure from a cif string.
 
@@ -256,7 +277,7 @@ class TransformedStructure(MSONable):
             cif_string (str): Input cif string. Should contain only one
                 structure. For cifs containing multiple structures, please use
                 CifTransmuter.
-            transformations ([Transformations]): Sequence of transformations
+            transformations (list[Transformation]): Sequence of transformations
                 to be applied to the input structure.
             primitive (bool): Option to set if the primitive cell should be
                 extracted. Defaults to True. However, there are certain
@@ -289,19 +310,21 @@ class TransformedStructure(MSONable):
         return TransformedStructure(s, transformations, history=[source_info])
 
     @staticmethod
-    def from_poscar_string(poscar_string, transformations=None):
+    def from_poscar_string(
+        poscar_string: str, transformations: list[AbstractTransformation] = None
+    ) -> TransformedStructure:
         """
         Generates TransformedStructure from a poscar string.
 
         Args:
             poscar_string (str): Input POSCAR string.
-            transformations ([Transformations]): Sequence of transformations
+            transformations (list[Transformation]): Sequence of transformations
                 to be applied to the input structure.
         """
         p = Poscar.from_string(poscar_string)
         if not p.true_names:
             raise ValueError(
-                "Transformation can be craeted only from POSCAR " "strings with proper VASP5 element symbols."
+                "Transformation can be created only from POSCAR strings with proper VASP5 element symbols."
             )
         raw_string = re.sub(r"'", '"', poscar_string)
         s = p.structure
@@ -312,27 +335,27 @@ class TransformedStructure(MSONable):
         }
         return TransformedStructure(s, transformations, history=[source_info])
 
-    def as_dict(self):
+    def as_dict(self) -> dict[str, Any]:
         """
         Dict representation of the TransformedStructure.
         """
         d = self.final_structure.as_dict()
-        d["@module"] = self.__class__.__module__
-        d["@class"] = self.__class__.__name__
+        d["@module"] = type(self).__module__
+        d["@class"] = type(self).__name__
         d["history"] = jsanitize(self.history)
         d["last_modified"] = str(datetime.datetime.utcnow())
         d["other_parameters"] = jsanitize(self.other_parameters)
         return d
 
     @classmethod
-    def from_dict(cls, d):
+    def from_dict(cls, d) -> TransformedStructure:
         """
         Creates a TransformedStructure from a dict.
         """
         s = Structure.from_dict(d)
         return cls(s, history=d["history"], other_parameters=d.get("other_parameters", None))
 
-    def to_snl(self, authors, **kwargs):
+    def to_snl(self, authors, **kwargs) -> StructureNL:
         """
         Generate SNL from TransformedStructure.
 
@@ -341,7 +364,7 @@ class TransformedStructure(MSONable):
         :return: StructureNL
         """
         if self.other_parameters:
-            warn("Data in TransformedStructure.other_parameters discarded " "during type conversion to SNL")
+            warn("Data in TransformedStructure.other_parameters discarded during type conversion to SNL")
         hist = []
         for h in self.history:
             snl_metadata = h.pop("_snl", {})
@@ -352,12 +375,11 @@ class TransformedStructure(MSONable):
                     "description": h,
                 }
             )
-        from pymatgen.util.provenance import StructureNL
 
         return StructureNL(self.final_structure, authors, history=hist, **kwargs)
 
     @classmethod
-    def from_snl(cls, snl):
+    def from_snl(cls, snl: StructureNL) -> TransformedStructure:
         """
         Create TransformedStructure from SNL.
 
