@@ -1,8 +1,6 @@
-# coding: utf-8
 # Copyright (c) Pymatgen Development Team.
 # Distributed under the terms of the MIT License.
 
-
 """
 This module contains the class describing the coordination geometries that can exist in a given structure. These
 "model" coordination geometries are described in the following articles :
@@ -55,9 +53,8 @@ class AbstractChemenvAlgorithm(MSONable, metaclass=abc.ABCMeta):
     @abc.abstractmethod
     def as_dict(self):
         """
-        A JSON serializable dict representation of the algorithm
+        A JSON-serializable dict representation of the algorithm
         """
-        pass
 
     @property
     def algorithm_type(self):
@@ -104,23 +101,23 @@ class ExplicitPermutationsAlgorithm(AbstractChemenvAlgorithm):
     @property
     def as_dict(self):
         """
-        Return the JSON serializable dict representation of this ExplicitPermutationsAlgorithm algorithm.
+        Return the JSON-serializable dict representation of this ExplicitPermutationsAlgorithm algorithm.
 
-        Returns: a JSON serializable dict representation of this ExplicitPermutationsAlgorithm algorithm.
+        Returns: a JSON-serializable dict representation of this ExplicitPermutationsAlgorithm algorithm.
         """
         return {
-            "@module": self.__class__.__module__,
-            "@class": self.__class__.__name__,
+            "@module": type(self).__module__,
+            "@class": type(self).__name__,
             "permutations": self._permutations,
         }
 
     @classmethod
     def from_dict(cls, dd):
         """
-        Reconstructs the ExplicitPermutationsAlgorithm algorithm from its JSON serializable dict representation.
+        Reconstructs the ExplicitPermutationsAlgorithm algorithm from its JSON-serializable dict representation.
 
         Args:
-            dd: a JSON serializable dict representation of an ExplicitPermutationsAlgorithm algorithm.
+            dd: a JSON-serializable dict representation of an ExplicitPermutationsAlgorithm algorithm.
 
         Returns: an ExplicitPermutationsAlgorithm algorithm.
         """
@@ -146,7 +143,7 @@ class SeparationPlane(AbstractChemenvAlgorithm):
         explicit_optimized_permutations=None,
         multiplicity=None,
         other_plane_points=None,
-    ):  # , plane_safe_permutations=False):
+    ):
         """
             Initializes a separation plane for a given perfect coordination geometry
 
@@ -182,10 +179,6 @@ class SeparationPlane(AbstractChemenvAlgorithm):
         self._hash = 10000 * len(plane_points) + 100 * len(point_groups[0]) + len(point_groups[1])
         self.ordered_plane = ordered_plane
         self.ordered_point_groups = [False, False] if ordered_point_groups is None else ordered_point_groups
-        # self._ordered_indices = list(point_groups[0])
-        # self._ordered_indices.extend(plane_points)
-        # self._ordered_indices.extend(point_groups[1])
-        # self._inv_ordered_indices = np.argsort(self._ordered_indices)
         self.explicit_permutations = explicit_permutations
         self.explicit_optimized_permutations = explicit_optimized_permutations
         self._safe_permutations = None
@@ -207,24 +200,6 @@ class SeparationPlane(AbstractChemenvAlgorithm):
             len(point_groups[1]),
         )
 
-    # @property
-    # def ordered_indices(self):
-    #     """
-    #     Ordered indices of the separation plane.
-    #
-    #     Examples:
-    #          For a separation plane of type 2|4|3, with plane_points indices [0, 3, 5, 8] and
-    #          point_groups indices [1, 4] and [2, 7, 6], the list of ordered indices is :
-    #          [0, 3, 5, 8, 1, 4, 2, 7, 6].
-    #
-    #     Returns: list of ordered indices of this separation plane.
-    #     """
-    #     return self._ordered_indices
-    #
-    # @property
-    # def inv_ordered_indices(self):
-    #     return self._inv_ordered_indices
-
     @property
     def permutations(self):
         """
@@ -328,13 +303,13 @@ class SeparationPlane(AbstractChemenvAlgorithm):
     @property
     def as_dict(self):
         """
-        Return the JSON serializable dict representation of this SeparationPlane algorithm.
+        Return the JSON-serializable dict representation of this SeparationPlane algorithm.
 
-        Returns: a JSON serializable dict representation of this SeparationPlane algorithm.
+        Returns: a JSON-serializable dict representation of this SeparationPlane algorithm.
         """
         return {
-            "@module": self.__class__.__module__,
-            "@class": self.__class__.__name__,
+            "@module": type(self).__module__,
+            "@class": type(self).__name__,
             "plane_points": self.plane_points,
             "mirror_plane": self.mirror_plane,
             "ordered_plane": self.ordered_plane,
@@ -354,10 +329,10 @@ class SeparationPlane(AbstractChemenvAlgorithm):
     @classmethod
     def from_dict(cls, dd):
         """
-        Reconstructs the SeparationPlane algorithm from its JSON serializable dict representation.
+        Reconstructs the SeparationPlane algorithm from its JSON-serializable dict representation.
 
         Args:
-            dd: a JSON serializable dict representation of an SeparationPlane algorithm.
+            dd: a JSON-serializable dict representation of an SeparationPlane algorithm.
 
         Returns: a SeparationPlane algorithm.
         """
@@ -417,7 +392,7 @@ class CoordinationGeometry:
                     neighbors set could be found from a "cap hint".
             """
             if hints_type not in self.ALLOWED_HINTS_TYPES:
-                raise ValueError('Type "{}" for NeighborsSetsHints is not allowed'.format(type))
+                raise ValueError(f'Type "{type}" for NeighborsSetsHints is not allowed')
             self.hints_type = hints_type
             self.options = options
 
@@ -433,7 +408,7 @@ class CoordinationGeometry:
             """
             if hints_info["csm"] > self.options["csm_max"]:
                 return []
-            return object.__getattribute__(self, "{}_hints".format(self.hints_type))(hints_info)
+            return object.__getattribute__(self, f"{self.hints_type}_hints")(hints_info)
 
         def single_cap_hints(self, hints_info):
             """
@@ -534,17 +509,17 @@ class CoordinationGeometry:
 
         def as_dict(self):
             """
-            A JSON serializable dict representation of this NeighborsSetsHints
+            A JSON-serializable dict representation of this NeighborsSetsHints
             """
             return {"hints_type": self.hints_type, "options": self.options}
 
         @classmethod
         def from_dict(cls, dd):
             """
-            Reconstructs the NeighborsSetsHints from its JSON serializable dict representation.
+            Reconstructs the NeighborsSetsHints from its JSON-serializable dict representation.
 
             Args:
-                dd: a JSON serializable dict representation of a NeighborsSetsHints.
+                dd: a JSON-serializable dict representation of a NeighborsSetsHints.
 
             Returns: a NeighborsSetsHints.
             """
@@ -592,7 +567,7 @@ class CoordinationGeometry:
             algorithms: Algorithms used to identify this coordination geometry.
             equivalent_indices: The equivalent sets of indices in this coordination geometry (can be used to skip
                 equivalent permutations that have already been performed).
-            neighbors_sets_hints: Neighors sets hints for this coordination geometry.
+            neighbors_sets_hints: Neighbors sets hints for this coordination geometry.
         """
         self._mp_symbol = mp_symbol
         self.name = name
@@ -620,7 +595,7 @@ class CoordinationGeometry:
 
     def as_dict(self):
         """
-        A JSON serializable dict representation of this CoordinationGeometry.
+        A JSON-serializable dict representation of this CoordinationGeometry.
         """
         return {
             "mp_symbol": self._mp_symbol,
@@ -645,10 +620,10 @@ class CoordinationGeometry:
     @classmethod
     def from_dict(cls, dd):
         """
-        Reconstructs the CoordinationGeometry from its JSON serializable dict representation.
+        Reconstructs the CoordinationGeometry from its JSON-serializable dict representation.
 
         Args:
-            dd: a JSON serializable dict representation of a CoordinationGeometry.
+            dd: a JSON-serializable dict representation of a CoordinationGeometry.
 
         Returns: a CoordinationGeometry.
         """
@@ -680,23 +655,23 @@ class CoordinationGeometry:
     def __str__(self):
         symbol = ""
         if self.IUPAC_symbol is not None:
-            symbol += " (IUPAC: {s}".format(s=self.IUPAC_symbol)
+            symbol += f" (IUPAC: {self.IUPAC_symbol}"
             if self.IUCr_symbol is not None:
-                symbol += " || IUCr: {s})".format(s=self.IUCr_symbol)
+                symbol += f" || IUCr: {self.IUCr_symbol})"
             else:
                 symbol += ")"
         elif self.IUCr_symbol is not None:
-            symbol += " (IUCr: {s})".format(s=self.IUCr_symbol)
+            symbol += f" (IUCr: {self.IUCr_symbol})"
         outs = [
-            "Coordination geometry type : {n}{s}\n".format(n=self.name, s=symbol),
-            "  - coordination number : {c}".format(c=self.coordination),
+            f"Coordination geometry type : {self.name}{symbol}\n",
+            f"  - coordination number : {self.coordination}",
         ]
         if self.points is None:
             outs.append("... not yet implemented")
         else:
             outs.append("  - list of points :")
             for pp in self.points:
-                outs.append("    - {p}".format(p=pp))
+                outs.append(f"    - {pp}")
         outs.append("------------------------------------------------------------")
         outs.append("")
 
@@ -705,16 +680,16 @@ class CoordinationGeometry:
     def __repr__(self):
         symbol = ""
         if self.IUPAC_symbol is not None:
-            symbol += " (IUPAC: {s}".format(s=self.IUPAC_symbol)
+            symbol += f" (IUPAC: {self.IUPAC_symbol}"
             if self.IUCr_symbol is not None:
-                symbol += " || IUCr: {s})".format(s=self.IUCr_symbol)
+                symbol += f" || IUCr: {self.IUCr_symbol})"
             else:
                 symbol += ")"
         elif self.IUCr_symbol is not None:
-            symbol += " (IUCr: {s})".format(s=self.IUCr_symbol)
+            symbol += f" (IUCr: {self.IUCr_symbol})"
         outs = [
-            "Coordination geometry type : {n}{s}\n".format(n=self.name, s=symbol),
-            "  - coordination number : {c}".format(c=self.coordination),
+            f"Coordination geometry type : {self.name}{symbol}\n",
+            f"  - coordination number : {self.coordination}",
         ]
         outs.append("------------------------------------------------------------")
         outs.append("")
@@ -936,12 +911,12 @@ class CoordinationGeometry:
                 np.array([np.mean([_vertices[face_vertex][ii] for face_vertex in face]) for ii in range(3)])
             )
 
-        out = "{}\n".format(len(_vertices) + len(_face_centers))
+        out = f"{len(_vertices) + len(_face_centers)}\n"
         for vv in _vertices:
-            out += "{:15.8f} {:15.8f} {:15.8f}\n".format(vv[0], vv[1], vv[2])
+            out += f"{vv[0]:15.8f} {vv[1]:15.8f} {vv[2]:15.8f}\n"
         for fc in _face_centers:
-            out += "{:15.8f} {:15.8f} {:15.8f}\n".format(fc[0], fc[1], fc[2])
-        out += "{:d}\n".format(number_of_faces)
+            out += f"{fc[0]:15.8f} {fc[1]:15.8f} {fc[2]:15.8f}\n"
+        out += f"{number_of_faces:d}\n"
         for iface, face in enumerate(self._faces):
             if len(face) == 3:
                 out += "4\n"
@@ -950,14 +925,14 @@ class CoordinationGeometry:
             else:
                 for ii, f in enumerate(face):
                     out += "4\n"
-                    out += "{:d}\n".format(len(_vertices) + iface)
-                    out += "{:d}\n".format(f)
-                    out += "{:d}\n".format(face[np.mod(ii + 1, len(face))])
-                    out += "{:d}\n".format(len(_vertices) + iface)
+                    out += f"{len(_vertices) + iface:d}\n"
+                    out += f"{f:d}\n"
+                    out += f"{face[np.mod(ii + 1, len(face))]:d}\n"
+                    out += f"{len(_vertices) + iface:d}\n"
             if len(face) in [3, 4]:
                 for face_vertex in face:
-                    out += "{:d}\n".format(face_vertex)
-                out += "{:d}\n".format(face[0])
+                    out += f"{face_vertex:d}\n"
+                out += f"{face[0]:d}\n"
         pmeshes.append({"pmesh_string": out})
         return pmeshes
 
@@ -979,18 +954,18 @@ class AllCoordinationGeometries(dict):
         dict.__init__(self)
         self.cg_list = []
         if only_symbols is None:
-            with open("{}/coordination_geometries_files/allcg.txt".format(module_dir), "r") as f:
+            with open(f"{module_dir}/coordination_geometries_files/allcg.txt") as f:
                 data = f.readlines()
             for line in data:
-                cg_file = "{}/{}".format(module_dir, line.strip())
-                with open(cg_file, "r") as f:
+                cg_file = f"{module_dir}/{line.strip()}"
+                with open(cg_file) as f:
                     dd = json.load(f)
                 self.cg_list.append(CoordinationGeometry.from_dict(dd))
         else:
             for symbol in only_symbols:
                 fsymbol = symbol.replace(":", "#")
-                cg_file = "{}/coordination_geometries_files/{}.json".format(module_dir, fsymbol)
-                with open(cg_file, "r") as f:
+                cg_file = f"{module_dir}/coordination_geometries_files/{fsymbol}.json"
+                with open(cg_file) as f:
                     dd = json.load(f)
                 self.cg_list.append(CoordinationGeometry.from_dict(dd))
 
@@ -1022,7 +997,7 @@ class AllCoordinationGeometries(dict):
                     self.separations_cg[cn][sep].append(cg.mp_symbol)
                     self.minpoints[cn] = min(self.minpoints[cn], algo.minimum_number_of_points)
                     self.maxpoints[cn] = max(self.maxpoints[cn], algo.maximum_number_of_points)
-        self.maxpoints_inplane = {cn: max([sep[1] for sep in seps.keys()]) for cn, seps in self.separations_cg.items()}
+        self.maxpoints_inplane = {cn: max(sep[1] for sep in seps.keys()) for cn, seps in self.separations_cg.items()}
 
     def __getitem__(self, key):
         return self.get_geometry_from_mp_symbol(key)
@@ -1196,7 +1171,7 @@ class AllCoordinationGeometries(dict):
         for gg in self.cg_list:
             if gg.name == name or name in gg.alternative_names:
                 return gg
-        raise LookupError('No coordination geometry found with name "{name}"'.format(name=name))
+        raise LookupError(f'No coordination geometry found with name "{name}"')
 
     def get_geometry_from_IUPAC_symbol(self, IUPAC_symbol):
         """
@@ -1208,7 +1183,7 @@ class AllCoordinationGeometries(dict):
         for gg in self.cg_list:
             if gg.IUPAC_symbol == IUPAC_symbol:
                 return gg
-        raise LookupError('No coordination geometry found with IUPAC symbol "{symbol}"'.format(symbol=IUPAC_symbol))
+        raise LookupError(f'No coordination geometry found with IUPAC symbol "{IUPAC_symbol}"')
 
     def get_geometry_from_IUCr_symbol(self, IUCr_symbol):
         """
@@ -1220,7 +1195,7 @@ class AllCoordinationGeometries(dict):
         for gg in self.cg_list:
             if gg.IUCr_symbol == IUCr_symbol:
                 return gg
-        raise LookupError('No coordination geometry found with IUCr symbol "{symbol}"'.format(symbol=IUCr_symbol))
+        raise LookupError(f'No coordination geometry found with IUCr symbol "{IUCr_symbol}"')
 
     def get_geometry_from_mp_symbol(self, mp_symbol):
         """
@@ -1232,7 +1207,7 @@ class AllCoordinationGeometries(dict):
         for gg in self.cg_list:
             if gg.mp_symbol == mp_symbol:
                 return gg
-        raise LookupError('No coordination geometry found with mp_symbol "{symbol}"'.format(symbol=mp_symbol))
+        raise LookupError(f'No coordination geometry found with mp_symbol "{mp_symbol}"')
 
     def is_a_valid_coordination_geometry(self, mp_symbol=None, IUPAC_symbol=None, IUCr_symbol=None, name=None, cn=None):
         """
@@ -1306,9 +1281,9 @@ class AllCoordinationGeometries(dict):
         if type == "all_geometries_latex_images":
             mystring = ""
             for cn in range(1, maxcn + 1):
-                mystring += "\\section*{{Coordination {cn}}}\n\n".format(cn=cn)
+                mystring += f"\\section*{{Coordination {cn}}}\n\n"
                 for cg in self.get_implemented_geometries(coordination=cn, returned="cg"):
-                    mystring += "\\subsubsection*{{{mp} : {name}}}\n\n".format(mp=cg.mp_symbol, name=cg.get_name())
+                    mystring += f"\\subsubsection*{{{cg.mp_symbol} : {cg.get_name()}}}\n\n"
                     mystring += "IUPAC : {iupac}\n\nIUCr : {iucr}\n\n".format(
                         iupac=cg.IUPAC_symbol, iucr=cg.IUCr_symbol
                     )
@@ -1318,14 +1293,14 @@ class AllCoordinationGeometries(dict):
                     )
                     mystring += "\\end{center}\n\n"
                 for cg in self.get_not_implemented_geometries(cn, returned="cg"):
-                    mystring += "\\subsubsection*{{{mp} : {name}}}\n\n".format(mp=cg.mp_symbol, name=cg.get_name())
+                    mystring += f"\\subsubsection*{{{cg.mp_symbol} : {cg.get_name()}}}\n\n"
                     mystring += "IUPAC : {iupac}\n\nIUCr : {iucr}\n\n".format(
                         iupac=cg.IUPAC_symbol, iucr=cg.IUCr_symbol
                     )
         elif type == "all_geometries_latex":
             mystring = ""
             for cn in range(1, maxcn + 1):
-                mystring += "\\subsection*{{Coordination {cn}}}\n\n".format(cn=cn)
+                mystring += f"\\subsection*{{Coordination {cn}}}\n\n"
                 mystring += "\\begin{itemize}\n"
                 for cg in self.get_implemented_geometries(coordination=cn, returned="cg"):
                     mystring += "\\item {mp} $\\rightarrow$ {name} ".format(
@@ -1347,7 +1322,7 @@ class AllCoordinationGeometries(dict):
         else:
             mystring = "+-------------------------+\n| Coordination geometries |\n+-------------------------+\n\n"
             for cn in range(1, maxcn + 1):
-                mystring += "==>> CN = {cn} <<==\n".format(cn=cn)
+                mystring += f"==>> CN = {cn} <<==\n"
                 if type == "implemented_geometries":
                     for cg in self.get_implemented_geometries(coordination=cn):
                         if additional_info is not None:
@@ -1360,11 +1335,9 @@ class AllCoordinationGeometries(dict):
                                 addinfo = ""
                         else:
                             addinfo = ""
-                        mystring += " - {mp} : {name}{addinfo}\n".format(
-                            mp=cg.mp_symbol, name=cg.get_name(), addinfo=addinfo
-                        )
+                        mystring += f" - {cg.mp_symbol} : {cg.get_name()}{addinfo}\n"
                 elif type == "all_geometries":
                     for cg in self.get_geometries(coordination=cn):
-                        mystring += " - {mp} : {name}\n".format(mp=cg.mp_symbol, name=cg.get_name())
+                        mystring += f" - {cg.mp_symbol} : {cg.get_name()}\n"
                 mystring += "\n"
         return mystring
