@@ -1,8 +1,6 @@
-# coding: utf-8
 # Copyright (c) Pymatgen Development Team.
 # Distributed under the terms of the MIT License.
 
-
 """
 This module contains the main object used to identify the coordination environments in a given structure.
 If you use this module, please cite the following:
@@ -11,6 +9,10 @@ Michael Goebel, Stephan Schenk, Peter Degelmann, Rute Andre, Robert Glaum, and G
 "Statistical analysis of coordination environments in oxides",
 Chem. Mater., 2017, 29 (19), pp 8346–8360,
 DOI: 10.1021/acs.chemmater.7b02766
+D. Waroquiers, J. George, M. Horton, S. Schenk, K. A. Persson, G.-M. Rignanese, X. Gonze, G. Hautier
+"ChemEnv: a fast and robust coordination environment identification tool",
+Acta Cryst. B 2020, 76, pp 683–695,
+DOI: 10.1107/S2052520620007994
 """
 
 __author__ = "David Waroquiers"
@@ -24,7 +26,6 @@ __date__ = "Feb 20, 2016"
 import itertools
 import logging
 import time
-from collections import OrderedDict
 from random import shuffle
 
 import numpy as np
@@ -151,9 +152,9 @@ class AbstractGeometry:
         String representation of the AbstractGeometry
         :return: String representation of the AbstractGeometry
         """
-        outs = ["\nAbstract Geometry with {n} points :".format(n=len(self.coords))]
+        outs = [f"\nAbstract Geometry with {len(self.coords)} points :"]
         for pp in self.coords:
-            outs.append("  {pp}".format(pp=pp))
+            outs.append(f"  {pp}")
         if self.centering_type == "standard":
             if self.include_central_site_in_centroid:
                 outs.append(
@@ -168,7 +169,7 @@ class AbstractGeometry:
                     " numbers >= 5 : {c}\n".format(c=self.centre)
                 )
         elif self.centering_type == "central_site":
-            outs.append("Points are referenced to the central site : {c}\n".format(c=self.centre))
+            outs.append(f"Points are referenced to the central site : {self.centre}\n")
         elif self.centering_type == "centroid":
             if self.include_central_site_in_centroid:
                 outs.append(
@@ -379,12 +380,18 @@ class LocalGeometryFinder:
         debug_level=None,
         plane_safe_permutations=False,
         only_symbols=None,
+        print_citation=False,
     ):
         """
-        Constructor for the LocalGeometryFinder, initializes the list of coordination geometries
-        :param permutations_safe_override: If set to True, all permutations are tested (very time-consuming for large
+
+        Args:
+            permutations_safe_override:  If set to True, all permutations are tested (very time-consuming for large
             coordination numbers!)
-        :param plane_ordering_override: If set to False, the ordering of the points in the plane is disabled
+            plane_ordering_override: If set to False, the ordering of the points in the plane is disabled
+            debug_level: decides the level of debugging
+            permutations_safe_override: Whether to use safe permutations.
+            only_symbols: Whether to restrict the list of environments to be identified.
+            print_citation: If True, the ChemEnv citation will be printed
         """
         self.allcg = AllCoordinationGeometries(
             permutations_safe_override=permutations_safe_override,
@@ -399,7 +406,8 @@ class LocalGeometryFinder:
             bva_distance_scale_factor=None,
             structure_refinement=self.STRUCTURE_REFINEMENT_NONE,
         )
-        print(chemenv_citations())
+        if print_citation:
+            print(chemenv_citations())
 
     def setup_parameters(
         self,
@@ -490,7 +498,7 @@ class LocalGeometryFinder:
         :param lattice: The lattice of the structure
         :param species: The species on the sites
         :param coords: The coordinates of the sites
-        :param coords_are_cartesian: If set to True, the coordinates are given in cartesian coordinates
+        :param coords_are_cartesian: If set to True, the coordinates are given in Cartesian coordinates
         """
         self.setup_structure(Structure(lattice, species, coords, coords_are_cartesian))
 
@@ -724,9 +732,7 @@ class LocalGeometryFinder:
         # Loop on all the sites
         for isite, site in enumerate(self.structure):
             if isite not in sites_indices:
-                logging.debug(
-                    " ... in site #{:d}/{:d} ({}) : " "skipped".format(isite, len(self.structure), site.species_string)
-                )
+                logging.debug(f" ... in site #{isite:d}/{len(self.structure):d} ({site.species_string}) : skipped")
                 continue
             if breakit:
                 logging.debug(
@@ -734,10 +740,10 @@ class LocalGeometryFinder:
                     "skipped (timelimit)".format(isite, len(self.structure), site.species_string)
                 )
                 continue
-            logging.debug(" ... in site #{:d}/{:d} ({})".format(isite, len(self.structure), site.species_string))
+            logging.debug(f" ... in site #{isite:d}/{len(self.structure):d} ({site.species_string})")
             t1 = time.process_time()
             if optimization > 0:
-                self.detailed_voronoi.local_planes[isite] = OrderedDict()
+                self.detailed_voronoi.local_planes[isite] = {}
                 self.detailed_voronoi.separations[isite] = {}
             se.init_neighbors_sets(
                 isite=isite,
@@ -752,7 +758,7 @@ class LocalGeometryFinder:
                 if cn not in all_cns:
                     continue
                 for inb_set, nb_set in enumerate(nb_sets):
-                    logging.debug("    ... getting environments for nb_set ({:d}, {:d})".format(cn, inb_set))
+                    logging.debug(f"    ... getting environments for nb_set ({cn:d}, {inb_set:d})")
                     tnbset1 = time.process_time()
                     ce = self.update_nb_set_environments(
                         se=se,
@@ -773,7 +779,7 @@ class LocalGeometryFinder:
                             # Get possibly missing neighbors sets
                             if cg.neighbors_sets_hints is None:
                                 continue
-                            logging.debug('       ... getting hints from cg with mp_symbol "{}" ...'.format(cg_symbol))
+                            logging.debug(f'       ... getting hints from cg with mp_symbol "{cg_symbol}" ...')
                             hints_info = {
                                 "csm": cg_dict["symmetry_measure"],
                                 "nb_set": nb_set,
@@ -782,7 +788,7 @@ class LocalGeometryFinder:
                             for nb_sets_hints in cg.neighbors_sets_hints:
                                 suggested_nb_set_voronoi_indices = nb_sets_hints.hints(hints_info)
                                 for inew, new_nb_set_voronoi_indices in enumerate(suggested_nb_set_voronoi_indices):
-                                    logging.debug("           hint # {:d}".format(inew))
+                                    logging.debug(f"           hint # {inew:d}")
                                     new_nb_set = se.NeighborsSet(
                                         structure=se.structure,
                                         isite=isite,
@@ -851,9 +857,9 @@ class LocalGeometryFinder:
                 if time_left < 2.0 * max_time_one_site:
                     breakit = True
             max_time_one_site = max(max_time_one_site, t2 - t1)
-            logging.debug("    ... computed in {:.2f} seconds".format(t2 - t1))
+            logging.debug(f"    ... computed in {t2 - t1:.2f} seconds")
         time_end = time.process_time()
-        logging.debug("    ... compute_structure_environments ended in {:.2f} seconds".format(time_end - time_init))
+        logging.debug(f"    ... compute_structure_environments ended in {time_end - time_init:.2f} seconds")
         return se
 
     def update_nb_set_environments(self, se, isite, cn, inb_set, nb_set, recompute=False, optimization=None):
@@ -878,7 +884,7 @@ class LocalGeometryFinder:
         self.setup_local_geometry(isite, coords=neighb_coords, optimization=optimization)
         if optimization > 0:
             logging.debug("Getting StructureEnvironments with optimized algorithm")
-            nb_set.local_planes = OrderedDict()
+            nb_set.local_planes = {}
             nb_set.separations = {}
             cncgsm = self.get_coordination_symmetry_measures_optim(nb_set=nb_set, optimization=optimization)
         else:
@@ -1167,10 +1173,10 @@ class LocalGeometryFinder:
                     "wcs_ctwcc",
                     "wcs_csc",
                 ]:
-                    result_dict["S:1"]["csm_{}".format(csmtype)] = 0.0
-                    result_dict["S:1"]["scaling_factor_{}".format(csmtype)] = None
-                    result_dict["S:1"]["rotation_matrix_{}".format(csmtype)] = None
-                    result_dict["S:1"]["translation_vector_{}".format(csmtype)] = None
+                    result_dict["S:1"][f"csm_{csmtype}"] = 0.0
+                    result_dict["S:1"][f"scaling_factor_{csmtype}"] = None
+                    result_dict["S:1"][f"rotation_matrix_{csmtype}"] = None
+                    result_dict["S:1"][f"translation_vector_{csmtype}"] = None
             return result_dict
         result_dict = {}
         for geometry in test_geometries:
@@ -1703,7 +1709,7 @@ class LocalGeometryFinder:
                         continue
                     if sep not in nb_set.separations:
                         nb_set.separations[sep] = {}
-                    mysep = [np.array(ss, dtype=np.int8) for ss in separation]
+                    mysep = [np.array(ss, dtype=int) for ss in separation]
                     nb_set.separations[sep][separation] = (plane, mysep)
                     if sep == separation_plane_algo.separation:
                         new_seps.append(mysep)
