@@ -1,4 +1,3 @@
-# coding: utf-8
 # Copyright (c) Pymatgen Development Team.
 # Distributed under the terms of the MIT License.
 # Credit to Dr. Shyue Ping Ong for the template of the calculator
@@ -7,12 +6,14 @@
 This module implements a TEM pattern calculator.
 """
 
+from __future__ import annotations
+
 import json
 import os
 from collections import namedtuple
 from fractions import Fraction
 from functools import lru_cache
-from typing import Dict, List, Tuple, cast, Union
+from typing import cast
 
 import numpy as np
 import pandas as pd
@@ -48,9 +49,9 @@ class TEMCalculator(AbstractDiffractionPatternCalculator):
         self,
         symprec: float = None,
         voltage: float = 200,
-        beam_direction: Tuple[int, int, int] = (0, 0, 1),
+        beam_direction: tuple[int, int, int] = (0, 0, 1),
         camera_length: int = 160,
-        debye_waller_factors: Dict[str, float] = None,
+        debye_waller_factors: dict[str, float] = None,
         cs: float = 1,
     ) -> None:
         """
@@ -90,9 +91,9 @@ class TEMCalculator(AbstractDiffractionPatternCalculator):
         wavelength_rel = (
             sc.h
             / np.sqrt(
-                2 * sc.m_e * sc.e * 1000 * self.voltage * (1 + (sc.e * 1000 * self.voltage) / (2 * sc.m_e * sc.c ** 2))
+                2 * sc.m_e * sc.e * 1000 * self.voltage * (1 + (sc.e * 1000 * self.voltage) / (2 * sc.m_e * sc.c**2))
             )
-            * (10 ** 10)
+            * (10**10)
         )
         return wavelength_rel
 
@@ -108,14 +109,14 @@ class TEMCalculator(AbstractDiffractionPatternCalculator):
         """
         points = [0, 0, 0]
         coord_values = np.arange(coord_left, coord_right + 1)
-        points[0], points[1], points[2] = np.meshgrid(coord_values, coord_values, coord_values)
+        points[0], points[1], points[2] = np.meshgrid(coord_values, coord_values, coord_values)  # type: ignore
         points_matrix = (np.ravel(points[i]) for i in range(0, 3))
         result = np.vstack(list(points_matrix)).transpose()
         return result
 
     def zone_axis_filter(
-        self, points: Union[List[Tuple[int, int, int]], np.ndarray], laue_zone: int = 0
-    ) -> Union[List[Tuple[int, int, int]]]:
+        self, points: list[tuple[int, int, int]] | np.ndarray, laue_zone: int = 0
+    ) -> list[tuple[int, int, int]]:
         """
         Filters out all points that exist within the specified Laue zone according to the zone axis rule.
         Args:
@@ -129,13 +130,13 @@ class TEMCalculator(AbstractDiffractionPatternCalculator):
         if len(points) == 0:
             return []
         filtered = np.where(np.dot(np.array(self.beam_direction), np.transpose(points)) == laue_zone)
-        result = points[filtered]
-        result_tuples = cast(List[Tuple[int, int, int]], [tuple(x) for x in result.tolist()])
+        result = points[filtered]  # type: ignore
+        result_tuples = cast(list[tuple[int, int, int]], [tuple(x) for x in result.tolist()])
         return result_tuples
 
     def get_interplanar_spacings(
-        self, structure: Structure, points: Union[List[Tuple[int, int, int]], np.ndarray]
-    ) -> Dict[Tuple[int, int, int], float]:
+        self, structure: Structure, points: list[tuple[int, int, int]] | np.ndarray
+    ) -> dict[tuple[int, int, int], float]:
         """
         Args:
             structure (Structure): the input structure.
@@ -151,8 +152,8 @@ class TEMCalculator(AbstractDiffractionPatternCalculator):
         return interplanar_spacings
 
     def bragg_angles(
-        self, interplanar_spacings: Dict[Tuple[int, int, int], float]
-    ) -> Dict[Tuple[int, int, int], float]:
+        self, interplanar_spacings: dict[tuple[int, int, int], float]
+    ) -> dict[tuple[int, int, int], float]:
         """
         Gets the Bragg angles for every hkl point passed in (where n = 1).
         Args:
@@ -166,7 +167,7 @@ class TEMCalculator(AbstractDiffractionPatternCalculator):
         bragg_angles = dict(zip(plane, bragg_angles_val))
         return bragg_angles
 
-    def get_s2(self, bragg_angles: Dict[Tuple[int, int, int], float]) -> Dict[Tuple[int, int, int], float]:
+    def get_s2(self, bragg_angles: dict[tuple[int, int, int], float]) -> dict[tuple[int, int, int], float]:
         """
         Calculates the s squared parameter (= square of sin theta over lambda) for each hkl plane.
         Args:
@@ -182,8 +183,8 @@ class TEMCalculator(AbstractDiffractionPatternCalculator):
         return s2
 
     def x_ray_factors(
-        self, structure: Structure, bragg_angles: Dict[Tuple[int, int, int], float]
-    ) -> Dict[str, Dict[Tuple[int, int, int], float]]:
+        self, structure: Structure, bragg_angles: dict[tuple[int, int, int], float]
+    ) -> dict[str, dict[tuple[int, int, int], float]]:
         """
         Calculates x-ray factors, which are required to calculate atomic scattering factors. Method partially inspired
         by the equivalent process in the xrd module.
@@ -201,7 +202,7 @@ class TEMCalculator(AbstractDiffractionPatternCalculator):
             coeffs = np.array(ATOMIC_SCATTERING_PARAMS[atom.symbol])
             for plane in bragg_angles:
                 scattering_factor_curr = atom.Z - 41.78214 * s2[plane] * np.sum(
-                    coeffs[:, 0] * np.exp(-coeffs[:, 1] * s2[plane]), axis=None
+                    coeffs[:, 0] * np.exp(-coeffs[:, 1] * s2[plane]), axis=None  # type: ignore
                 )
                 scattering_factors_for_atom[plane] = scattering_factor_curr
             x_ray_factors[atom.symbol] = scattering_factors_for_atom
@@ -209,8 +210,8 @@ class TEMCalculator(AbstractDiffractionPatternCalculator):
         return x_ray_factors
 
     def electron_scattering_factors(
-        self, structure: Structure, bragg_angles: Dict[Tuple[int, int, int], float]
-    ) -> Dict[str, Dict[Tuple[int, int, int], float]]:
+        self, structure: Structure, bragg_angles: dict[tuple[int, int, int], float]
+    ) -> dict[str, dict[tuple[int, int, int], float]]:
         """
         Calculates atomic scattering factors for electrons using the Mott-Bethe formula (1st order Born approximation).
         Args:
@@ -234,8 +235,8 @@ class TEMCalculator(AbstractDiffractionPatternCalculator):
         return electron_scattering_factors
 
     def cell_scattering_factors(
-        self, structure: Structure, bragg_angles: Dict[Tuple[int, int, int], float]
-    ) -> Dict[Tuple[int, int, int], int]:
+        self, structure: Structure, bragg_angles: dict[tuple[int, int, int], float]
+    ) -> dict[tuple[int, int, int], int]:
         """
         Calculates the scattering factor for the whole cell.
         Args:
@@ -259,8 +260,8 @@ class TEMCalculator(AbstractDiffractionPatternCalculator):
         return cell_scattering_factors
 
     def cell_intensity(
-        self, structure: Structure, bragg_angles: Dict[Tuple[int, int, int], float]
-    ) -> Dict[Tuple[int, int, int], float]:
+        self, structure: Structure, bragg_angles: dict[tuple[int, int, int], float]
+    ) -> dict[tuple[int, int, int], float]:
         """
         Calculates cell intensity for each hkl plane. For simplicity's sake, take I = |F|**2.
         Args:
@@ -280,7 +281,7 @@ class TEMCalculator(AbstractDiffractionPatternCalculator):
         self,
         structure: Structure,
         scaled: bool = None,
-        two_theta_range: Tuple[float, float] = None,
+        two_theta_range: tuple[float, float] = None,
     ) -> pd.DataFrame:
         """
         Returns all relevant TEM DP info in a pandas dataframe.
@@ -317,8 +318,8 @@ class TEMCalculator(AbstractDiffractionPatternCalculator):
         return df
 
     def normalized_cell_intensity(
-        self, structure: Structure, bragg_angles: Dict[Tuple[int, int, int], float]
-    ) -> Dict[Tuple[int, int, int], float]:
+        self, structure: Structure, bragg_angles: dict[tuple[int, int, int], float]
+    ) -> dict[tuple[int, int, int], float]:
         """
         Normalizes the cell_intensity dict to 1, for use in plotting.
         Args:
@@ -338,8 +339,8 @@ class TEMCalculator(AbstractDiffractionPatternCalculator):
     def is_parallel(
         self,
         structure: Structure,
-        plane: Tuple[int, int, int],
-        other_plane: Tuple[int, int, int],
+        plane: tuple[int, int, int],
+        other_plane: tuple[int, int, int],
     ) -> bool:
         """
         Checks if two hkl planes are parallel in reciprocal space.
@@ -353,7 +354,7 @@ class TEMCalculator(AbstractDiffractionPatternCalculator):
         phi = self.get_interplanar_angle(structure, plane, other_plane)
         return phi in (180, 0) or np.isnan(phi)
 
-    def get_first_point(self, structure: Structure, points: list) -> Dict[Tuple[int, int, int], float]:
+    def get_first_point(self, structure: Structure, points: list) -> dict[tuple[int, int, int], float]:
         """
         Gets the first point to be plotted in the 2D DP, corresponding to maximum d/minimum R.
         Args:
@@ -373,7 +374,7 @@ class TEMCalculator(AbstractDiffractionPatternCalculator):
         return {max_d_plane: max_d}
 
     @staticmethod
-    def get_interplanar_angle(structure: Structure, p1: Tuple[int, int, int], p2: Tuple[int, int, int]) -> float:
+    def get_interplanar_angle(structure: Structure, p1: tuple[int, int, int], p2: tuple[int, int, int]) -> float:
         """
         Returns the interplanar angle (in degrees) between the normal of two crystal planes.
         Formulas from International Tables for Crystallography Volume C pp. 2-9.
@@ -398,25 +399,25 @@ class TEMCalculator(AbstractDiffractionPatternCalculator):
         cos_beta_star = (np.cos(alpha) * np.cos(gamma) - np.cos(beta)) / (np.sin(alpha) * np.sin(gamma))
         cos_gamma_star = (np.cos(alpha) * np.cos(beta) - np.cos(gamma)) / (np.sin(alpha) * np.sin(beta))
         r1_norm = np.sqrt(
-            p1[0] ** 2 * a_star ** 2
-            + p1[1] ** 2 * b_star ** 2
-            + p1[2] ** 2 * c_star ** 2
+            p1[0] ** 2 * a_star**2
+            + p1[1] ** 2 * b_star**2
+            + p1[2] ** 2 * c_star**2
             + 2 * p1[0] * p1[1] * a_star * b_star * cos_gamma_star
             + 2 * p1[0] * p1[2] * a_star * c_star * cos_beta_star
             + 2 * p1[1] * p1[2] * b_star * c_star * cos_gamma_star
         )
         r2_norm = np.sqrt(
-            p2[0] ** 2 * a_star ** 2
-            + p2[1] ** 2 * b_star ** 2
-            + p2[2] ** 2 * c_star ** 2
+            p2[0] ** 2 * a_star**2
+            + p2[1] ** 2 * b_star**2
+            + p2[2] ** 2 * c_star**2
             + 2 * p2[0] * p2[1] * a_star * b_star * cos_gamma_star
             + 2 * p2[0] * p2[2] * a_star * c_star * cos_beta_star
             + 2 * p2[1] * p2[2] * b_star * c_star * cos_gamma_star
         )
         r1_dot_r2 = (
-            p1[0] * p2[0] * a_star ** 2
-            + p1[1] * p2[1] * b_star ** 2
-            + p1[2] * p2[2] * c_star ** 2
+            p1[0] * p2[0] * a_star**2
+            + p1[1] * p2[1] * b_star**2
+            + p1[2] * p2[2] * c_star**2
             + (p1[0] * p2[1] + p2[0] * p1[1]) * a_star * b_star * cos_gamma_star
             + (p1[0] * p2[2] + p2[0] * p1[1]) * a_star * c_star * cos_beta_star
             + (p1[1] * p2[2] + p2[1] * p1[2]) * b_star * c_star * cos_alpha_star
@@ -426,9 +427,9 @@ class TEMCalculator(AbstractDiffractionPatternCalculator):
 
     @staticmethod
     def get_plot_coeffs(
-        p1: Tuple[int, int, int],
-        p2: Tuple[int, int, int],
-        p3: Tuple[int, int, int],
+        p1: tuple[int, int, int],
+        p2: tuple[int, int, int],
+        p3: tuple[int, int, int],
     ) -> np.ndarray:
         """
         Calculates coefficients of the vector addition required to generate positions for each DP point
@@ -446,7 +447,7 @@ class TEMCalculator(AbstractDiffractionPatternCalculator):
         x = np.dot(a_pinv, b)
         return np.ravel(x)
 
-    def get_positions(self, structure: Structure, points: list) -> Dict[Tuple[int, int, int], np.ndarray]:
+    def get_positions(self, structure: Structure, points: list) -> dict[tuple[int, int, int], np.ndarray]:
         """
         Calculates all the positions of each hkl point in the 2D diffraction pattern by vector addition.
         Distance in centimeters.
@@ -497,7 +498,7 @@ class TEMCalculator(AbstractDiffractionPatternCalculator):
 
         return positions
 
-    def tem_dots(self, structure: Structure, points) -> List:
+    def tem_dots(self, structure: Structure, points) -> list:
         """
         Generates all TEM_dot as named tuples that will appear on the 2D diffraction pattern.
         Args:
@@ -514,7 +515,7 @@ class TEMCalculator(AbstractDiffractionPatternCalculator):
         for hkl, intensity in cell_intensity.items():
             dot = namedtuple("dot", ["position", "hkl", "intensity", "film_radius", "d_spacing"])
             position = positions[hkl]
-            film_radius = 0.91 * (10 ** -3 * self.cs * self.wavelength_rel() ** 3) ** Fraction("1/4")
+            film_radius = 0.91 * (10**-3 * self.cs * self.wavelength_rel() ** 3) ** Fraction("1/4")
             d_spacing = interplanar_spacings[hkl]
             tem_dot = dot(position, hkl, intensity, film_radius, d_spacing)
             dots.append(tem_dot)
