@@ -1,32 +1,25 @@
-# coding: utf-8
-
-from __future__ import division, unicode_literals
+# Copyright (c) Pymatgen Development Team.
+# Distributed under the terms of the MIT License.
 
 """
 This module implements an XRD pattern calculator.
 """
 
-from six.moves import filter
-from six.moves import map
-from six.moves import zip
-
-__author__ = "Shyue Ping Ong"
-__copyright__ = "Copyright 2012, The Materials Project"
-__version__ = "0.1"
-__maintainer__ = "Shyue Ping Ong"
-__email__ = "ongsp@ucsd.edu"
-__date__ = "5/22/14"
-
-
-from math import sin, cos, asin, pi, degrees, radians
+import json
 import os
+from math import asin, cos, degrees, pi, radians, sin
 
 import numpy as np
-import json
 
 from pymatgen.symmetry.analyzer import SpacegroupAnalyzer
 
-#XRD wavelengths in angstroms
+from .core import (
+    AbstractDiffractionPatternCalculator,
+    DiffractionPattern,
+    get_unique_families,
+)
+
+# XRD wavelengths in angstroms
 WAVELENGTHS = {
     "CuKa": 1.54184,
     "CuKa2": 1.54439,
@@ -54,13 +47,12 @@ WAVELENGTHS = {
     "AgKb1": 0.497082,
 }
 
-with open(os.path.join(os.path.dirname(__file__),
-                       "atomic_scattering_params.json")) as f:
+with open(os.path.join(os.path.dirname(__file__), "atomic_scattering_params.json")) as f:
     ATOMIC_SCATTERING_PARAMS = json.load(f)
 
 
-class XRDCalculator(object):
-    """
+class XRDCalculator(AbstractDiffractionPatternCalculator):
+    r"""
     Computes the XRD pattern of a crystal structure.
 
     This code is implemented by Shyue Ping Ong as part of UCSD's NANO106 -
@@ -71,32 +63,32 @@ class XRDCalculator(object):
     the Debye-Waller (temperature) factor (for which data is typically not
     available). Note that the multiplicity correction is not needed since
     this code simply goes through all reciprocal points within the limiting
-    sphere, which includes all symmetrically equivalent planes. The algorithm
+    sphere, which includes all symmetrically equivalent facets. The algorithm
     is as follows
 
     1. Calculate reciprocal lattice of structure. Find all reciprocal points
-       within the limiting sphere given by :math:`\\frac{2}{\\lambda}`.
+       within the limiting sphere given by :math:` \frac{2}{ \lambda}`.
 
-    2. For each reciprocal point :math:`\\mathbf{g_{hkl}}` corresponding to
+    2. For each reciprocal point :math:` \mathbf{g_{hkl}}` corresponding to
        lattice plane :math:`(hkl)`, compute the Bragg condition
-       :math:`\\sin(\\theta) = \\frac{\\lambda}{2d_{hkl}}`
+       :math:` \sin( \theta) =  \frac{ \lambda}{2d_{hkl}}`
 
     3. Compute the structure factor as the sum of the atomic scattering
        factors. The atomic scattering factors are given by
 
        .. math::
 
-           f(s) = Z - 41.78214 \\times s^2 \\times \\sum\\limits_{i=1}^n a_i \
-           \exp(-b_is^2)
+           f(s) = Z - 41.78214 \times s^2 \times \sum \limits_{i=1}^n a_i \
+            \exp(-b_is^2)
 
-       where :math:`s = \\frac{\\sin(\\theta)}{\\lambda}` and :math:`a_i`
+       where :math:`s = \ frac{\ sin(\ theta)}{\ lambda}` and :math:`a_i`
        and :math:`b_i` are the fitted parameters for each element. The
        structure factor is then given by
 
        .. math::
 
-           F_{hkl} = \\sum\\limits_{j=1}^N f_j \\exp(2\\pi i \\mathbf{g_{hkl}}
-           \cdot \\mathbf{r})
+           F_{hkl} =  \sum \limits_{j=1}^N f_j  \exp(2 \pi i  \mathbf{g_{hkl}}
+            \cdot  \mathbf{r})
 
     4. The intensity is then given by the modulus square of the structure
        factor.
@@ -110,23 +102,13 @@ class XRDCalculator(object):
 
        .. math::
 
-           P(\\theta) = \\frac{1 + \\cos^2(2\\theta)}
-           {\\sin^2(\\theta)\\cos(\\theta)}
+           P( \theta) =  \frac{1 +  \cos^2(2 \theta)}
+           { \sin^2( \theta) \cos( \theta)}
     """
 
-    #Tuple of available radiation keywords.
+    # Tuple of available radiation keywords.
     AVAILABLE_RADIATION = tuple(WAVELENGTHS.keys())
 
-    #Tolerance in which to treat two peaks as having the same two theta.
-    TWO_THETA_TOL = 1e-5
-
-    # Tolerance in which to treat a peak as effectively 0 if the scaled
-    # intensity is less than this number. Since the max intensity is 100,
-    # this means the peak must be less than 1e-5 of the peak intensity to be
-    # considered as zero. This deals with numerical issues where systematic
-    # absences do not cancel exactly to zero.
-    SCALED_INTENSITY_TOL = 1e-3
-
     def __init__(self, wavelength="CuKa", symprec=0, debye_waller_factors=None):
         """
         Initializes the XRD calculator with a given radiation.
@@ -145,17 +127,19 @@ class XRDCalculator(object):
                 specification of Debye-Waller factors. Note that these
                 factors are temperature dependent.
         """
-        if isinstance(wavelength, float):
+        if isinstance(wavelength, (float, int)):
             self.wavelength = wavelength
-        else:
+        elif isinstance(wavelength, str):
             self.radiation = wavelength
             self.wavelength = WAVELENGTHS[wavelength]
+        else:
+            raise TypeError("'wavelength' must be either of: float, int or str")
         self.symprec = symprec
         self.debye_waller_factors = debye_waller_factors or {}
 
-    def get_xrd_data(self, structure, scaled=True, two_theta_range=(0, 90)):
+    def get_pattern(self, structure, scaled=True, two_theta_range=(0, 90)):
         """
-        Calculates the XRD data for a structure.
+        Calculates the diffraction pattern for a structure.
 
         Args:
             structure (Structure): Input structure
@@ -168,13 +152,7 @@ class XRDCalculator(object):
                 sphere of radius 2 / wavelength.
 
         Returns:
-            (XRD pattern) in the form of
-            [[two_theta, intensity, {(h, k, l): mult}, d_hkl], ...]
-            Two_theta is in degrees. Intensity is in arbitrary units and if
-            scaled (the default), has a maximum value of 100 for the highest
-            peak. {(h, k, l): mult} is a dict of Miller indices for all
-            diffracted lattice planes contributing to that intensity and
-            their multiplicities. d_hkl is the interplanar spacing.
+            (XRDPattern)
         """
         if self.symprec:
             finder = SpacegroupAnalyzer(structure, symprec=self.symprec)
@@ -186,15 +164,17 @@ class XRDCalculator(object):
 
         # Obtained from Bragg condition. Note that reciprocal lattice
         # vector length is 1 / d_hkl.
-        min_r, max_r = (0, 2 / wavelength) if two_theta_range is None else \
-            [2 * sin(radians(t / 2)) / wavelength for t in two_theta_range]
+        min_r, max_r = (
+            (0, 2 / wavelength)
+            if two_theta_range is None
+            else [2 * sin(radians(t / 2)) / wavelength for t in two_theta_range]
+        )
 
         # Obtain crystallographic reciprocal lattice points within range
         recip_latt = latt.reciprocal_lattice_crystallographic
-        recip_pts = recip_latt.get_points_in_sphere(
-            [[0, 0, 0]], [0, 0, 0], max_r)
+        recip_pts = recip_latt.get_points_in_sphere([[0, 0, 0]], [0, 0, 0], max_r)
         if min_r:
-            recip_pts = filter(lambda d: d[1] >= min_r, recip_pts)
+            recip_pts = [pt for pt in recip_pts if pt[1] >= min_r]
 
         # Create a flattened array of zs, coeffs, fcoords and occus. This is
         # used to perform vectorized computation of atomic scattering factors
@@ -208,14 +188,14 @@ class XRDCalculator(object):
         dwfactors = []
 
         for site in structure:
-            for sp, occu in site.species_and_occu.items():
+            for sp, occu in site.species.items():
                 zs.append(sp.Z)
                 try:
                     c = ATOMIC_SCATTERING_PARAMS[sp.symbol]
                 except KeyError:
-                    raise ValueError("Unable to calculate XRD pattern as "
-                                     "there is no scattering coefficients for"
-                                     " %s." % sp.symbol)
+                    raise ValueError(
+                        f"Unable to calculate XRD pattern as there is no scattering coefficients for {sp.symbol}."
+                    )
                 coeffs.append(c)
                 dwfactors.append(self.debye_waller_factors.get(sp.symbol, 0))
                 fcoords.append(site.frac_coords)
@@ -229,8 +209,9 @@ class XRDCalculator(object):
         peaks = {}
         two_thetas = []
 
-        for hkl, g_hkl, ind in sorted(
-                recip_pts, key=lambda i: (i[1], -i[0][0], -i[0][1], -i[0][2])):
+        for hkl, g_hkl, ind, _ in sorted(recip_pts, key=lambda i: (i[1], -i[0][0], -i[0][1], -i[0][2])):
+            # Force miller indices to be integers.
+            hkl = [int(round(i)) for i in hkl]
             if g_hkl != 0:
 
                 d_hkl = 1 / g_hkl
@@ -242,8 +223,8 @@ class XRDCalculator(object):
                 # 1/|ghkl|)
                 s = g_hkl / 2
 
-                #Store s^2 since we are using it a few times.
-                s2 = s ** 2
+                # Store s^2 since we are using it a few times.
+                s2 = s**2
 
                 # Vectorized computation of g.r for all fractional coords and
                 # hkl.
@@ -257,20 +238,17 @@ class XRDCalculator(object):
                 #      coeff = ATOMIC_SCATTERING_PARAMS[el.symbol]
                 #      fs = el.Z - 41.78214 * s2 * sum(
                 #          [d[0] * exp(-d[1] * s2) for d in coeff])
-                fs = zs - 41.78214 * s2 * np.sum(
-                    coeffs[:, :, 0] * np.exp(-coeffs[:, :, 1] * s2), axis=1)
+                fs = zs - 41.78214 * s2 * np.sum(coeffs[:, :, 0] * np.exp(-coeffs[:, :, 1] * s2), axis=1)
 
                 dw_correction = np.exp(-dwfactors * s2)
 
                 # Structure factor = sum of atomic scattering factors (with
                 # position factor exp(2j * pi * g.r and occupancies).
                 # Vectorized computation.
-                f_hkl = np.sum(fs * occus * np.exp(2j * pi * g_dot_r)
-                               * dw_correction)
+                f_hkl = np.sum(fs * occus * np.exp(2j * pi * g_dot_r) * dw_correction)
 
-                #Lorentz polarization correction for hkl
-                lorentz_factor = (1 + cos(2 * theta) ** 2) / \
-                    (sin(theta) ** 2 * cos(theta))
+                # Lorentz polarization correction for hkl
+                lorentz_factor = (1 + cos(2 * theta) ** 2) / (sin(theta) ** 2 * cos(theta))
 
                 # Intensity for hkl is modulus square of structure factor.
                 i_hkl = (f_hkl * f_hkl.conjugate()).real
@@ -278,108 +256,34 @@ class XRDCalculator(object):
                 two_theta = degrees(2 * theta)
 
                 if is_hex:
-                    #Use Miller-Bravais indices for hexagonal lattices.
-                    hkl = (hkl[0], hkl[1], - hkl[0] - hkl[1], hkl[2])
-                #Deal with floating point precision issues.
-                ind = np.where(np.abs(np.subtract(two_thetas, two_theta)) <
-                               XRDCalculator.TWO_THETA_TOL)
+                    # Use Miller-Bravais indices for hexagonal lattices.
+                    hkl = (hkl[0], hkl[1], -hkl[0] - hkl[1], hkl[2])
+                # Deal with floating point precision issues.
+                ind = np.where(
+                    np.abs(np.subtract(two_thetas, two_theta)) < AbstractDiffractionPatternCalculator.TWO_THETA_TOL
+                )
                 if len(ind[0]) > 0:
-                    peaks[two_thetas[ind[0]]][0] += i_hkl * lorentz_factor
-                    peaks[two_thetas[ind[0]]][1].append(tuple(hkl))
+                    peaks[two_thetas[ind[0][0]]][0] += i_hkl * lorentz_factor
+                    peaks[two_thetas[ind[0][0]]][1].append(tuple(hkl))
                 else:
-                    peaks[two_theta] = [i_hkl * lorentz_factor, [tuple(hkl)],
-                                        d_hkl]
+                    peaks[two_theta] = [i_hkl * lorentz_factor, [tuple(hkl)], d_hkl]
                     two_thetas.append(two_theta)
 
         # Scale intensities so that the max intensity is 100.
-        max_intensity = max([v[0] for v in peaks.values()])
-        data = []
+        max_intensity = max(v[0] for v in peaks.values())
+        x = []
+        y = []
+        hkls = []
+        d_hkls = []
         for k in sorted(peaks.keys()):
             v = peaks[k]
-            scaled_intensity = v[0] / max_intensity * 100 if scaled else v[0]
             fam = get_unique_families(v[1])
-            if scaled_intensity > XRDCalculator.SCALED_INTENSITY_TOL:
-                data.append([k, scaled_intensity, fam, v[2]])
-        return data
-
-    def get_xrd_plot(self, structure, two_theta_range=(0, 90),
-                     annotate_peaks=True):
-        """
-        Returns the XRD plot as a matplotlib.pyplot.
-
-        Args:
-            structure: Input structure
-            two_theta_range ([float of length 2]): Tuple for range of
-                two_thetas to calculate in degrees. Defaults to (0, 90). Set to
-                None if you want all diffracted beams within the limiting
-                sphere of radius 2 / wavelength.
-            annotate_peaks: Whether to annotate the peaks with plane
-                information.
-
-        Returns:
-            (matplotlib.pyplot)
-        """
-        from pymatgen.util.plotting_utils import get_publication_quality_plot
-        plt = get_publication_quality_plot(16, 10)
-        for two_theta, i, hkls, d_hkl in self.get_xrd_data(
-                structure, two_theta_range=two_theta_range):
-            if two_theta_range[0] <= two_theta <= two_theta_range[1]:
-                label = ", ".join([str(hkl) for hkl in hkls.keys()])
-                plt.plot([two_theta, two_theta], [0, i], color='k',
-                         linewidth=3, label=label)
-                if annotate_peaks:
-                    plt.annotate(label, xy=[two_theta, i],
-                                 xytext=[two_theta, i], fontsize=16)
-        plt.xlabel(r"2\theta (degrees)")
-        plt.ylabel("Intensities (scaled)")
-        plt.tight_layout()
-
-        return plt
-
-    def show_xrd_plot(self, structure, two_theta_range=(0, 90),
-                      annotate_peaks=True):
-        """
-        Shows the XRD plot.
-
-        Args:
-            structure (Structure): Input structure
-            two_theta_range ([float of length 2]): Tuple for range of
-                two_thetas to calculate in degrees. Defaults to (0, 90). Set to
-                None if you want all diffracted beams within the limiting
-                sphere of radius 2 / wavelength.
-            annotate_peaks (bool): Whether to annotate the peaks with plane
-                information.
-        """
-        self.get_xrd_plot(structure, two_theta_range=two_theta_range,
-                          annotate_peaks=annotate_peaks).show()
-
-
-def get_unique_families(hkls):
-    """
-    Returns unique families of Miller indices. Families must be permutations
-    of each other.
-
-    Args:
-        hkls ([h, k, l]): List of Miller indices.
-
-    Returns:
-        {hkl: multiplicity}: A dict with unique hkl and multiplicity.
-    """
-    #TODO: Definitely can be sped up.
-    def is_perm(hkl1, hkl2):
-        h1 = map(abs, hkl1)
-        h2 = map(abs, hkl2)
-        return all([i == j for i, j in zip(sorted(h1), sorted(h2))])
-
-    unique = {}
-    for hkl1 in hkls:
-        found = False
-        for hkl2 in unique.keys():
-            if is_perm(hkl1, hkl2):
-                found = True
-                unique[hkl2] += 1
-                break
-        if not found:
-            unique[hkl1] = 1
-
-    return unique
+            if v[0] / max_intensity * 100 > AbstractDiffractionPatternCalculator.SCALED_INTENSITY_TOL:
+                x.append(k)
+                y.append(v[0])
+                hkls.append([{"hkl": hkl, "multiplicity": mult} for hkl, mult in fam.items()])
+                d_hkls.append(v[2])
+        xrd = DiffractionPattern(x, y, hkls, d_hkls)
+        if scaled:
+            xrd.normalize(mode="max", value=100)
+        return xrd
