@@ -1,4 +1,3 @@
-# coding: utf-8
 # Copyright (c) Pymatgen Development Team.
 # Distributed under the terms of the MIT License.
 
@@ -6,10 +5,11 @@
 JahnTeller distortion analysis.
 """
 
+from __future__ import annotations
 
 import os
 import warnings
-from typing import Any, Dict, Optional, Tuple, Union
+from typing import Any, Dict, Literal, cast
 
 import numpy as np
 
@@ -91,7 +91,7 @@ class JahnTellerAnalyzer:
         calculate_valences: bool = True,
         guesstimate_spin: bool = False,
         op_threshold: float = 0.1,
-    ) -> Tuple[Dict, Structure]:
+    ) -> tuple[dict, Structure]:
         """Obtain an analysis of a given structure and if it may be Jahn-Teller
         active or not. This is a heuristic, and may give false positives and
         false negatives (false positives are preferred).
@@ -153,6 +153,8 @@ class JahnTellerAnalyzer:
 
                 if motif in ["oct", "tet"]:
 
+                    motif = cast(Literal["oct", "tet"], motif)  # mypy needs help
+
                     # guess spin of metal ion
                     if guesstimate_spin and "magmom" in site.properties:
 
@@ -175,7 +177,7 @@ class JahnTellerAnalyzer:
                             """
                             Avoid storing to unreasonable precision, hurts readability.
                             """
-                            return float("{:.4f}".format(f))
+                            return float(f"{f:.4f}")
 
                         # to be Jahn-Teller active, all ligands have to be the same
                         if len(ligands_species) == 1:
@@ -199,7 +201,7 @@ class JahnTellerAnalyzer:
                             {
                                 "site_indices": indices,
                                 "strength": "none",
-                                "reason": "Not Jahn-Teller active for this " "electronic configuration.",
+                                "reason": "Not Jahn-Teller active for this electronic configuration.",
                             }
                         )
                 else:
@@ -207,7 +209,7 @@ class JahnTellerAnalyzer:
                         {
                             "site_indices": indices,
                             "strength": "none",
-                            "reason": "motif is {}".format(motif),
+                            "reason": f"motif is {motif}",
                         }
                     )
 
@@ -231,7 +233,7 @@ class JahnTellerAnalyzer:
         calculate_valences: bool = True,
         guesstimate_spin: bool = False,
         op_threshold: float = 0.1,
-    ) -> Dict:
+    ) -> dict:
         """
         Convenience method, uses get_analysis_and_structure method.
 
@@ -300,7 +302,7 @@ class JahnTellerAnalyzer:
             )
             active = analysis["active"]
         except Exception as e:
-            warnings.warn("Error analyzing {}: {}".format(structure.composition.reduced_formula, e))
+            warnings.warn(f"Error analyzing {structure.composition.reduced_formula}: {e}")
 
         return active
 
@@ -345,7 +347,7 @@ class JahnTellerAnalyzer:
                         structure.add_site_property("possible_jt_active", jt_sites)
             return structure
         except Exception as e:
-            warnings.warn("Error analyzing {}: {}".format(structure.composition.reduced_formula, e))
+            warnings.warn(f"Error analyzing {structure.composition.reduced_formula}: {e}")
             return structure
 
     @staticmethod
@@ -364,14 +366,14 @@ class JahnTellerAnalyzer:
         # taken from get_crystal_field_spin
         elec = species.full_electronic_structure
         if len(elec) < 4 or elec[-1][1] != "s" or elec[-2][1] != "d":
-            raise AttributeError("Invalid element {} for crystal field calculation.".format(species.symbol))
+            raise AttributeError(f"Invalid element {species.symbol} for crystal field calculation.")
         nelectrons = int(elec[-1][2] + elec[-2][2] - species.oxi_state)
         if nelectrons < 0 or nelectrons > 10:
-            raise AttributeError("Invalid oxidation state {} for element {}".format(species.oxi_state, species.symbol))
+            raise AttributeError(f"Invalid oxidation state {species.oxi_state} for element {species.symbol}")
 
         return nelectrons
 
-    def get_magnitude_of_effect_from_species(self, species: Union[str, Species], spin_state: str, motif: str) -> str:
+    def get_magnitude_of_effect_from_species(self, species: str | Species, spin_state: str, motif: str) -> str:
         """
         Get magnitude of Jahn-Teller effect from provided species, spin state and motif.
 
@@ -404,7 +406,7 @@ class JahnTellerAnalyzer:
         return magnitude
 
     @staticmethod
-    def get_magnitude_of_effect_from_spin_config(motif: str, spin_config: Dict[str, float]) -> str:
+    def get_magnitude_of_effect_from_spin_config(motif: str, spin_config: dict[str, float]) -> str:
         """
         Roughly, the magnitude of Jahn-Teller distortion will be:
         * in octahedral environments, strong if e_g orbitals
@@ -435,7 +437,9 @@ class JahnTellerAnalyzer:
         return magnitude
 
     @staticmethod
-    def _estimate_spin_state(species: Union[str, Species], motif: str, known_magmom: float) -> str:
+    def _estimate_spin_state(
+        species: str | Species, motif: Literal["oct", "tet"], known_magmom: float
+    ) -> Literal["undefined", "low", "high", "unknown"]:
         """Simple heuristic to estimate spin state. If magnetic moment
         is sufficiently close to that predicted for a given spin state,
         we assign it that state. If we only have data for one spin
@@ -443,12 +447,12 @@ class JahnTellerAnalyzer:
         complexes are high-spin, since this is typically the case).
 
         Args:
-          species: str or Species
-          motif: "oct" or "tet"
-          known_magmom: magnetic moment in Bohr magnetons
+            species: str or Species
+            motif ("oct" | "tet"): Tetrahedron or octahedron crystal site coordination
+            known_magmom: magnetic moment in Bohr magnetons
 
-        Returns: "undefined" (if only one spin state possible), "low",
-        "high" or "unknown"
+        Returns:
+            "undefined" (if only one spin state possible), "low", "high" or "unknown"
         """
         mu_so_high = JahnTellerAnalyzer.mu_so(species, motif=motif, spin_state="high")
         mu_so_low = JahnTellerAnalyzer.mu_so(species, motif=motif, spin_state="low")
@@ -469,18 +473,18 @@ class JahnTellerAnalyzer:
         return "unknown"
 
     @staticmethod
-    def mu_so(species: Union[str, Species], motif: str, spin_state: str) -> Optional[float]:
+    def mu_so(species: str | Species, motif: Literal["oct", "tet"], spin_state: Literal["high", "low"]) -> float | None:
         """Calculates the spin-only magnetic moment for a
         given species. Only supports transition metals.
 
         Args:
-          species: Species
-          motif: "oct" or "tet"
-          spin_state: "high" or "low"
+            species: Species
+            motif ("oct" | "tet"): Tetrahedron or octahedron crystal site coordination
+            spin_state ("low" | "high"): Whether the species is in a high or low spin state
 
         Returns:
-          Spin-only magnetic moment in Bohr magnetons or None if
-          species crystal field not defined
+            float: Spin-only magnetic moment in Bohr magnetons or None if
+                species crystal field not defined
         """
         try:
             sp = get_el_sp(species)
