@@ -1,4 +1,3 @@
-# coding: utf-8
 # Copyright (c) Pymatgen Development Team.
 # Distributed under the terms of the MIT License.
 
@@ -349,7 +348,7 @@ class WulffShape:
         )
 
     def show(self, *args, **kwargs):
-        r"""
+        """
         Show the Wulff plot.
 
         Args:
@@ -406,7 +405,7 @@ class WulffShape:
         Args:
             color_set: default is 'PuBu'
             grid_off (bool): default is True
-            axis_off (bool): default is Ture
+            axis_off (bool): default is True
             show_area (bool): default is False
             alpha (float): chosen from 0 to 1 (float), default is 1
             off_color: Default color for facets not present on the Wulff shape.
@@ -469,7 +468,7 @@ class WulffShape:
         # set ranges of x, y, z
         # find the largest distance between on_wulff pts and the origin,
         # to ensure complete and consistent display for all directions
-        r_range = max([np.linalg.norm(x) for x in wulff_pt_list])
+        r_range = max(np.linalg.norm(x) for x in wulff_pt_list)
         ax.set_xlim([-r_range * 1.1, r_range * 1.1])
         ax.set_ylim([-r_range * 1.1, r_range * 1.1])
         ax.set_zlim([-r_range * 1.1, r_range * 1.1])  # pylint: disable=E1101
@@ -520,7 +519,7 @@ class WulffShape:
                 orientation="vertical",
             )
             units = "$J/m^2$" if units_in_JPERM2 else r"$eV/\AA^2$"
-            cbar.set_label("Surface Energies (%s)" % (units), fontsize=25)
+            cbar.set_label(f"Surface Energies ({units})", fontsize=25)
 
         if grid_off:
             ax.grid("off")
@@ -602,9 +601,9 @@ class WulffShape:
                     i=tri_indices[0],
                     j=tri_indices[1],
                     k=tri_indices[2],
-                    hovertemplate="<br>%{text}<br>" + "%s=%.3f %s<br>" % ("\u03b3", plane.e_surf, units),
+                    hovertemplate="<br>%{text}<br>" + f"γ={plane.e_surf:.3f} {units}<br>",
                     color=color,
-                    text=[r"Miller index: %s" % hkl] * len(x_pts),
+                    text=[f"Miller index: {hkl}"] * len(x_pts),
                     hoverinfo="name",
                     name="",
                 )
@@ -615,7 +614,7 @@ class WulffShape:
             c = [norm_e, color]
             if c not in color_scale:
                 color_scale.append(c)
-                ticktext.append("%.3f" % plane.e_surf)
+                ticktext.append(f"{plane.e_surf:.3f}")
                 tickvals.append(norm_e)
 
         # Add colorbar
@@ -626,7 +625,7 @@ class WulffShape:
             z=[0],
             colorbar=go.ColorBar(
                 title={
-                    "text": r"Surface energy %s" % units,
+                    "text": f"Surface energy {units}",
                     "side": "right",
                     "font": {"size": 25},
                 },
