@@ -30,29 +30,29 @@ If you use this module, please cite the following:
 
 A. Otero-de-la-Roza, E. R. Johnson and V. Luaña,
 Comput. Phys. Commun. 185, 1007-1018 (2014)
-(http://dx.doi.org/10.1016/j.cpc.2013.10.026)
+(https://doi.org/10.1016/j.cpc.2013.10.026)
 
 A. Otero-de-la-Roza, M. A. Blanco, A. Martín Pendás and
 V. Luaña, Comput. Phys. Commun. 180, 157–166 (2009)
-(http://dx.doi.org/10.1016/j.cpc.2008.07.018)
+(https://doi.org/10.1016/j.cpc.2008.07.018)
 """
 
+import glob
 import logging
 import os
 import subprocess
 import warnings
 from enum import Enum
+from shutil import which
 
 import numpy as np
 from monty.dev import requires
 from monty.json import MSONable
-from monty.os.path import which
 from monty.serialization import loadfn
 from monty.tempfile import ScratchDir
 from scipy.spatial import KDTree
 
 from pymatgen.analysis.graphs import StructureGraph
-from pymatgen.command_line.bader_caller import get_filepath
 from pymatgen.core.periodic_table import DummySpecies
 from pymatgen.io.vasp.inputs import Potcar
 from pymatgen.io.vasp.outputs import Chgcar, VolumetricData
@@ -95,7 +95,7 @@ class Critic2Caller:
             warnings.warn(stderr)
 
         if rs.returncode != 0:
-            raise RuntimeError("critic2 exited with return code {}: {}".format(rs.returncode, stdout))
+            raise RuntimeError(f"critic2 exited with return code {rs.returncode}: {stdout}")
 
         self._stdout = stdout
         self._stderr = stderr
@@ -146,7 +146,7 @@ class Critic2Caller:
         * CPEPS, float (Bohr units in crystals), minimum distance between
           critical points for them to be equivalent
         * NUCEPS, same as CPEPS but specifically for nucleus critical
-          points (critic2 default is depedent on grid dimensions)
+          points (critic2 default is dependent on grid dimensions)
         * NUCEPSH, same as NUCEPS but specifically for hydrogen nuclei
           since associated charge density can be significantly displaced
           from hydrogen nucleus
@@ -192,7 +192,7 @@ class Critic2Caller:
         if chgcar:
             input_script += ["load int.CHGCAR id chg_int", "integrable chg_int"]
             if zpsp:
-                zpsp_str = " zpsp " + " ".join(["{} {}".format(symbol, int(zval)) for symbol, zval in zpsp.items()])
+                zpsp_str = " zpsp " + " ".join([f"{symbol} {int(zval)}" for symbol, zval in zpsp.items()])
                 input_script[-2] += zpsp_str
 
         # Command to run automatic analysis
@@ -200,9 +200,9 @@ class Critic2Caller:
         for k, v in settings.items():
             if isinstance(v, list):
                 for item in v:
-                    auto += "{} {} ".format(k, item)
+                    auto += f"{k} {item} "
             else:
-                auto += "{} {} ".format(k, v)
+                auto += f"{k} {v} "
         input_script += [auto]
 
         if write_cml:
@@ -322,6 +322,29 @@ class CriticalPointType(Enum):
     nnattr = "nnattr"  # (3, -3), non-nuclear attractor
 
 
+def get_filepath(filename, warning, path, suffix):
+    """
+    Args:
+        filename: Filename
+        warning: Warning message
+        path: Path to search
+        suffix: Suffixes to search.
+    """
+    paths = glob.glob(os.path.join(path, filename + suffix + "*"))
+    if not paths:
+        warnings.warn(warning)
+        return None
+    if len(paths) > 1:
+        # using reverse=True because, if multiple files are present,
+        # they likely have suffixes 'static', 'relax', 'relax2', etc.
+        # and this would give 'static' over 'relax2' over 'relax'
+        # however, better to use 'suffix' kwarg to avoid this!
+        paths.sort(reverse=True)
+        warnings.warn(f"Multiple files detected, using {os.path.basename(path)}")
+    path = paths[0]
+    return path
+
+
 class CriticalPoint(MSONable):
     """
     Access information about a critical point and the field values at that point.
@@ -348,8 +371,8 @@ class CriticalPoint(MSONable):
 
         :param index: index of point
         :param type: type of point, given as a string
-        :param coords: Cartesian co-ordinates in Angstroms
-        :param frac_coords: fractional co-ordinates
+        :param coords: Cartesian coordinates in Angstroms
+        :param frac_coords: fractional coordinates
         :param point_group: point group associated with critical point
         :param multiplicity: number of equivalent critical points
         :param field: value of field at point (f)
@@ -374,7 +397,7 @@ class CriticalPoint(MSONable):
         return CriticalPointType(self._type)
 
     def __str__(self):
-        return "Critical Point: {} ({})".format(self.type.name, self.frac_coords)
+        return f"Critical Point: {self.type.name} ({self.frac_coords})"
 
     @property
     def laplacian(self):
@@ -487,7 +510,7 @@ class Critic2Analysis(MSONable):
             for idx, node in self.nodes.items():
                 cp = self.critical_points[node["unique_idx"]]
                 if cp.type.value in include_critical_points:
-                    specie = DummySpecies("X{}cp".format(cp.type.value[0]), oxidation_state=None)
+                    specie = DummySpecies(f"X{cp.type.value[0]}cp", oxidation_state=None)
                     structure.append(
                         specie,
                         node["frac_coords"],
@@ -706,9 +729,7 @@ class Critic2Analysis(MSONable):
         def get_volume_and_charge(nonequiv_idx):
             attractor = yt["integration"]["attractors"][nonequiv_idx - 1]
             if attractor["id"] != nonequiv_idx:
-                raise ValueError(
-                    "List of attractors may be un-ordered (wanted id={}): {}".format(nonequiv_idx, attractor)
-                )
+                raise ValueError(f"List of attractors may be un-ordered (wanted id={nonequiv_idx}): {attractor}")
             return (
                 attractor["integrals"][volume_idx],
                 attractor["integrals"][charge_idx],
@@ -744,7 +765,7 @@ class Critic2Analysis(MSONable):
 
         if zpsp:
             if len(charge_transfer) != len(charges):
-                warnings.warn("Something went wrong calculating charge transfer: {}".format(charge_transfer))
+                warnings.warn(f"Something went wrong calculating charge transfer: {charge_transfer}")
             else:
                 structure.add_site_property("bader_charge_transfer", charge_transfer)
 
@@ -868,7 +889,7 @@ class Critic2Analysis(MSONable):
         :param idx: index
         :param unique_idx: index of unique CriticalPoint,
             used to look up more information of point (field etc.)
-        :param frac_coord: fractional co-ordinates of point
+        :param frac_coord: fractional coordinates of point
         :return:
         """
         self.nodes[idx] = {"unique_idx": unique_idx, "frac_coords": frac_coords}
