@@ -1,21 +1,21 @@
-# coding: utf-8
 # Copyright (c) Pymatgen Development Team.
 # Distributed under the terms of the MIT License.
 """
 This module provides classes to store, generate, and manipulate material interfaces.
 """
+
 from __future__ import annotations
 
 from itertools import chain, combinations, product
-from typing import Dict, List, Tuple
 
 import numpy as np
 from scipy.cluster.hierarchy import fcluster, linkage
 from scipy.spatial.distance import squareform
 
 from pymatgen.analysis.adsorption import AdsorbateSiteFinder
-from pymatgen.core import Lattice, Site, Structure
+from pymatgen.core.lattice import Lattice
 from pymatgen.core.sites import PeriodicSite
+from pymatgen.core.structure import Site, Structure
 from pymatgen.core.surface import Slab
 from pymatgen.symmetry.analyzer import SpacegroupAnalyzer
 
@@ -35,14 +35,14 @@ class Interface(Structure):
         validate_proximity=False,
         to_unit_cell=False,
         coords_are_cartesian=False,
-        in_plane_offset: Tuple[float, float] = (0, 0),
+        in_plane_offset: tuple[float, float] = (0, 0),
         gap: float = 0,
         vacuum_over_film: float = 0.0,
-        interface_properties: Dict = {},
+        interface_properties: dict | None = None,
     ):
         """
         Makes an interface structure, a structure object with additional information
-        and methods pertaining to interfaces
+        and methods pertaining to interfaces.
 
         Args:
             lattice (Lattice/3x3 array): The lattice, either as a
@@ -65,7 +65,7 @@ class Interface(Structure):
             validate_proximity (bool): Whether to check if there are sites
                 that are less than 0.01 Ang apart. Defaults to False.
             coords_are_cartesian (bool): Set to True if you are providing
-                coordinates in cartesian coordinates. Defaults to False.
+                coordinates in Cartesian coordinates. Defaults to False.
             site_properties (dict): Properties associated with the sites as a
                 dict of sequences, e.g., {"magmom":[5,5,5,5]}. The sequences
                 have to be the same length as the atomic species and
@@ -77,13 +77,14 @@ class Interface(Structure):
             vacuum_over_film: vacuum space above the film in Angstroms
         """
 
-        assert "interface_label" in site_properties, ValueError(
-            "Must provide labeling of substrate and film sites in site properties"
-        )
+        assert (
+            "interface_label" in site_properties
+        ), "Must provide labeling of substrate and film sites in site properties"
+
         self._in_plane_offset = np.array(in_plane_offset, dtype="float")
         self._gap = gap
         self._vacuum_over_film = vacuum_over_film
-        self.interface_properties = interface_properties
+        self.interface_properties = interface_properties or {}
 
         super().__init__(
             lattice,
@@ -117,7 +118,7 @@ class Interface(Structure):
     @property
     def gap(self) -> float:
         """
-        The gap in cartesian units between the film and the substrate
+        The gap in Cartesian units between the film and the substrate
         """
         return self._gap
 
@@ -135,7 +136,7 @@ class Interface(Structure):
     @property
     def vacuum_over_film(self) -> float:
         """
-        The vacuum space over the film in cartesian units
+        The vacuum space over the film in Cartesian units
         """
         return self._vacuum_over_film
 
@@ -150,15 +151,15 @@ class Interface(Structure):
         self.__update_c(self.lattice.c + delta)
 
     @property
-    def substrate_indicies(self) -> List[int]:
+    def substrate_indices(self) -> list[int]:
         """
-        Site indicies for the substrate atoms
+        Site indices for the substrate atoms
         """
-        sub_indicies = [i for i, tag in enumerate(self.site_properties["interface_label"]) if "substrate" in tag]
-        return sub_indicies
+        sub_indices = [i for i, tag in enumerate(self.site_properties["interface_label"]) if "substrate" in tag]
+        return sub_indices
 
     @property
-    def substrate_sites(self) -> List[Site]:
+    def substrate_sites(self) -> list[Site]:
         """
         The site objects in the substrate
         """
@@ -173,15 +174,15 @@ class Interface(Structure):
         return Structure.from_sites(self.substrate_sites)
 
     @property
-    def film_indices(self) -> List[int]:
+    def film_indices(self) -> list[int]:
         """
         Site indices of the film sites
         """
-        f_indicies = [i for i, tag in enumerate(self.site_properties["interface_label"]) if "film" in tag]
-        return f_indicies
+        f_indices = [i for i, tag in enumerate(self.site_properties["interface_label"]) if "film" in tag]
+        return f_indices
 
     @property
-    def film_sites(self) -> List[Site]:
+    def film_sites(self) -> list[Site]:
         """
         Return the film sites of the interface.
         """
@@ -195,13 +196,10 @@ class Interface(Structure):
         """
         return Structure.from_sites(self.film_sites)
 
-    def copy(self) -> Interface:  # type:ignore
+    def copy(self):
         """
-        Convenience method to get a copy of the structure, with options to add
-        site properties.
-
         Returns:
-            A copy of the Interface.
+            Interface: A copy of the Interface.
         """
 
         return Interface.from_dict(self.as_dict())
@@ -223,7 +221,7 @@ class Interface(Structure):
         struct_copy.sort(key=key, reverse=reverse)
         return struct_copy
 
-    def get_shifts_based_on_adsorbate_sites(self, tolerance: float = 0.1) -> List[Tuple[float, float]]:
+    def get_shifts_based_on_adsorbate_sites(self, tolerance: float = 0.1) -> list[tuple[float, float]]:
         """
         Computes possible in-plane shifts based on an adsorbate site  algorithm
 
@@ -257,7 +255,7 @@ class Interface(Structure):
         # Round shifts to tolerance
         pos_shift[:, 0] = _base_round(pos_shift[:, 0], base=tolerance / substrate.lattice.a)
         pos_shift[:, 1] = _base_round(pos_shift[:, 1], base=tolerance / substrate.lattice.b)
-        # C-axis is not usefull
+        # C-axis is not useful
         pos_shift = pos_shift[:, 0:2]
 
         return list(np.unique(pos_shift, axis=0))
@@ -290,8 +288,8 @@ class Interface(Structure):
 
     def __update_c(self, new_c: float) -> None:
         """
-        Modifies the c-direction of the lattice without changing the site cartesian coordinates
-        Be carefull you can mess up the interface by setting a c-length that can't accomodate all the sites
+        Modifies the c-direction of the lattice without changing the site Cartesian coordinates
+        Be careful you can mess up the interface by setting a c-length that can't accommodate all the sites
         """
         if new_c <= 0:
             raise ValueError("New c-length must be greater than 0")
@@ -302,7 +300,7 @@ class Interface(Structure):
 
         for site, c_coords in zip(self, self.cart_coords):
             site._lattice = new_latice  # Update the lattice
-            site.coords = c_coords  # Put back into original cartesian space
+            site.coords = c_coords  # Put back into original Cartesian space
 
     def as_dict(self):
         """
@@ -344,10 +342,10 @@ class Interface(Structure):
         cls,
         substrate_slab: Slab,
         film_slab: Slab,
-        in_plane_offset: Tuple[float, float] = (0, 0),
+        in_plane_offset: tuple[float, float] = (0, 0),
         gap: float = 1.6,
         vacuum_over_film: float = 0.0,
-        interface_properties: Dict = {},
+        interface_properties: dict | None = None,
         center_slab: bool = True,
     ) -> Interface:
         """
@@ -367,9 +365,9 @@ class Interface(Structure):
             vacuum_over_film: vacuum space above the film in Angstroms
             structure_properties: dictionary of misc properties for this structure
             center_slab: center the slab
-
-
         """
+        interface_properties = interface_properties or {}
+
         # Ensure c-axis is orthogonal to a/b plane
         if isinstance(substrate_slab, Slab):
             substrate_slab = substrate_slab.get_orthogonal_c_slab()
@@ -480,7 +478,7 @@ def label_termination(slab: Structure) -> str:
     z = linkage(condensed_m)
     clusters = fcluster(z, 0.25, criterion="distance")
 
-    clustered_sites: Dict[int, List[Site]] = {c: [] for c in clusters}
+    clustered_sites: dict[int, list[Site]] = {c: [] for c in clusters}
     for i, c in enumerate(clusters):
         clustered_sites[c].append(slab[i])
 
@@ -522,7 +520,7 @@ def count_layers(struc: Structure, el=None) -> int:
     z = linkage(condensed_m)
     clusters = fcluster(z, 0.25, criterion="distance")
 
-    clustered_sites: Dict[int, List[Site]] = {c: [] for c in clusters}
+    clustered_sites: dict[int, list[Site]] = {c: [] for c in clusters}
     for i, c in enumerate(clusters):
         clustered_sites[c].append(struc[i])
 
