@@ -1,4 +1,3 @@
-# coding: utf-8
 # Copyright (c) Pymatgen Development Team.
 # Distributed under the terms of the MIT License.
 
@@ -6,13 +5,15 @@
 Defines the classes relating to 3D lattices.
 """
 
+from __future__ import annotations
+
 import collections
 import itertools
 import math
 import warnings
 from fractions import Fraction
 from functools import reduce
-from typing import Dict, Iterator, List, Optional, Sequence, Tuple, Union
+from typing import Dict, Iterator, List, Optional, Sequence, Tuple
 
 import numpy as np
 from monty.dev import deprecated
@@ -32,7 +33,7 @@ __email__ = "shyuep@gmail.com"
 
 class Lattice(MSONable):
     """
-    A lattice object.  Essentially a matrix with conversion matrices. In
+    A lattice object. Essentially a matrix with conversion matrices. In
     general, it is assumed that length units are in Angstroms and angles are in
     degrees unless otherwise stated.
     """
@@ -65,14 +66,14 @@ class Lattice(MSONable):
         self._lll_inverse = None
 
     @property
-    def lengths(self) -> Tuple[float, float, float]:
+    def lengths(self) -> tuple[float, float, float]:
         """
         :return: The lengths (a, b, c) of the lattice.
         """
-        return tuple(np.sqrt(np.sum(self._matrix ** 2, axis=1)).tolist())  # type: ignore
+        return tuple(np.sqrt(np.sum(self._matrix**2, axis=1)).tolist())  # type: ignore
 
     @property
-    def angles(self) -> Tuple[float, float, float]:
+    def angles(self) -> tuple[float, float, float]:
         """
         Returns the angles (alpha, beta, gamma) of the lattice.
         """
@@ -117,7 +118,7 @@ class Lattice(MSONable):
             m = (self.lengths, self.angles)
         else:
             fmt = "{} {} {}\n{} {} {}\n{} {} {}"
-        return fmt.format(*[format(c, fmt_spec) for row in m for c in row])
+        return fmt.format(*(format(c, fmt_spec) for row in m for c in row))
 
     def copy(self):
         """Deep copy of self."""
@@ -147,7 +148,7 @@ class Lattice(MSONable):
 
     def get_cartesian_coords(self, fractional_coords: ArrayLike) -> np.ndarray:
         """
-        Returns the cartesian coordinates given fractional coordinates.
+        Returns the Cartesian coordinates given fractional coordinates.
 
         Args:
             fractional_coords (3x1 array): Fractional coords.
@@ -159,7 +160,7 @@ class Lattice(MSONable):
 
     def get_fractional_coords(self, cart_coords: ArrayLike) -> np.ndarray:
         """
-        Returns the fractional coordinates given cartesian coordinates.
+        Returns the fractional coordinates given Cartesian coordinates.
 
         Args:
             cart_coords (3x1 array): Cartesian coords.
@@ -171,9 +172,9 @@ class Lattice(MSONable):
 
     def get_vector_along_lattice_directions(self, cart_coords: ArrayLike) -> np.ndarray:
         """
-        Returns the coordinates along lattice directions given cartesian coordinates.
+        Returns the coordinates along lattice directions given Cartesian coordinates.
 
-        Note, this is different than a projection of the cartesian vector along the
+        Note, this is different than a projection of the Cartesian vector along the
         lattice parameters. It is simply the fractional coordinates multiplied by the
         lattice vector magnitudes.
 
@@ -187,7 +188,7 @@ class Lattice(MSONable):
         Returns:
             Lattice coordinates.
         """
-        return self.lengths * self.get_fractional_coords(cart_coords)
+        return self.lengths * self.get_fractional_coords(cart_coords)  # type: ignore
 
     def d_hkl(self, miller_index: ArrayLike) -> float:
         """
@@ -205,7 +206,7 @@ class Lattice(MSONable):
         return 1 / ((dot(dot(hkl, gstar), hkl.T)) ** (1 / 2))
 
     @staticmethod
-    def cubic(a: float) -> "Lattice":
+    def cubic(a: float) -> Lattice:
         """
         Convenience constructor for a cubic lattice.
 
@@ -218,7 +219,7 @@ class Lattice(MSONable):
         return Lattice([[a, 0.0, 0.0], [0.0, a, 0.0], [0.0, 0.0, a]])
 
     @staticmethod
-    def tetragonal(a: float, c: float) -> "Lattice":
+    def tetragonal(a: float, c: float) -> Lattice:
         """
         Convenience constructor for a tetragonal lattice.
 
@@ -232,7 +233,7 @@ class Lattice(MSONable):
         return Lattice.from_parameters(a, a, c, 90, 90, 90)
 
     @staticmethod
-    def orthorhombic(a: float, b: float, c: float) -> "Lattice":
+    def orthorhombic(a: float, b: float, c: float) -> Lattice:
         """
         Convenience constructor for an orthorhombic lattice.
 
@@ -247,7 +248,7 @@ class Lattice(MSONable):
         return Lattice.from_parameters(a, b, c, 90, 90, 90)
 
     @staticmethod
-    def monoclinic(a: float, b: float, c: float, beta: float) -> "Lattice":
+    def monoclinic(a: float, b: float, c: float, beta: float) -> Lattice:
         """
         Convenience constructor for a monoclinic lattice.
 
@@ -265,7 +266,7 @@ class Lattice(MSONable):
         return Lattice.from_parameters(a, b, c, 90, beta, 90)
 
     @staticmethod
-    def hexagonal(a: float, c: float) -> "Lattice":
+    def hexagonal(a: float, c: float) -> Lattice:
         """
         Convenience constructor for a hexagonal lattice.
 
@@ -279,7 +280,7 @@ class Lattice(MSONable):
         return Lattice.from_parameters(a, a, c, 90, 90, 120)
 
     @staticmethod
-    def rhombohedral(a: float, alpha: float) -> "Lattice":
+    def rhombohedral(a: float, alpha: float) -> Lattice:
         """
         Convenience constructor for a rhombohedral lattice.
 
@@ -292,21 +293,6 @@ class Lattice(MSONable):
         """
         return Lattice.from_parameters(a, a, a, alpha, alpha, alpha)
 
-    @staticmethod
-    @deprecated(message="Use Lattice.from_parameters instead. This will be removed in v2020.*")
-    def from_lengths_and_angles(abc: Sequence[float], ang: Sequence[float]):
-        """
-        Create a Lattice using unit cell lengths and angles (in degrees).
-
-        Args:
-            abc (3x1 array): Lattice parameters, e.g. (4, 4, 5).
-            ang (3x1 array): Lattice angles in degrees, e.g., (90,90,120).
-
-        Returns:
-            A Lattice with the specified lattice parameters.
-        """
-        return Lattice.from_parameters(abc[0], abc[1], abc[2], ang[0], ang[1], ang[2])
-
     @classmethod
     def from_parameters(
         cls,
@@ -344,7 +330,7 @@ class Lattice(MSONable):
 
             vector_a = [float(a), 0.0, 0.0]
             vector_b = [b * cos_gamma, b * sin_gamma, 0]
-            vector_c = [c1, c2, math.sqrt(c ** 2 - c1 ** 2 - c2 ** 2)]
+            vector_c = [c1, c2, math.sqrt(c**2 - c1**2 - c2**2)]
 
         else:
             val = (cos_alpha * cos_beta - cos_gamma) / (sin_alpha * sin_beta)
@@ -363,7 +349,7 @@ class Lattice(MSONable):
         return Lattice([vector_a, vector_b, vector_c])
 
     @classmethod
-    def from_dict(cls, d: Dict, fmt: str = None, **kwargs):
+    def from_dict(cls, d: dict, fmt: str = None, **kwargs):
         """
         Create a Lattice from a dictionary containing the a, b, c, alpha, beta,
         and gamma parameters if fmt is None.
@@ -409,7 +395,7 @@ class Lattice(MSONable):
         return self.lengths[2]
 
     @property
-    def abc(self) -> Tuple[float, float, float]:
+    def abc(self) -> tuple[float, float, float]:
         """
         Lengths of the lattice vectors, i.e. (a, b, c)
         """
@@ -445,24 +431,14 @@ class Lattice(MSONable):
         return float(abs(dot(np.cross(m[0], m[1]), m[2])))
 
     @property
-    def parameters(self) -> Tuple[float, float, float, float, float, float]:
+    def parameters(self) -> tuple[float, float, float, float, float, float]:
         """
         Returns: (a, b, c, alpha, beta, gamma).
         """
         return (*self.lengths, *self.angles)
 
-    @property  # type: ignore
-    @deprecated(message="Use Lattice.parameters instead. This will be removed in v2020.*")
-    def lengths_and_angles(
-        self,
-    ) -> Tuple[Tuple[float, float, float], Tuple[float, float, float]]:
-        """
-        Returns (lattice lengths, lattice angles).
-        """
-        return self.lengths, self.angles
-
     @property
-    def reciprocal_lattice(self) -> "Lattice":
+    def reciprocal_lattice(self) -> Lattice:
         """
         Return the reciprocal lattice. Note that this is the standard
         reciprocal lattice used for solid state physics with a factor of 2 *
@@ -474,7 +450,7 @@ class Lattice(MSONable):
         return Lattice(v * 2 * np.pi)
 
     @property
-    def reciprocal_lattice_crystallographic(self) -> "Lattice":
+    def reciprocal_lattice_crystallographic(self) -> Lattice:
         """
         Returns the *crystallographic* reciprocal lattice, i.e., no factor of
         2 * pi.
@@ -921,7 +897,7 @@ class Lattice(MSONable):
         for reflection_matrix in reflection_matrices:
             all_reflections.append(np.dot(selling1, reflection_matrix))
 
-        return min([np.linalg.norm(reflection - selling2) for reflection in all_reflections])
+        return min(np.linalg.norm(reflection - selling2) for reflection in all_reflections)
 
     def __repr__(self):
         outs = [
@@ -953,9 +929,9 @@ class Lattice(MSONable):
         return 7
 
     def __str__(self):
-        return "\n".join([" ".join(["%.6f" % i for i in row]) for row in self._matrix])
+        return "\n".join([" ".join([f"{i:.6f}" for i in row]) for row in self._matrix])
 
-    def as_dict(self, verbosity: int = 0) -> Dict:
+    def as_dict(self, verbosity: int = 0) -> dict:
         """
         Json-serialization dict representation of the Lattice.
 
@@ -965,8 +941,8 @@ class Lattice(MSONable):
         """
 
         d = {
-            "@module": self.__class__.__module__,
-            "@class": self.__class__.__name__,
+            "@module": type(self).__module__,
+            "@class": type(self).__name__,
             "matrix": self._matrix.tolist(),
         }
         a, b, c, alpha, beta, gamma = self.parameters
@@ -987,11 +963,11 @@ class Lattice(MSONable):
 
     def find_all_mappings(
         self,
-        other_lattice: "Lattice",
+        other_lattice: Lattice,
         ltol: float = 1e-5,
         atol: float = 1,
         skip_rotation_matrix: bool = False,
-    ) -> Iterator[Tuple["Lattice", Optional[np.ndarray], np.ndarray]]:
+    ) -> Iterator[tuple[Lattice, np.ndarray | None, np.ndarray]]:
         """
         Finds all mappings between current lattice and another lattice.
 
@@ -1029,8 +1005,8 @@ class Lattice(MSONable):
         # this can't be broadcast because they're different lengths
         inds = [np.logical_and(dist / l < 1 + ltol, dist / l > 1 / (1 + ltol)) for l in lengths]  # type: ignore
         c_a, c_b, c_c = (cart[i] for i in inds)
-        f_a, f_b, f_c = (frac[i] for i in inds)
-        l_a, l_b, l_c = (np.sum(c ** 2, axis=-1) ** 0.5 for c in (c_a, c_b, c_c))
+        f_a, f_b, f_c = (frac[i] for i in inds)  # type: ignore
+        l_a, l_b, l_c = (np.sum(c**2, axis=-1) ** 0.5 for c in (c_a, c_b, c_c))
 
         def get_angles(v1, v2, l1, l2):
             x = np.inner(v1, v2) / l1[:, None] / l2
@@ -1046,7 +1022,7 @@ class Lattice(MSONable):
         for i, all_j in enumerate(gammab):
             inds = np.logical_and(all_j[:, None], np.logical_and(alphab, betab[i][None, :]))
             for j, k in np.argwhere(inds):
-                scale_m = np.array((f_a[i], f_b[j], f_c[k]), dtype=np.int_)  # type: ignore
+                scale_m = np.array((f_a[i], f_b[j], f_c[k]), dtype=int)  # type: ignore
                 if abs(np.linalg.det(scale_m)) < 1e-8:
                     continue
 
@@ -1061,11 +1037,11 @@ class Lattice(MSONable):
 
     def find_mapping(
         self,
-        other_lattice: "Lattice",
+        other_lattice: Lattice,
         ltol: float = 1e-5,
         atol: float = 1,
         skip_rotation_matrix: bool = False,
-    ) -> Optional[Tuple["Lattice", Optional[np.ndarray], np.ndarray]]:
+    ) -> tuple[Lattice, np.ndarray | None, np.ndarray] | None:
         """
         Finds a mapping between current lattice and another lattice. There
         are an infinite number of choices of basis vectors for two entirely
@@ -1098,7 +1074,7 @@ class Lattice(MSONable):
             return x
         return None
 
-    def get_lll_reduced_lattice(self, delta: float = 0.75) -> "Lattice":
+    def get_lll_reduced_lattice(self, delta: float = 0.75) -> Lattice:
         """
         :param delta: Delta parameter.
         :return: LLL reduced Lattice.
@@ -1107,7 +1083,7 @@ class Lattice(MSONable):
             self._lll_matrix_mappings[delta] = self._calculate_lll()
         return Lattice(self._lll_matrix_mappings[delta][0])
 
-    def _calculate_lll(self, delta: float = 0.75) -> Tuple[np.ndarray, np.ndarray]:
+    def _calculate_lll(self, delta: float = 0.75) -> tuple[np.ndarray, np.ndarray]:
         """
         Performs a Lenstra-Lenstra-Lovasz lattice basis reduction to obtain a
         c-reduced basis. This method returns a basis which is as "good" as
@@ -1124,7 +1100,7 @@ class Lattice(MSONable):
         """
         # Transpose the lattice matrix first so that basis vectors are columns.
         # Makes life easier.
-        # pylint: disable=E1136,E1137
+        # pylint: disable=E1136,E1137,E1126
         a = self._matrix.copy().T
 
         b = np.zeros((3, 3))  # Vectors after the Gram-Schmidt process
@@ -1180,7 +1156,8 @@ class Lattice(MSONable):
                 else:
                     # We have to do p/q, so do lstsq(q.T, p.T).T instead.
                     p = dot(a[:, k:3].T, b[:, (k - 2) : k])
-                    q = np.diag(m[(k - 2) : k])
+                    q = np.diag(m[(k - 2) : k])  # type: ignore
+                    # pylint: disable=E1101
                     result = np.linalg.lstsq(q.T, p.T, rcond=None)[0].T  # type: ignore
                     u[k:3, (k - 2) : k] = result
 
@@ -1200,7 +1177,7 @@ class Lattice(MSONable):
         """
         return dot(lll_frac_coords, self.lll_mapping)
 
-    def get_niggli_reduced_lattice(self, tol: float = 1e-5) -> "Lattice":
+    def get_niggli_reduced_lattice(self, tol: float = 1e-5) -> Lattice:
         """
         Get the Niggli reduced lattice using the numerically stable algo
         proposed by R. W. Grosse-Kunstleve, N. K. Sauter, & P. D. Adams,
@@ -1328,7 +1305,7 @@ class Lattice(MSONable):
 
         raise ValueError("can't find niggli")
 
-    def scale(self, new_volume: float) -> "Lattice":
+    def scale(self, new_volume: float) -> Lattice:
         """
         Return a new Lattice with volume new_volume by performing a
         scaling of the lattice vectors so that length proportions and angles
@@ -1351,7 +1328,7 @@ class Lattice(MSONable):
 
         return Lattice(versors * (new_c * ratios))
 
-    def get_wigner_seitz_cell(self) -> List[List[np.ndarray]]:
+    def get_wigner_seitz_cell(self) -> list[list[np.ndarray]]:
         """
         Returns the Wigner-Seitz cell for the given lattice.
 
@@ -1379,7 +1356,7 @@ class Lattice(MSONable):
 
         return to_return
 
-    def get_brillouin_zone(self) -> List[List[np.ndarray]]:
+    def get_brillouin_zone(self) -> list[list[np.ndarray]]:
         """
         Returns the Wigner-Seitz cell for the reciprocal lattice, aka the
         Brillouin Zone.
@@ -1399,7 +1376,7 @@ class Lattice(MSONable):
         Args:
             coords_a, coords_b: Array-like objects with the coordinates.
             frac_coords (bool): Boolean stating whether the vector
-                corresponds to fractional or cartesian coordinates.
+                corresponds to fractional or Cartesian coordinates.
 
         Returns:
             one-dimensional `numpy` array.
@@ -1418,8 +1395,8 @@ class Lattice(MSONable):
         if not frac_coords:
             cart_a, cart_b = coords_a, coords_b
         else:
-            cart_a = np.reshape([self.get_cartesian_coords(vec) for vec in coords_a], (-1, 3))  # type: ignore
-            cart_b = np.reshape([self.get_cartesian_coords(vec) for vec in coords_b], (-1, 3))  # type: ignore
+            cart_a = np.reshape([self.get_cartesian_coords(vec) for vec in coords_a], (-1, 3))
+            cart_b = np.reshape([self.get_cartesian_coords(vec) for vec in coords_b], (-1, 3))
 
         return np.array([dot(a, b) for a, b in zip(cart_a, cart_b)])
 
@@ -1432,12 +1409,12 @@ class Lattice(MSONable):
                 Array-like object with the coordinates.
             frac_coords:
                 Boolean stating whether the vector corresponds to fractional or
-                cartesian coordinates.
+                Cartesian coordinates.
 
         Returns:
             one-dimensional `numpy` array.
         """
-        return np.sqrt(self.dot(coords, coords, frac_coords=frac_coords))
+        return np.sqrt(self.dot(coords, coords, frac_coords=frac_coords))  # type: ignore
 
     def get_points_in_sphere(
         self,
@@ -1445,7 +1422,7 @@ class Lattice(MSONable):
         center: ArrayLike,
         r: float,
         zip_results=True,
-    ) -> Union[List[Tuple[np.ndarray, float, int, np.ndarray]], List[np.ndarray], List]:
+    ) -> list[tuple[np.ndarray, float, int, np.ndarray]] | list[np.ndarray] | list:
         """
         Find all points within a sphere from the point taking into account
         periodic boundary conditions. This includes sites in other periodic
@@ -1479,13 +1456,11 @@ class Lattice(MSONable):
         """
         try:
             # pylint: disable=C0415
-            from pymatgen.optimization.neighbors import (
-                find_points_in_spheres,  # type: ignore
-            )
+            from pymatgen.optimization.neighbors import find_points_in_spheres
         except ImportError:
             return self.get_points_in_sphere_py(frac_points=frac_points, center=center, r=r, zip_results=zip_results)
         else:
-            frac_points = np.ascontiguousarray(frac_points, dtype=np.float_)  # type: ignore
+            frac_points = np.ascontiguousarray(frac_points, dtype=np.float_)
             r = float(r)
             lattice_matrix = np.array(self.matrix)
             lattice_matrix = np.ascontiguousarray(lattice_matrix)
@@ -1523,7 +1498,7 @@ class Lattice(MSONable):
         center: ArrayLike,
         r: float,
         zip_results=True,
-    ) -> Union[List[Tuple[np.ndarray, float, int, np.ndarray]], List[np.ndarray],]:
+    ) -> list[tuple[np.ndarray, float, int, np.ndarray]] | list[np.ndarray]:
         """
         Find all points within a sphere from the point taking into account
         periodic boundary conditions. This includes sites in other periodic
@@ -1578,10 +1553,10 @@ class Lattice(MSONable):
         center: ArrayLike,
         r: float,
         zip_results=True,
-    ) -> Union[
-        List[Tuple[np.ndarray, float, int, np.ndarray]],
-        Tuple[List[np.ndarray], List[float], List[int], List[np.ndarray]],
-    ]:
+    ) -> (
+        list[tuple[np.ndarray, float, int, np.ndarray]]
+        | tuple[list[np.ndarray], list[float], list[int], list[np.ndarray]]
+    ):
         """
         Find all points within a sphere from the point taking into account
         periodic boundary conditions. This includes sites in other periodic
@@ -1631,12 +1606,12 @@ class Lattice(MSONable):
         # Generate all possible images that could be within `r` of `center`
         mins = np.floor(pcoords - nmax)
         maxes = np.ceil(pcoords + nmax)
-        arange = np.arange(start=mins[0], stop=maxes[0], dtype=np.int_)
-        brange = np.arange(start=mins[1], stop=maxes[1], dtype=np.int_)
-        crange = np.arange(start=mins[2], stop=maxes[2], dtype=np.int_)
-        arange = arange[:, None] * np.array([1, 0, 0], dtype=np.int_)[None, :]
-        brange = brange[:, None] * np.array([0, 1, 0], dtype=np.int_)[None, :]
-        crange = crange[:, None] * np.array([0, 0, 1], dtype=np.int_)[None, :]
+        arange = np.arange(start=mins[0], stop=maxes[0], dtype=int)
+        brange = np.arange(start=mins[1], stop=maxes[1], dtype=int)
+        crange = np.arange(start=mins[2], stop=maxes[2], dtype=int)
+        arange = arange[:, None] * np.array([1, 0, 0], dtype=int)[None, :]
+        brange = brange[:, None] * np.array([0, 1, 0], dtype=int)[None, :]
+        crange = crange[:, None] * np.array([0, 0, 1], dtype=int)[None, :]
         images = arange[:, None, None] + brange[None, :, None] + crange[None, None, :]
 
         # Generate the coordinates of all atoms within these images
@@ -1645,13 +1620,13 @@ class Lattice(MSONable):
         # Determine distance from `center`
         cart_coords = self.get_cartesian_coords(fcoords)
         cart_images = self.get_cartesian_coords(images)
-        coords = cart_coords[:, None, None, None, :] + cart_images[None, :, :, :, :]
+        coords = cart_coords[:, None, None, None, :] + cart_images[None, :, :, :, :]  # pylint: disable=E1126
         coords -= center[None, None, None, None, :]
         coords **= 2
         d_2 = np.sum(coords, axis=4)
 
         # Determine which points are within `r` of `center`
-        within_r = np.where(d_2 <= r ** 2)
+        within_r = np.where(d_2 <= r**2)
         #  `within_r` now contains the coordinates of each image that is
         #    inside of the cutoff distance. It has 4 coordinates:
         #   0 - index of the image within `frac_points`
@@ -1692,7 +1667,7 @@ class Lattice(MSONable):
             fcoords2: Second set of fractional coordinates.
 
         Returns:
-            2d array of cartesian distances. E.g the distance between
+            2d array of Cartesian distances. E.g the distance between
             fcoords1[i] and fcoords2[j] is distances[i,j]
         """
         v, d2 = pbc_shortest_vectors(self, fcoords1, fcoords2, return_d2=True)
@@ -1721,8 +1696,8 @@ class Lattice(MSONable):
         self,
         frac_coords1: ArrayLike,
         frac_coords2: ArrayLike,
-        jimage: Optional[ArrayLike] = None,
-    ) -> Tuple[float, np.ndarray]:
+        jimage: ArrayLike | None = None,
+    ) -> tuple[float, np.ndarray]:
         """
         Gets distance between two frac_coords assuming periodic boundary
         conditions. If the index jimage is not specified it selects the j
@@ -1748,13 +1723,13 @@ class Lattice(MSONable):
         """
         if jimage is None:
             v, d2 = pbc_shortest_vectors(self, frac_coords1, frac_coords2, return_d2=True)
-            fc = self.get_fractional_coords(v[0][0]) + frac_coords1 - frac_coords2
-            fc = np.array(np.round(fc), dtype=np.int_)
+            fc = self.get_fractional_coords(v[0][0]) + frac_coords1 - frac_coords2  # type: ignore
+            fc = np.array(np.round(fc), dtype=int)
             return np.sqrt(d2[0, 0]), fc
 
         jimage = np.array(jimage)
-        mapped_vec = self.get_cartesian_coords(jimage + frac_coords2 - frac_coords1)
-        return np.linalg.norm(mapped_vec), jimage
+        mapped_vec = self.get_cartesian_coords(jimage + frac_coords2 - frac_coords1)  # type: ignore
+        return np.linalg.norm(mapped_vec), jimage  # type: ignore
 
     def get_miller_index_from_coords(
         self,
@@ -1762,7 +1737,7 @@ class Lattice(MSONable):
         coords_are_cartesian: bool = True,
         round_dp: int = 4,
         verbose: bool = True,
-    ) -> Tuple[int, int, int]:
+    ) -> tuple[int, int, int]:
         """
         Get the Miller index of a plane from a list of site coordinates.
 
@@ -1772,11 +1747,11 @@ class Lattice(MSONable):
 
         Args:
             coords (iterable): A list or numpy array of coordinates. Can be
-                cartesian or fractional coordinates. If more than three sets of
+                Cartesian or fractional coordinates. If more than three sets of
                 coordinates are provided, the best plane that minimises the
                 distance to all sites will be calculated.
             coords_are_cartesian (bool, optional): Whether the coordinates are
-                in cartesian space. If using fractional coordinates set to
+                in Cartesian space. If using fractional coordinates set to
                 False.
             round_dp (int, optional): The number of decimal places to round the
                 miller index to.
@@ -1798,7 +1773,7 @@ class Lattice(MSONable):
         u_norm = vh[2, :]
         return get_integer_index(u_norm, round_dp=round_dp, verbose=verbose)
 
-    def get_recp_symmetry_operation(self, symprec: float = 0.01) -> List:
+    def get_recp_symmetry_operation(self, symprec: float = 0.01) -> list:
         """
         Find the symmetric operations of the reciprocal lattice,
         to be used for hkl transformations
@@ -1825,7 +1800,7 @@ class Lattice(MSONable):
         return recp_symmops
 
 
-def get_integer_index(miller_index: Sequence[float], round_dp: int = 4, verbose: bool = True) -> Tuple[int, int, int]:
+def get_integer_index(miller_index: Sequence[float], round_dp: int = 4, verbose: bool = True) -> tuple[int, int, int]:
     """
     Attempt to convert a vector of floats to whole numbers.
 
@@ -1841,20 +1816,20 @@ def get_integer_index(miller_index: Sequence[float], round_dp: int = 4, verbose:
     mi = np.asarray(miller_index)
     # deal with the case we have small irregular floats
     # that are all equal or factors of each other
-    mi /= min([m for m in mi if m != 0])
+    mi /= min(m for m in mi if m != 0)
     mi /= np.max(np.abs(mi))
 
     # deal with the case we have nice fractions
     md = [Fraction(n).limit_denominator(12).denominator for n in mi]
     mi *= reduce(lambda x, y: x * y, md)
-    int_miller_index = np.int_(np.round(mi, 1))
+    int_miller_index = np.round(mi, 1).astype(int)
     mi /= np.abs(reduce(math.gcd, int_miller_index))
 
     # round to a reasonable precision
     mi = np.array([round(h, round_dp) for h in mi])
 
     # need to recalculate this after rounding as values may have changed
-    int_miller_index = np.int_(np.round(mi, 1))
+    int_miller_index = np.round(mi, 1).astype(int)
     if np.any(np.abs(mi - int_miller_index) > 1e-6) and verbose:
         warnings.warn("Non-integer encountered in Miller index")
     else:
@@ -1881,18 +1856,18 @@ def get_points_in_spheres(
     all_coords: np.ndarray,
     center_coords: np.ndarray,
     r: float,
-    pbc: Union[bool, List[bool]] = True,
+    pbc: bool | list[bool] = True,
     numerical_tol: float = 1e-8,
     lattice: Lattice = None,
     return_fcoords: bool = False,
-) -> List[List[Tuple[np.ndarray, float, int, np.ndarray]]]:
+) -> list[list[tuple[np.ndarray, float, int, np.ndarray]]]:
     """
     For each point in `center_coords`, get all the neighboring points in `all_coords` that are within the
     cutoff radius `r`.
 
     Args:
-        all_coords: (list of cartesian coordinates) all available points
-        center_coords: (list of cartesian coordinates) all centering points
+        all_coords: (list of Cartesian coordinates) all available points
+        center_coords: (list of Cartesian coordinates) all centering points
         r: (float) cutoff radius
         pbc: (bool or a list of bool) whether to set periodic boundaries
         numerical_tol: (float) numerical tolerance
@@ -1903,7 +1878,7 @@ def get_points_in_spheres(
     """
     if isinstance(pbc, bool):
         pbc = [pbc] * 3
-    pbc = np.array(pbc, dtype=np.bool_)  # type: ignore
+    pbc = np.array(pbc, dtype=bool)  # type: ignore
     if return_fcoords and lattice is None:
         raise ValueError("Lattice needs to be supplied to compute fractional coordinates")
     center_coords_min = np.min(center_coords, axis=0)
@@ -1960,7 +1935,7 @@ def get_points_in_spheres(
     else:
         valid_coords = all_coords  # type: ignore
         valid_images = [[0, 0, 0]] * len(valid_coords)
-        valid_indices = np.arange(len(valid_coords))
+        valid_indices = np.arange(len(valid_coords))  # type: ignore
 
     # Divide the valid 3D space into cubes and compute the cube ids
     all_cube_index = _compute_cube_index(valid_coords, global_min, r)  # type: ignore
@@ -1989,10 +1964,10 @@ def get_points_in_spheres(
             neighbors.append([])
             continue
         nn_coords = np.concatenate([cube_to_coords[k] for k in ks], axis=0)
-        nn_images = itertools.chain(*[cube_to_images[k] for k in ks])
-        nn_indices = itertools.chain(*[cube_to_indices[k] for k in ks])
+        nn_images = itertools.chain(*(cube_to_images[k] for k in ks))
+        nn_indices = itertools.chain(*(cube_to_indices[k] for k in ks))
         dist = np.linalg.norm(nn_coords - i[None, :], axis=1)
-        nns: List[Tuple[np.ndarray, float, int, np.ndarray]] = []
+        nns: list[tuple[np.ndarray, float, int, np.ndarray]] = []
         for coord, index, image, d in zip(nn_coords, nn_indices, nn_images, dist):
             # filtering out all sites that are beyond the cutoff
             # Here there is no filtering of overlapping sites
@@ -2045,7 +2020,7 @@ def _three_to_one(label3d: np.ndarray, ny: int, nz: int) -> np.ndarray:
     return np.array(label3d[:, 0] * ny * nz + label3d[:, 1] * nz + label3d[:, 2]).reshape((-1, 1))
 
 
-def find_neighbors(label: np.ndarray, nx: int, ny: int, nz: int) -> List[np.ndarray]:
+def find_neighbors(label: np.ndarray, nx: int, ny: int, nz: int) -> list[np.ndarray]:
     """
     Given a cube index, find the neighbor cube indices
 
