@@ -1,4 +1,3 @@
-# coding: utf-8
 # Copyright (c) Pymatgen Development Team.
 # Distributed under the terms of the MIT License.
 
@@ -131,7 +130,7 @@ class Slab(Structure):
             reconstruction (str): Type of reconstruction. Defaults to None if
                 the slab is not reconstructed.
             coords_are_cartesian (bool): Set to True if you are providing
-                coordinates in cartesian coordinates. Defaults to False.
+                coordinates in Cartesian coordinates. Defaults to False.
             site_properties (dict): Properties associated with the sites as a
                 dict of sequences, e.g., {"magmom":[5,5,5,5]}. The sequences
                 have to be the same length as the atomic species and
@@ -321,7 +320,7 @@ class Slab(Structure):
             # Check for inversion symmetry. Or if sites from surface (a) can be translated
             # to surface (b) along the [hkl]-axis, surfaces are symmetric. Or because the
             # two surfaces of our slabs are always parallel to the (hkl) plane,
-            # any operation where theres an (hkl) mirror plane has surface symmetry
+            # any operation where there's an (hkl) mirror plane has surface symmetry
             return True
         return False
 
@@ -401,7 +400,7 @@ class Slab(Structure):
         mid_pt = np.sum(self.cart_coords, axis=0) / len(self)
         normal = self.normal
         for site in self:
-            charge = sum([getattr(sp, "oxi_state", 0) * amt for sp, amt in site.species.items()])
+            charge = sum(getattr(sp, "oxi_state", 0) * amt for sp, amt in site.species.items())
             dipole += charge * np.dot(site.coords - mid_pt, normal) * normal
         return dipole
 
@@ -461,7 +460,7 @@ class Slab(Structure):
             distance (float): between centers of the adsorbed atom and the
                 given site in Angstroms.
         """
-        # Let's do the work in cartesian coords
+        # Let's do the work in Cartesian coords
         center = np.sum([self[i].coords for i in indices], axis=0) / len(indices)
 
         coords = center + self.normal * distance / np.linalg.norm(self.normal)
@@ -471,18 +470,18 @@ class Slab(Structure):
     def __str__(self):
         comp = self.composition
         outs = [
-            "Slab Summary (%s)" % comp.formula,
-            "Reduced Formula: %s" % comp.reduced_formula,
-            "Miller index: %s" % (self.miller_index,),
-            "Shift: %.4f, Scale Factor: %s" % (self.shift, self.scale_factor.__str__()),
+            f"Slab Summary ({comp.formula})",
+            f"Reduced Formula: {comp.reduced_formula}",
+            f"Miller index: {self.miller_index}",
+            f"Shift: {self.shift:.4f}, Scale Factor: {self.scale_factor.__str__()}",
         ]
 
         def to_s(x):
-            return "%0.6f" % x
+            return f"{x:0.6f}"
 
         outs.append("abc   : " + " ".join([to_s(i).rjust(10) for i in self.lattice.abc]))
         outs.append("angles: " + " ".join([to_s(i).rjust(10) for i in self.lattice.angles]))
-        outs.append("Sites ({i})".format(i=len(self)))
+        outs.append(f"Sites ({len(self)})")
         for i, site in enumerate(self):
             outs.append(
                 " ".join(
@@ -500,8 +499,8 @@ class Slab(Structure):
         :return: MSONAble dict
         """
         d = super().as_dict()
-        d["@module"] = self.__class__.__module__
-        d["@class"] = self.__class__.__name__
+        d["@module"] = type(self).__module__
+        d["@class"] = type(self).__name__
         d["oriented_unit_cell"] = self.oriented_unit_cell.as_dict()
         d["miller_index"] = self.miller_index
         d["shift"] = self.shift
@@ -579,7 +578,7 @@ class Slab(Structure):
             # species, eg. bond distance of each neighbor or neighbor species. The
             # decimal place to get some cn to be equal.
             cn = v.get_cn(ucell, i, use_weights=True)
-            cn = float("%.5f" % (round(cn, 5)))
+            cn = float(f"{round(cn, 5):.5f}")
             if cn not in cn_dict[el]:
                 cn_dict[el].append(cn)
 
@@ -593,7 +592,7 @@ class Slab(Structure):
             try:
                 # A site is a surface site, if its environment does
                 # not fit the environment of other sites
-                cn = float("%.5f" % (round(v.get_cn(self, i, use_weights=True), 5)))
+                cn = float(f"{round(v.get_cn(self, i, use_weights=True), 5):.5f}")
                 if cn < min(cn_dict[site.species_string]):
                     properties.append(True)
                     key = "top" if top else "bottom"
@@ -634,7 +633,7 @@ class Slab(Structure):
         for op in ops:
             slab = self.copy()
             site2 = op.operate(point)
-            if "%.6f" % (site2[2]) == "%.6f" % (point[2]):
+            if f"{site2[2]:.6f}" == f"{point[2]:.6f}":
                 continue
 
             # Add dummy site to check the overall structure is symmetric
@@ -660,7 +659,7 @@ class Slab(Structure):
         Arg:
             specie (str): The specie to add
             point (coords): The coordinate of the site in the slab to add.
-            coords_are_cartesian (bool): Is the point in cartesian coordinates
+            coords_are_cartesian (bool): Is the point in Cartesian coordinates
 
         Returns:
             (Slab): The modified slab
@@ -725,7 +724,7 @@ class Slab(Structure):
 class SlabGenerator:
     """
     This class generates different slabs using shift values determined by where
-    a unique termination can be found along with other criterias such as where a
+    a unique termination can be found along with other criteria such as where a
     termination doesn't break a polyhedral bond. The shift value then indicates
     where the slab layer will begin and terminate in the slab-vacuum system.
 
@@ -845,7 +844,7 @@ class SlabGenerator:
 
         slab_scale_factor = []
         non_orth_ind = []
-        eye = np.eye(3, dtype=np.int_)
+        eye = np.eye(3, dtype=int)
         for i, j in enumerate(miller_index):
             if j == 0:
                 # Lattice vector is perpendicular to surface normal, i.e.,
@@ -863,7 +862,7 @@ class SlabGenerator:
         c_index, dist = max(non_orth_ind, key=lambda t: t[1])
 
         if len(non_orth_ind) > 1:
-            lcm_miller = lcm(*[miller_index[i] for i, d in non_orth_ind])
+            lcm_miller = lcm(*(miller_index[i] for i, d in non_orth_ind))
             for (i, di), (j, dj) in itertools.combinations(non_orth_ind, 2):
                 l = [0, 0, 0]
                 l[i] = -int(round(lcm_miller / miller_index[i]))
@@ -1116,7 +1115,7 @@ class SlabGenerator:
                 specified as a dict of tuples: float of specie1, specie2
                 and the max bonding distance. For example, PO4 groups may be
                 defined as {("P", "O"): 3}.
-            tol (float): General tolerance paramter for getting primitive
+            tol (float): General tolerance parameter for getting primitive
                 cells and matching structures
             ftol (float): Threshold parameter in fcluster in order to check
                 if two atoms are lying on the same plane. Default thresh set
@@ -1416,7 +1415,7 @@ class ReconstructionGenerator:
                         Indicates what kind of structure is this reconstruction.
                     "miller_index" ([h,k,l]): Miller index of your reconstruction
                     "Woods_notation" (str): For a reconstruction, the a and b
-                        lattice may change to accomodate the symmetry of the
+                        lattice may change to accommodate the symmetry of the
                         reconstruction. This notation indicates the change in
                         the vectors relative to the primitive (p) or
                         conventional (c) slab cell. E.g. p(2x1):
@@ -1475,10 +1474,10 @@ class ReconstructionGenerator:
 
         if reconstruction_name not in reconstructions_archive.keys():
             raise KeyError(
-                "The reconstruction_name entered (%s) does not exist in the "
-                "archive. Please select from one of the following reconstructions: %s "
+                f"The reconstruction_name entered ({reconstruction_name}) does not exist in the "
+                f"archive. Please select from one of the following reconstructions: {list(reconstructions_archive)} "
                 "or add the appropriate dictionary to the archive file "
-                "reconstructions_archive.json." % (reconstruction_name, list(reconstructions_archive.keys()))
+                "reconstructions_archive.json."
             )
 
         # Get the instructions to build the reconstruction
@@ -1580,7 +1579,7 @@ def get_d(slab):
     """
     sorted_sites = sorted(slab, key=lambda site: site.frac_coords[2])
     for i, site in enumerate(sorted_sites):
-        if not "%.6f" % (site.frac_coords[2]) == "%.6f" % (sorted_sites[i + 1].frac_coords[2]):
+        if not f"{site.frac_coords[2]:.6f}" == f"{sorted_sites[i + 1].frac_coords[2]:.6f}":
             d = abs(site.frac_coords[2] - sorted_sites[i + 1].frac_coords[2])
             break
     return slab.lattice.get_cartesian_coords([0, 0, d])[2]
@@ -1769,7 +1768,9 @@ def generate_all_slabs(
             specified as a dict of tuples: float of specie1, specie2
             and the max bonding distance. For example, PO4 groups may be
             defined as {("P", "O"): 3}.
-        tol (float): Threshold parameter in fcluster in order to check
+        tol (float): General tolerance parameter for getting primitive
+            cells and matching structures
+        ftol (float): Threshold parameter in fcluster in order to check
             if two atoms are lying on the same plane. Default thresh set
             to 0.1 Angstrom in the direction of the surface normal.
         max_broken_bonds (int): Maximum number of allowable broken bonds
@@ -1828,7 +1829,7 @@ def generate_all_slabs(
         )
 
         if len(slabs) > 0:
-            logger.debug("%s has %d slabs... " % (miller, len(slabs)))
+            logger.debug(f"{miller} has {len(slabs)} slabs... ")
             all_slabs.extend(slabs)
 
     if include_reconstructions:
@@ -1919,11 +1920,11 @@ def miller_index_from_sites(lattice, coords, coords_are_cartesian=True, round_dp
         lattice (list or Lattice): A 3x3 lattice matrix or `Lattice` object (for
             example obtained from Structure.lattice).
         coords (iterable): A list or numpy array of coordinates. Can be
-            cartesian or fractional coordinates. If more than three sets of
+            Cartesian or fractional coordinates. If more than three sets of
             coordinates are provided, the best plane that minimises the
             distance to all sites will be calculated.
         coords_are_cartesian (bool, optional): Whether the coordinates are
-            in cartesian space. If using fractional coordinates set to False.
+            in Cartesian space. If using fractional coordinates set to False.
         round_dp (int, optional): The number of decimal places to round the
             miller index to.
         verbose (bool, optional): Whether to print warnings.
@@ -1971,7 +1972,7 @@ def center_slab(slab):
     """
 
     # get a reasonable r cutoff to sample neighbors
-    bdists = sorted([nn[1] for nn in slab.get_neighbors(slab[0], 10) if nn[1] > 0])
+    bdists = sorted(nn[1] for nn in slab.get_neighbors(slab[0], 10) if nn[1] > 0)
     r = bdists[0] * 3
 
     all_indices = [i for i, site in enumerate(slab)]
