@@ -1,4 +1,3 @@
-# coding: utf-8
 # Copyright (c) Pymatgen Development Team.
 # Distributed under the terms of the MIT License.
 
@@ -10,12 +9,14 @@ structure codes. For example, ComputedEntries can be used as inputs for phase
 diagram analysis.
 """
 
+from __future__ import annotations
+
 import abc
 import json
 import os
 import warnings
 from itertools import combinations
-from typing import List, Union, Dict
+from typing import Literal
 
 import numpy as np
 from monty.json import MontyDecoder, MontyEncoder, MSONable
@@ -98,12 +99,12 @@ class EnergyAdjustment(MSONable):
 
     def __repr__(self):
         output = [
-            "{}:".format(self.__class__.__name__),
-            "  Name: {}".format(self.name),
-            "  Value: {:.3f} eV".format(self.value),
-            "  Uncertainty: {:.3f} eV".format(self.uncertainty),
-            "  Description: {}".format(self.explain),
-            "  Generated by: {}".format(self.cls.get("@class", None)),
+            f"{type(self).__name__}:",
+            f"  Name: {self.name}",
+            f"  Value: {self.value:.3f} eV",
+            f"  Uncertainty: {self.uncertainty:.3f} eV",
+            f"  Description: {self.explain}",
+            f"  Generated by: {self.cls.get('@class', None)}",
         ]
         return "\n".join(output)
 
@@ -141,7 +142,7 @@ class ConstantEnergyAdjustment(EnergyAdjustment):
         """
         Return an explanaion of how the energy adjustment is calculated.
         """
-        return self.description + " ({:.3f} eV)".format(self.value)
+        return self.description + f" ({self.value:.3f} eV)"
 
     def normalize(self, factor):
         """
@@ -221,7 +222,7 @@ class CompositionEnergyAdjustment(EnergyAdjustment):
         """
         Return an explanaion of how the energy adjustment is calculated.
         """
-        return self.description + " ({:.3f} eV/atom x {} atoms)".format(self._adj_per_atom, self.n_atoms)
+        return self.description + f" ({self._adj_per_atom:.3f} eV/atom x {self.n_atoms} atoms)"
 
     def normalize(self, factor):
         """
@@ -304,13 +305,13 @@ class TemperatureEnergyAdjustment(EnergyAdjustment):
 class ComputedEntry(Entry):
     """
     Lightweight Entry object for computed data. Contains facilities
-    for applying corrections to the .energy attribute and for storing
+    for applying corrections to the energy attribute and for storing
     calculation parameters.
     """
 
     def __init__(
         self,
-        composition: Union[Composition, str, Dict[str, float]],
+        composition: Composition | str | dict[str, float],
         energy: float,
         correction: float = 0.0,
         energy_adjustments: list = None,
@@ -389,7 +390,7 @@ class ComputedEntry(Entry):
                 in eV.
         """
         # adds to ufloat(0.0, 0.0) to ensure that no corrections still result in ufloat object
-        corr = ufloat(0.0, 0.0) + sum([ufloat(ea.value, ea.uncertainty) for ea in self.energy_adjustments])
+        corr = ufloat(0.0, 0.0) + sum(ufloat(ea.value, ea.uncertainty) for ea in self.energy_adjustments)
         return corr.nominal_value
 
     @correction.setter
@@ -414,10 +415,8 @@ class ComputedEntry(Entry):
         """
         # adds to ufloat(0.0, 0.0) to ensure that no corrections still result in ufloat object
         unc = ufloat(0.0, 0.0) + sum(
-            [
-                ufloat(ea.value, ea.uncertainty) if not np.isnan(ea.uncertainty) else ufloat(ea.value, 0)
-                for ea in self.energy_adjustments
-            ]
+            ufloat(ea.value, ea.uncertainty) if not np.isnan(ea.uncertainty) else ufloat(ea.value, 0)
+            for ea in self.energy_adjustments
         )
 
         if unc.nominal_value != 0 and unc.std_dev == 0:
@@ -434,14 +433,13 @@ class ComputedEntry(Entry):
         """
         return self.correction_uncertainty / self.composition.num_atoms
 
-    def normalize(self, mode: str = "formula_unit") -> "ComputedEntry":
+    def normalize(self, mode: Literal["formula_unit", "atom"] = "formula_unit") -> ComputedEntry:
         """
         Normalize the entry's composition and energy.
 
         Args:
-            mode: "formula_unit" is the default, which normalizes to
-                composition.reduced_formula. The other option is "atom", which
-                normalizes such that the composition amounts sum to 1.
+            mode ("formula_unit" | "atom"): "formula_unit" (the default) normalizes to composition.reduced_formula.
+                "atom" normalizes such that the composition amounts sum to 1.
         """
 
         factor = self._normalization_factor(mode)
@@ -465,28 +463,28 @@ class ComputedEntry(Entry):
         output = [
             "{} {:<10} - {:<12} ({})".format(
                 self.entry_id,
-                self.__class__.__name__,
+                type(self).__name__,
                 self.composition.formula,
                 self.composition.reduced_formula,
             ),
             "{:<24} = {:<9.4f} eV ({:<8.4f} eV/atom)".format(
                 "Energy (Uncorrected)", self._energy, self._energy / n_atoms
             ),
-            "{:<24} = {:<9.4f} eV ({:<8.4f} eV/atom)".format("Correction", self.correction, self.correction / n_atoms),
-            "{:<24} = {:<9.4f} eV ({:<8.4f} eV/atom)".format("Energy (Final)", self.energy, self.energy_per_atom),
+            f"{'Correction':<24} = {self.correction:<9.4f} eV ({self.correction / n_atoms:<8.4f} eV/atom)",
+            f"{'Energy (Final)':<24} = {self.energy:<9.4f} eV ({self.energy_per_atom:<8.4f} eV/atom)",
             "Energy Adjustments:",
         ]
         if len(self.energy_adjustments) == 0:
             output.append("  None")
         else:
             for e in self.energy_adjustments:
-                output.append("  {:<23}: {:<9.4f} eV ({:<8.4f} eV/atom)".format(e.name, e.value, e.value / n_atoms))
+                output.append(f"  {e.name:<23}: {e.value:<9.4f} eV ({e.value / n_atoms:<8.4f} eV/atom)")
         output.append("Parameters:")
         for k, v in self.parameters.items():
-            output.append("  {:<22} = {}".format(k, v))
+            output.append(f"  {k:<22} = {v}")
         output.append("Data:")
         for k, v in self.data.items():
-            output.append("  {:<22} = {}".format(k, v))
+            output.append(f"  {k:<22} = {v}")
         return "\n".join(output)
 
     def __str__(self):
@@ -516,7 +514,7 @@ class ComputedEntry(Entry):
         return True
 
     @classmethod
-    def from_dict(cls, d) -> "ComputedEntry":
+    def from_dict(cls, d) -> ComputedEntry:
         """
         :param d: Dict representation.
         :return: ComputedEntry
@@ -566,7 +564,7 @@ class ComputedEntry(Entry):
         # NOTE It is assumed that the user will ensure entry_id is a
         # unique identifier for ComputedEntry type classes.
         if self.entry_id is not None:
-            return hash(f"{self.__class__.__name__}{self.entry_id}")
+            return hash(f"{type(self).__name__}{self.entry_id}")
 
         return super().__hash__()
 
@@ -582,7 +580,7 @@ class ComputedStructureEntry(ComputedEntry):
         structure: Structure,
         energy: float,
         correction: float = 0.0,
-        composition: Union[Composition, str, Dict[str, float]] = None,
+        composition: Composition | str | dict[str, float] | None = None,
         energy_adjustments: list = None,
         parameters: dict = None,
         data: dict = None,
@@ -646,7 +644,7 @@ class ComputedStructureEntry(ComputedEntry):
         return d
 
     @classmethod
-    def from_dict(cls, d) -> "ComputedStructureEntry":
+    def from_dict(cls, d) -> ComputedStructureEntry:
         """
         :param d: Dict representation.
         :return: ComputedStructureEntry
@@ -678,23 +676,20 @@ class ComputedStructureEntry(ComputedEntry):
             entry_id=d.get("entry_id", None),
         )
 
-    def normalize(self, mode: str = "formula_unit") -> "ComputedStructureEntry":
+    def normalize(self, mode: Literal["formula_unit", "atom"] = "formula_unit") -> ComputedStructureEntry:
         """
         Normalize the entry's composition and energy. The structure remains
         unchanged.
         Args:
-            mode: "formula_unit" is the default, which normalizes to
-                composition.reduced_formula. The other option is "atom",
-                which normalizes such that the composition amounts sum to 1.
+            mode ("formula_unit" | "atom"): "formula_unit" (the default) normalizes to composition.reduced_formula.
+                "atom" normalizes such that the composition amounts sum to 1.
         """
         # TODO: this should raise TypeError since normalization does not make sense
         # raise TypeError("You cannot normalize a structure.")
         warnings.warn(
-            (
-                f"Normalization of a `{self.__class__.__name__}` makes "
-                "`self.composition` and `self.structure.composition` inconsistent"
-                " - please use self.composition for all further calculations."
-            )
+            f"Normalization of a `{type(self).__name__}` makes "
+            "`self.composition` and `self.structure.composition` inconsistent"
+            " - please use self.composition for all further calculations."
         )
         # TODO: find a better solution for creating copies instead of as/from dict
         factor = self._normalization_factor(mode)
@@ -716,7 +711,7 @@ class GibbsComputedStructureEntry(ComputedStructureEntry):
         structure: Structure,
         formation_enthalpy_per_atom: float,
         temp: float = 300,
-        gibbs_model: str = "SISSO",
+        gibbs_model: Literal["SISSO"] = "SISSO",
         composition: Composition = None,
         correction: float = 0.0,
         energy_adjustments: list = None,
@@ -733,9 +728,9 @@ class GibbsComputedStructureEntry(ComputedStructureEntry):
             temp (float): Temperature in Kelvin. If temperature is not selected from
                 one of [300, 400, 500, ... 2000 K], then free energies will
                 be interpolated. Defaults to 300 K.
-            gibbs_model (str): Model for Gibbs Free energy. Currently the default (and
-                only supported) option is "SISSO", the descriptor created by Bartel et
-                al. (2018) -- see reference in documentation.
+            gibbs_model ('SISSO'): Model for Gibbs Free energy. "SISSO", the descriptor
+                created by Bartel et al. (2018) -- see reference in documentation, is
+                currently the only supported) option.
             correction (float): A correction to be applied to the energy. Defaults to 0
             parameters (dict): An optional dict of parameters associated with
                 the entry. Defaults to None.
@@ -776,7 +771,7 @@ class GibbsComputedStructureEntry(ComputedStructureEntry):
         if gibbs_model.lower() == "sisso":
             self.gibbs_fn = self.gf_sisso
         else:
-            raise ValueError(f"{gibbs_model} not a valid model. The only currently " f"available model is 'SISSO'.")
+            raise ValueError(f"{gibbs_model} not a valid model. The only currently available model is 'SISSO'.")
 
         self._energy = self.gibbs_fn()
 
@@ -849,7 +844,7 @@ class GibbsComputedStructureEntry(ComputedStructureEntry):
                 )
                 sum_g_i += amt * g_interp(self.temp)
         else:
-            sum_g_i = sum([amt * G_ELEMS[str(self.temp)][elem] for elem, amt in elems.items()])
+            sum_g_i = sum(amt * G_ELEMS[str(self.temp)][elem] for elem, amt in elems.items())
 
         return sum_g_i
 
@@ -879,7 +874,7 @@ class GibbsComputedStructureEntry(ComputedStructureEntry):
             alpha_i = pair[0][1]
             alpha_j = pair[1][1]
 
-            mass_sum += (alpha_i + alpha_j) * (m_i * m_j) / (m_i + m_j)  # type: ignore
+            mass_sum += (alpha_i + alpha_j) * (m_i * m_j) / (m_i + m_j)
 
         reduced_mass = (1 / denominator) * mass_sum
 
@@ -908,7 +903,7 @@ class GibbsComputedStructureEntry(ComputedStructureEntry):
         )
 
     @classmethod
-    def from_pd(cls, pd, temp=300, gibbs_model="SISSO") -> List["GibbsComputedStructureEntry"]:
+    def from_pd(cls, pd, temp=300, gibbs_model="SISSO") -> list[GibbsComputedStructureEntry]:
         """
         Constructor method for initializing a list of GibbsComputedStructureEntry
         objects from an existing T = 0 K phase diagram composed of
@@ -943,7 +938,7 @@ class GibbsComputedStructureEntry(ComputedStructureEntry):
         return gibbs_entries
 
     @classmethod
-    def from_entries(cls, entries, temp=300, gibbs_model="SISSO") -> List["GibbsComputedStructureEntry"]:
+    def from_entries(cls, entries, temp=300, gibbs_model="SISSO") -> list[GibbsComputedStructureEntry]:
         """
         Constructor method for initializing GibbsComputedStructureEntry objects from
         T = 0 K ComputedStructureEntry objects, as acquired from a thermochemical
@@ -977,7 +972,7 @@ class GibbsComputedStructureEntry(ComputedStructureEntry):
         return d
 
     @classmethod
-    def from_dict(cls, d) -> "GibbsComputedStructureEntry":
+    def from_dict(cls, d) -> GibbsComputedStructureEntry:
         """
         :param d: Dict representation.
         :return: GibbsComputedStructureEntry
@@ -998,7 +993,7 @@ class GibbsComputedStructureEntry(ComputedStructureEntry):
 
     def __repr__(self):
         output = [
-            "GibbsComputedStructureEntry {} - {}".format(self.entry_id, self.composition.formula),
-            "Gibbs Free Energy (Formation) = {:.4f}".format(self.energy),
+            f"GibbsComputedStructureEntry {self.entry_id} - {self.composition.formula}",
+            f"Gibbs Free Energy (Formation) = {self.energy:.4f}",
         ]
         return "\n".join(output)
