@@ -1,13 +1,12 @@
-# coding: utf-8
 # Copyright (c) Pymatgen Development Team.
 # Distributed under the terms of the MIT License.
 
-
 """
 This module implements functions to perform various useful operations on
 entries, such as grouping entries by structure.
 """
 
+from __future__ import annotations
 
 import collections
 import csv
@@ -16,7 +15,7 @@ import itertools
 import json
 import logging
 import re
-from typing import Iterable, List, Set, Union
+from typing import Iterable, List, Literal
 
 from monty.json import MontyDecoder, MontyEncoder, MSONable
 from monty.string import unicode2str
@@ -25,6 +24,7 @@ from pymatgen.analysis.phase_diagram import PDEntry
 from pymatgen.analysis.structure_matcher import SpeciesComparator, StructureMatcher
 from pymatgen.core.composition import Composition
 from pymatgen.core.periodic_table import Element
+from pymatgen.entries import Entry
 from pymatgen.entries.computed_entries import ComputedEntry, ComputedStructureEntry
 
 logger = logging.getLogger(__name__)
@@ -56,15 +56,15 @@ def _perform_grouping(args):
     unmatched = list(zip(entries, hosts))
     while len(unmatched) > 0:
         ref_host = unmatched[0][1]
-        logger.info("Reference tid = {}, formula = {}".format(unmatched[0][0].entry_id, ref_host.formula))
+        logger.info(f"Reference tid = {unmatched[0][0].entry_id}, formula = {ref_host.formula}")
         ref_formula = ref_host.composition.reduced_formula
-        logger.info("Reference host = {}".format(ref_formula))
+        logger.info(f"Reference host = {ref_formula}")
         matches = [unmatched[0]]
         for i in range(1, len(unmatched)):
             test_host = unmatched[i][1]
-            logger.info("Testing tid = {}, formula = {}".format(unmatched[i][0].entry_id, test_host.formula))
+            logger.info(f"Testing tid = {unmatched[i][0].entry_id}, formula = {test_host.formula}")
             test_formula = test_host.composition.reduced_formula
-            logger.info("Test host = {}".format(test_formula))
+            logger.info(f"Test host = {test_formula}")
             m = StructureMatcher(
                 ltol=ltol,
                 stol=stol,
@@ -78,7 +78,7 @@ def _perform_grouping(args):
                 matches.append(unmatched[i])
         groups.append(json.dumps([m[0] for m in matches], cls=MontyEncoder))
         unmatched = list(filter(lambda x: x not in matches, unmatched))
-        logger.info("{} unmatched remaining".format(len(unmatched)))
+        logger.info(f"{len(unmatched)} unmatched remaining")
 
 
 def group_entries_by_structure(
@@ -119,7 +119,7 @@ def group_entries_by_structure(
         [[ entry1, entry2], [entry3, entry4, entry5]]
     """
     start = datetime.datetime.now()
-    logger.info("Started at {}".format(start))
+    logger.info(f"Started at {start}")
     entries_host = [(entry, _get_host(entry.structure, species_to_remove)) for entry in entries]
     if ncpus:
         symm_entries = collections.defaultdict(list)
@@ -127,7 +127,7 @@ def group_entries_by_structure(
             symm_entries[comparator.get_structure_hash(host)].append((entry, host))
         import multiprocessing as mp
 
-        logging.info("Using {} cpus".format(ncpus))
+        logging.info(f"Using {ncpus} cpus")
         manager = mp.Manager()
         groups = manager.list()
         with mp.Pool(ncpus) as p:
@@ -168,8 +168,34 @@ def group_entries_by_structure(
     entry_groups = []
     for g in groups:
         entry_groups.append(json.loads(g, cls=MontyDecoder))
-    logging.info("Finished at {}".format(datetime.datetime.now()))
-    logging.info("Took {}".format(datetime.datetime.now() - start))
+    logging.info(f"Finished at {datetime.datetime.now()}")
+    logging.info(f"Took {datetime.datetime.now() - start}")
+    return entry_groups
+
+
+def group_entries_by_composition(entries, sort_by_e_per_atom=True):
+    """
+    Given a sequence of Entry-like objects, group them by composition and
+        optionally sort by energy above hull.
+
+    Args:
+        entries (List): Sequence of Entry-like objects.
+        sort_by_e_per_atom (bool): Whether to sort the grouped entries by
+            energy per atom (lowest energy first). Default True.
+
+    Returns:
+        Sequence of sequence of entries by composition. e.g,
+        [[ entry1, entry2], [entry3, entry4, entry5]]
+    """
+    entry_groups = []
+    entries = sorted(entries, key=lambda e: e.composition.reduced_formula)
+    for _, g in itertools.groupby(entries, key=lambda e: e.composition.reduced_formula):
+        group = list(g)
+        if sort_by_e_per_atom:
+            group = sorted(group, key=lambda e: e.energy_per_atom)
+
+        entry_groups.append(group)
+
     return entry_groups
 
 
@@ -179,7 +205,7 @@ class EntrySet(collections.abc.MutableSet, MSONable):
     subsets, dumping into files, etc.
     """
 
-    def __init__(self, entries: Iterable[Union[PDEntry, ComputedEntry, ComputedStructureEntry]]):
+    def __init__(self, entries: Iterable[PDEntry | ComputedEntry | ComputedStructureEntry]):
         """
         Args:
             entries: All the entries.
@@ -222,18 +248,32 @@ class EntrySet(collections.abc.MutableSet, MSONable):
             chemsys.update([el.symbol for el in e.composition.keys()])
         return chemsys
 
-    def remove_non_ground_states(self):
+    @property
+    def ground_states(self) -> set:
         """
-        Removes all non-ground state entries, i.e., only keep the lowest energy
+        A set containing only the entries that are ground states, i.e., the lowest energy
         per atom entry at each composition.
         """
         entries = sorted(self.entries, key=lambda e: e.composition.reduced_formula)
         ground_states = set()
         for _, g in itertools.groupby(entries, key=lambda e: e.composition.reduced_formula):
             ground_states.add(min(g, key=lambda e: e.energy_per_atom))
-        self.entries = ground_states
+        return ground_states
 
-    def get_subset_in_chemsys(self, chemsys: List[str]):
+    def remove_non_ground_states(self):
+        """
+        Removes all non-ground state entries, i.e., only keep the lowest energy
+        per atom entry at each composition.
+        """
+        self.entries = self.ground_states
+
+    def is_ground_state(self, entry) -> bool:
+        """
+        Boolean indicating whether a given Entry is a ground state
+        """
+        return entry in self.ground_states
+
+    def get_subset_in_chemsys(self, chemsys: list[str]):
         """
         Returns an EntrySet containing only the set of entries belonging to
         a particular chemical system (in this definition, it includes all sub
@@ -250,7 +290,7 @@ class EntrySet(collections.abc.MutableSet, MSONable):
         """
         chem_sys = set(chemsys)
         if not chem_sys.issubset(self.chemsys):
-            raise ValueError("%s is not a subset of %s" % (chem_sys, self.chemsys))
+            raise ValueError(f"{chem_sys} is not a subset of {self.chemsys}")
         subset = set()
         for e in self.entries:
             elements = [sp.symbol for sp in e.composition.keys()]
@@ -258,13 +298,13 @@ class EntrySet(collections.abc.MutableSet, MSONable):
                 subset.add(e)
         return EntrySet(subset)
 
-    def as_dict(self):
+    def as_dict(self) -> dict[Literal["entries"], list[Entry]]:
         """
-        :return: MSONable dict
+        Returns MSONable dict.
         """
         return {"entries": list(self.entries)}
 
-    def to_csv(self, filename: str, latexify_names: bool = False):
+    def to_csv(self, filename: str, latexify_names: bool = False) -> None:
         """
         Exports PDEntries to a csv
 
@@ -275,7 +315,7 @@ class EntrySet(collections.abc.MutableSet, MSONable):
                 e.g., Li_{2}O
         """
 
-        els = set()  # type: Set[Element]
+        els: set[Element] = set()
         for entry in self.entries:
             els.update(entry.composition.elements)
         elements = sorted(list(els), key=lambda a: a.X)
@@ -288,8 +328,8 @@ class EntrySet(collections.abc.MutableSet, MSONable):
             )
             writer.writerow(["Name"] + [el.symbol for el in elements] + ["Energy"])
             for entry in self.entries:
-                row = [entry.name if not latexify_names else re.sub(r"([0-9]+)", r"_{\1}", entry.name)]
-                row.extend([entry.composition[el] for el in elements])
+                row: list[str] = [entry.name if not latexify_names else re.sub(r"([0-9]+)", r"_{\1}", entry.name)]
+                row.extend([str(entry.composition[el]) for el in elements])
                 row.append(str(entry.energy))
                 writer.writerow(row)
 
@@ -304,7 +344,7 @@ class EntrySet(collections.abc.MutableSet, MSONable):
         Returns:
             List of Elements, List of PDEntries
         """
-        with open(filename, "r", encoding="utf-8") as f:
+        with open(filename, encoding="utf-8") as f:
             reader = csv.reader(
                 f,
                 delimiter=unicode2str(","),
