@@ -1,4 +1,3 @@
-# coding: utf-8
 # Copyright (c) Pymatgen Development Team.
 # Distributed under the terms of the MIT License.
 
@@ -12,22 +11,27 @@ Materials Project, and obtain an API key by going to your dashboard at
 https://www.materialsproject.org/dashboard.
 """
 
+from __future__ import annotations
+
 import itertools
 import json
 import logging
+import math
+import os
 import platform
 import re
 import sys
 import warnings
-from collections import defaultdict
 from enum import Enum, unique
 from time import sleep
 
 import requests
 from monty.json import MontyDecoder, MontyEncoder
-from monty.serialization import dumpfn
+from ruamel.yaml import YAML
+from tqdm import tqdm
 
-from pymatgen.core import SETTINGS, SETTINGS_FILE, yaml
+from pymatgen.core import SETTINGS
+from pymatgen.core import __version__ as PMG_VERSION
 from pymatgen.core.composition import Composition
 from pymatgen.core.periodic_table import Element
 from pymatgen.core.structure import Structure
@@ -35,12 +39,23 @@ from pymatgen.core.surface import get_symmetrically_equivalent_miller_indices
 from pymatgen.entries.computed_entries import ComputedEntry, ComputedStructureEntry
 from pymatgen.entries.exp_entries import ExpEntry
 from pymatgen.symmetry.analyzer import SpacegroupAnalyzer
-from pymatgen.util.sequence import PBar, get_chunks
-from pymatgen.core import __version__ as PMG_VERSION
-
 
 logger = logging.getLogger(__name__)
 
+MP_LOG_FILE = os.path.join(os.path.expanduser("~"), ".mprester.log.yaml")
+
+
+def get_chunks(sequence, size=1):
+    """
+    Args:
+        sequence ():
+        size ():
+
+    Returns:
+    """
+    chunks = int(math.ceil(len(sequence) / float(size)))
+    return [sequence[i * size : (i + 1) * size] for i in range(chunks)]
+
 
 @unique
 class TaskType(Enum):
@@ -170,7 +185,7 @@ class MPRester:
             self.preamble = SETTINGS.get("PMG_MAPI_ENDPOINT", "https://materialsproject.org/rest/v2")
 
         if self.preamble != "https://materialsproject.org/rest/v2":
-            warnings.warn("Non-default endpoint used: {}".format(self.preamble))
+            warnings.warn(f"Non-default endpoint used: {self.preamble}")
 
         self.session = requests.Session()
         self.session.headers = {"x-api-key": self.api_key}
@@ -179,23 +194,31 @@ class MPRester:
             python_info = "Python/{}.{}.{}".format(
                 sys.version_info.major, sys.version_info.minor, sys.version_info.micro
             )
-            platform_info = "{}/{}".format(platform.system(), platform.release())
-            self.session.headers["user-agent"] = "{} ({} {})".format(pymatgen_info, python_info, platform_info)
+            platform_info = f"{platform.system()}/{platform.release()}"
+            self.session.headers["user-agent"] = f"{pymatgen_info} ({python_info} {platform_info})"
 
         if notify_db_version:
+            yaml = YAML()
             db_version = self.get_database_version()
             logger.debug(f"Connection established to Materials Project database, version {db_version}.")
 
             try:
-                with open(SETTINGS_FILE, "rt") as f:
-                    d = yaml.safe_load(f)
-            except IOError:
+                with open(MP_LOG_FILE) as f:
+                    d = dict(yaml.load(f))
+            except OSError:
                 d = {}
 
             d = d if d else {}
 
             if "MAPI_DB_VERSION" not in d:
                 d["MAPI_DB_VERSION"] = {"LOG": {}, "LAST_ACCESSED": None}
+            else:
+                # ensure data is parsed as dict, rather than ordered dict,
+                # due to change in YAML parsing behavior
+                d["MAPI_DB_VERSION"] = dict(d["MAPI_DB_VERSION"])
+
+            if "LOG" in d["MAPI_DB_VERSION"]:
+                d["MAPI_DB_VERSION"]["LOG"] = dict(d["MAPI_DB_VERSION"]["LOG"])
 
             # store a log of what database versions are being connected to
             if db_version not in d["MAPI_DB_VERSION"]["LOG"]:
@@ -216,7 +239,8 @@ class MPRester:
             # bare except is not ideal (perhaps a PermissionError, etc.) but this is not critical
             # and should be allowed to fail regardless of reason
             try:
-                dumpfn(d, SETTINGS_FILE)
+                with open(MP_LOG_FILE, "wt") as f:
+                    yaml.dump(d, f)
             except Exception:
                 pass
 
@@ -251,10 +275,10 @@ class MPRester:
                     return data["response"]
                 raise MPRestError(data["error"])
 
-            raise MPRestError("REST query returned with error status code {}".format(response.status_code))
+            raise MPRestError(f"REST query returned with error status code {response.status_code}")
 
         except Exception as ex:
-            msg = "{}. Content: {}".format(str(ex), response.content) if hasattr(response, "content") else str(ex)
+            msg = f"{ex}. Content: {response.content}" if hasattr(response, "content") else str(ex)
             raise MPRestError(msg)
 
     def get_database_version(self):
@@ -285,7 +309,7 @@ class MPRester:
         Returns:
             materials_id (str)
         """
-        return self._make_request("/materials/mid_from_tid/%s" % task_id)
+        return self._make_request(f"/materials/mid_from_tid/{task_id}")
 
     def get_materials_id_references(self, material_id):
         """
@@ -297,7 +321,7 @@ class MPRester:
         Returns:
             BibTeX (str)
         """
-        return self._make_request("/materials/%s/refs" % material_id)
+        return self._make_request(f"/materials/{material_id}/refs")
 
     def get_data(self, chemsys_formula_id, data_type="vasp", prop=""):
         """
@@ -322,7 +346,7 @@ class MPRester:
                 MPRester.supported_task_properties. Leave as empty string for a
                 general list of useful properties.
         """
-        sub_url = "/materials/%s/%s" % (chemsys_formula_id, data_type)
+        sub_url = f"/materials/{chemsys_formula_id}/{data_type}"
         if prop:
             sub_url += "/" + prop
         return self._make_request(sub_url)
@@ -338,7 +362,7 @@ class MPRester:
         Returns:
             ([str]) List of all materials ids.
         """
-        return self._make_request("/materials/%s/mids" % chemsys_formula, mp_decode=False)
+        return self._make_request(f"/materials/{chemsys_formula}/mids", mp_decode=False)
 
     def get_doc(self, materials_id):
         """
@@ -353,7 +377,7 @@ class MPRester:
             Dict of json document of all data that is displayed on a materials
             details page.
         """
-        return self._make_request("/materials/%s/doc" % materials_id, mp_decode=False)
+        return self._make_request(f"/materials/{materials_id}/doc", mp_decode=False)
 
     def get_xas_data(self, material_id, absorbing_element):
         """
@@ -376,7 +400,7 @@ class MPRester:
                 "mp_id: {}".format(absorbing_element, material_id)
             )
         data = self._make_request(
-            "/materials/{}/xas/{}".format(material_id, absorbing_element),
+            f"/materials/{material_id}/xas/{absorbing_element}",
             mp_decode=False,
         )
         return data[0]
@@ -400,7 +424,7 @@ class MPRester:
                 MPRester.supported_properties. Leave as empty string for a
                 general list of useful properties.
         """
-        sub_url = "/tasks/%s" % chemsys_formula_id
+        sub_url = f"/tasks/{chemsys_formula_id}"
         if prop:
             sub_url += "/" + prop
         return self._make_request(sub_url)
@@ -444,13 +468,13 @@ class MPRester:
             else:
                 raise MPRestError("Provide filename or Structure object.")
             payload = {"structure": json.dumps(s.as_dict(), cls=MontyEncoder)}
-            response = self.session.post("{}/find_structure".format(self.preamble), data=payload)
+            response = self.session.post(f"{self.preamble}/find_structure", data=payload)
             if response.status_code in [200, 400]:
                 resp = json.loads(response.text, cls=MontyDecoder)
                 if resp["valid_response"]:
                     return resp["response"]
                 raise MPRestError(resp["error"])
-            raise MPRestError("REST error with status code {} and error {}".format(response.status_code, response.text))
+            raise MPRestError(f"REST error with status code {response.status_code} and error {response.text}")
         except Exception as ex:
             raise MPRestError(str(ex))
 
@@ -526,7 +550,7 @@ class MPRester:
         entries = []
         for d in data:
             d["potcar_symbols"] = [
-                "%s %s" % (d["pseudo_potential"]["functional"], l) for l in d["pseudo_potential"]["labels"]
+                f"{d['pseudo_potential']['functional']} {l}" for l in d["pseudo_potential"]["labels"]
             ]
             data = {"oxide_type": d["oxide_type"]}
             if property_data:
@@ -570,13 +594,13 @@ class MPRester:
     def get_pourbaix_entries(self, chemsys, solid_compat="MaterialsProject2020Compatibility"):
         """
         A helper function to get all entries necessary to generate
-        a pourbaix diagram from the rest interface.
+        a Pourbaix diagram from the rest interface.
 
         Args:
             chemsys (str or [str]): Chemical system string comprising element
                 symbols separated by dashes, e.g., "Li-Fe-O" or List of element
                 symbols, e.g., ["Li", "Fe", "O"].
-            solid_compat: Compatiblity scheme used to pre-process solid DFT energies prior to applying aqueous
+            solid_compat: Compatibility scheme used to pre-process solid DFT energies prior to applying aqueous
                 energy adjustments. May be passed as a class (e.g. MaterialsProject2020Compatibility) or an instance
                 (e.g., MaterialsProject2020Compatibility()). If None, solid DFT energies are used as-is.
                 Default: MaterialsProject2020Compatibility
@@ -588,8 +612,8 @@ class MPRester:
         from pymatgen.core.ion import Ion
         from pymatgen.entries.compatibility import (
             Compatibility,
-            MaterialsProjectAqueousCompatibility,
             MaterialsProject2020Compatibility,
+            MaterialsProjectAqueousCompatibility,
             MaterialsProjectCompatibility,
         )
 
@@ -602,7 +626,7 @@ class MPRester:
         else:
             raise ValueError(
                 "Solid compatibility can only be 'MaterialsProjectCompatibility', "
-                "'MaterialsProject2020Compatibility', or an instance of a Compatability class"
+                "'MaterialsProject2020Compatibility', or an instance of a Compatibility class"
             )
 
         pbx_entries = []
@@ -644,14 +668,16 @@ class MPRester:
                 raise ValueError("Reference solid not contained in entry list")
             stable_ref = sorted(refs, key=lambda x: x.data["e_above_hull"])[0]
             rf = stable_ref.composition.get_reduced_composition_and_factor()[1]
+
             solid_diff = ion_ref_pd.get_form_energy(stable_ref) - i_d["Reference solid energy"] * rf
             elt = i_d["Major_Elements"][0]
             correction_factor = ion.composition[elt] / stable_ref.composition[elt]
             energy = i_d["Energy"] + solid_diff * correction_factor
             ion_entry = IonEntry(ion, energy)
-            pbx_entries.append(PourbaixEntry(ion_entry, "ion-{}".format(n)))
+            pbx_entries.append(PourbaixEntry(ion_entry, f"ion-{n}"))
+
+        # Construct the solid Pourbaix entries from filtered ion_ref entries
 
-        # Construct the solid pourbaix entries from filtered ion_ref entries
         extra_elts = set(ion_ref_elts) - {Element(s) for s in chemsys} - {Element("H"), Element("O")}
         for entry in ion_ref_entries:
             entry_elts = set(entry.composition.elements)
@@ -795,7 +821,7 @@ class MPRester:
         Returns:
             CompletePhononDos: A phonon DOS object.
         """
-        return self._make_request("/materials/{}/phonondos".format(material_id))
+        return self._make_request(f"/materials/{material_id}/phonondos")
 
     def get_phonon_bandstructure_by_material_id(self, material_id):
         """
@@ -807,7 +833,7 @@ class MPRester:
         Returns:
             PhononBandStructureSymmLine: A phonon band structure.
         """
-        return self._make_request("/materials/{}/phononbs".format(material_id))
+        return self._make_request(f"/materials/{material_id}/phononbs")
 
     def get_phonon_ddb_by_material_id(self, material_id):
         """
@@ -819,7 +845,7 @@ class MPRester:
         Returns:
             str: ABINIT DDB file as a string.
         """
-        return self._make_request("/materials/{}/abinit_ddb".format(material_id))
+        return self._make_request(f"/materials/{material_id}/abinit_ddb")
 
     def get_entries_in_chemsys(
         self,
@@ -828,13 +854,12 @@ class MPRester:
         inc_structure=None,
         property_data=None,
         conventional_unit_cell=False,
+        additional_criteria=None,
     ):
         """
-        Helper method to get a list of ComputedEntries in a chemical system.
-        For example, elements = ["Li", "Fe", "O"] will return a list of all
-        entries in the Li-Fe-O chemical system, i.e., all LixOy,
-        FexOy, LixFey, LixFeyOz, Li, Fe and O phases. Extremely useful for
-        creating phase diagrams of entire chemical systems.
+        Helper method to get a list of ComputedEntries in a chemical system. For example, elements = ["Li", "Fe", "O"]
+        will return a list of all entries in the Li-Fe-O chemical system, i.e., all LixOy, FexOy, LixFey, LixFeyOz,
+        Li, Fe and O phases. Extremely useful for creating phase diagrams of entire chemical systems.
 
         Args:
             elements (str or [str]): Chemical system string comprising element
@@ -856,6 +881,8 @@ class MPRester:
                 supported_properties.
             conventional_unit_cell (bool): Whether to get the standard
                 conventional unit cell
+            additional_criteria (dict): Any additional criteria to pass. For instance, if you are only interested in
+                stable entries, you can pass {"e_above_hull": {"$lte": 0.001}}.
 
         Returns:
             List of ComputedEntries.
@@ -869,8 +896,12 @@ class MPRester:
             for els in itertools.combinations(elements, i + 1):
                 all_chemsyses.append("-".join(sorted(els)))
 
+        criteria = {"chemsys": {"$in": all_chemsyses}}
+        if additional_criteria:
+            criteria.update(additional_criteria)
+
         entries = self.get_entries(
-            {"chemsys": {"$in": all_chemsyses}},
+            criteria,
             compatible_only=compatible_only,
             inc_structure=inc_structure,
             property_data=property_data,
@@ -909,12 +940,12 @@ class MPRester:
         self,
         criteria,
         properties,
-        chunk_size=500,
-        max_tries_per_chunk=5,
-        mp_decode=True,
+        chunk_size: int = 500,
+        max_tries_per_chunk: int = 5,
+        mp_decode: bool = True,
+        show_progress_bar: bool = True,
     ):
         r"""
-
         Performs an advanced query using MongoDB-like syntax for directly
         querying the Materials Project database. This allows one to perform
         queries which are otherwise too cumbersome to perform using the standard
@@ -971,6 +1002,8 @@ class MPRester:
             mp_decode (bool): Whether to do a decoding to a Pymatgen object
                 where possible. In some cases, it might be useful to just get
                 the raw python dict, i.e., set to False.
+            show_progress_bar (bool): Whether to show a progress bar for large queries.
+                Defaults to True. Set to False to reduce visual noise.
 
         Returns:
             List of results. E.g.,
@@ -997,7 +1030,7 @@ class MPRester:
         data = []
         mids = [d["material_id"] for d in self.query(criteria, ["material_id"], chunk_size=0)]
         chunks = get_chunks(mids, size=chunk_size)
-        progress_bar = PBar(total=len(mids))
+        progress_bar = tqdm(total=len(mids), disable=not show_progress_bar)
         for chunk in chunks:
             chunk_criteria = criteria.copy()
             chunk_criteria.update({"material_id": {"$in": chunk}})
@@ -1111,7 +1144,7 @@ class MPRester:
             snl = snl if isinstance(snl, list) else [snl]
             jsondata = [s.as_dict() for s in snl]
             payload = {"snl": json.dumps(jsondata, cls=MontyEncoder)}
-            response = self.session.post("{}/snl/submit".format(self.preamble), data=payload)
+            response = self.session.post(f"{self.preamble}/snl/submit", data=payload)
             if response.status_code in [200, 400]:
                 resp = json.loads(response.text, cls=MontyDecoder)
                 if resp["valid_response"]:
@@ -1120,7 +1153,7 @@ class MPRester:
                     return resp["inserted_ids"]
                 raise MPRestError(resp["error"])
 
-            raise MPRestError("REST error with status code {} and error {}".format(response.status_code, response.text))
+            raise MPRestError(f"REST error with status code {response.status_code} and error {response.text}")
 
         except Exception as ex:
             raise MPRestError(str(ex))
@@ -1143,7 +1176,7 @@ class MPRester:
         """
         try:
             payload = {"ids": json.dumps(snl_ids)}
-            response = self.session.post("{}/snl/delete".format(self.preamble), data=payload)
+            response = self.session.post(f"{self.preamble}/snl/delete", data=payload)
 
             if response.status_code in [200, 400]:
                 resp = json.loads(response.text, cls=MontyDecoder)
@@ -1153,7 +1186,7 @@ class MPRester:
                     return resp
                 raise MPRestError(resp["error"])
 
-            raise MPRestError("REST error with status code {} and error {}".format(response.status_code, response.text))
+            raise MPRestError(f"REST error with status code {response.status_code} and error {response.text}")
 
         except Exception as ex:
             raise MPRestError(str(ex))
@@ -1179,7 +1212,7 @@ class MPRester:
         """
         try:
             payload = {"criteria": json.dumps(criteria)}
-            response = self.session.post("{}/snl/query".format(self.preamble), data=payload)
+            response = self.session.post(f"{self.preamble}/snl/query", data=payload)
             if response.status_code in [200, 400]:
                 resp = json.loads(response.text)
                 if resp["valid_response"]:
@@ -1188,7 +1221,7 @@ class MPRester:
                     return resp["response"]
                 raise MPRestError(resp["error"])
 
-            raise MPRestError("REST error with status code {} and error {}".format(response.status_code, response.text))
+            raise MPRestError(f"REST error with status code {response.status_code} and error {response.text}")
 
         except Exception as ex:
             raise MPRestError(str(ex))
@@ -1284,7 +1317,7 @@ class MPRester:
         try:
             payload = {"entries": json.dumps(entries, cls=MontyEncoder)}
             response = self.session.post(
-                "{}/phase_diagram/calculate_stability".format(self.preamble),
+                f"{self.preamble}/phase_diagram/calculate_stability",
                 data=payload,
             )
             if response.status_code in [200, 400]:
@@ -1294,7 +1327,7 @@ class MPRester:
                         warnings.warn(resp["warning"])
                     return resp["response"]
                 raise MPRestError(resp["error"])
-            raise MPRestError("REST error with status code {} and error {}".format(response.status_code, response.text))
+            raise MPRestError(f"REST error with status code {response.status_code} and error {response.text}")
         except Exception as ex:
             raise MPRestError(str(ex))
 
@@ -1315,7 +1348,7 @@ class MPRester:
 
         isolated_atom_e_sum, n = 0, 0
         for el in comp_dict.keys():
-            e = self._make_request("/element/%s/tasks/isolated_atom" % (el), mp_decode=False)[0]
+            e = self._make_request(f"/element/{el}/tasks/isolated_atom", mp_decode=False)[0]
             isolated_atom_e_sum += e["output"]["final_energy_per_atom"] * comp_dict[el]
             n += comp_dict[el]
         ecoh_per_formula = isolated_atom_e_sum - ebulk
@@ -1353,9 +1386,9 @@ class MPRester:
         Returns:
             list of dicts with substrate matches
         """
-        req = "/materials/{}/substrates?n={}".format(material_id, number)
+        req = f"/materials/{material_id}/substrates?n={number}"
         if orient:
-            req += "&orient={}".format(" ".join(map(str, orient)))
+            req += f"&orient={' '.join(map(str, orient))}"
         return self._make_request(req)
 
     def get_all_substrates(self):
@@ -1378,7 +1411,7 @@ class MPRester:
         Tran, R., Xu, Z., Radhakrishnan, B., Winston, D., Sun, W., Persson, K.
         A., & Ong, S. P. (2016). Data Descripter: Surface energies of elemental
         crystals. Scientific Data, 3(160080), 1–13.
-        http://dx.doi.org/10.1038/sdata.2016.80
+        https://doi.org/10.1038/sdata.2016.80
 
         Args:
             material_id (str): Materials Project material_id, e.g. 'mp-123'.
@@ -1390,7 +1423,7 @@ class MPRester:
         Returns:
             Surface data for material. Energies are given in SI units (J/m^2).
         """
-        req = "/materials/{}/surfaces".format(material_id)
+        req = f"/materials/{material_id}/surfaces"
         if inc_structures:
             req += "?include_structures=true"
 
@@ -1551,32 +1584,73 @@ class MPRester:
         # task_id's correspond to NoMaD external_id's
         task_types = [t.value for t in task_types if isinstance(t, TaskType)] if task_types else []
 
-        meta = defaultdict(list)
-        for doc in self.query({"material_id": {"$in": material_ids}}, ["material_id", "blessed_tasks"]):
-
+        meta = {}
+        for doc in self.query({"task_id": {"$in": material_ids}}, ["task_id", "blessed_tasks"]):
             for task_type, task_id in doc["blessed_tasks"].items():
                 if task_types and task_type not in task_types:
                     continue
-                meta[doc["material_id"]].append({"task_id": task_id, "task_type": task_type})
-
+                mp_id = doc["task_id"]
+                if meta.get(mp_id) is None:
+                    meta[mp_id] = [{"task_id": task_id, "task_type": task_type}]
+                else:
+                    meta[mp_id].append({"task_id": task_id, "task_type": task_type})
         if not meta:
-            raise ValueError("No tasks found.")
+            raise ValueError(f"No tasks found for material id {material_ids}.")
 
         # return a list of URLs for NoMaD Downloads containing the list of files
         # for every external_id in `task_ids`
-        prefix = "http://labdev-nomad.esc.rzg.mpg.de/fairdi/nomad/mp/api/raw/query?"
+        # For reference, please visit https://nomad-lab.eu/prod/rae/api/
+
+        # check if these task ids exist on NOMAD
+        prefix = "https://nomad-lab.eu/prod/rae/api/repo/?"
         if file_patterns is not None:
             for file_pattern in file_patterns:
                 prefix += f"file_pattern={file_pattern}&"
         prefix += "external_id="
 
-        # NOTE: IE has 2kb URL char limit
-        nmax = int((2000 - len(prefix)) / 11)  # mp-<7-digit> + , = 11
         task_ids = [t["task_id"] for tl in meta.values() for t in tl]
-        chunks = get_chunks(task_ids, size=nmax)
-        urls = [prefix + ",".join(tids) for tids in chunks]
+        nomad_exist_task_ids = self._check_get_download_info_url_by_task_id(prefix=prefix, task_ids=task_ids)
+        if len(nomad_exist_task_ids) != len(task_ids):
+            self._print_help_message(nomad_exist_task_ids, task_ids, file_patterns, task_types)
+
+        # generate download links for those that exist
+        prefix = "https://nomad-lab.eu/prod/rae/api/raw/query?"
+        if file_patterns is not None:
+            for file_pattern in file_patterns:
+                prefix += f"file_pattern={file_pattern}&"
+        prefix += "external_id="
+
+        urls = [prefix + tids for tids in nomad_exist_task_ids]
         return meta, urls
 
+    @staticmethod
+    def _print_help_message(nomad_exist_task_ids, task_ids, file_patterns, task_types):
+        non_exist_ids = set(task_ids) - set(nomad_exist_task_ids)
+        warnings.warn(
+            f"For file patterns [{file_patterns}] and task_types [{task_types}], \n"
+            f"the following ids are not found on NOMAD [{list(non_exist_ids)}]. \n"
+            f"If you need to upload them, please contact Patrick Huck at phuck@lbl.gov"
+        )
+
+    def _check_get_download_info_url_by_task_id(self, prefix, task_ids) -> list[str]:
+        nomad_exist_task_ids: list[str] = []
+        prefix = prefix.replace("/raw/query", "/repo/")
+        for task_id in task_ids:
+            url = prefix + task_id
+            if self._check_nomad_exist(url):
+                nomad_exist_task_ids.append(task_id)
+        return nomad_exist_task_ids
+
+    @staticmethod
+    def _check_nomad_exist(url) -> bool:
+        response = requests.get(url=url)
+        if response.status_code != 200:
+            return False
+        content = json.loads(response.text)
+        if content["pagination"]["total"] == 0:
+            return False
+        return True
+
     @staticmethod
     def parse_criteria(criteria_string):
         """
@@ -1654,5 +1728,3 @@ class MPRestError(Exception):
     Exception class for MPRestAdaptor.
     Raised when the query has problems, e.g., bad query format.
     """
-
-    pass
