@@ -2,20 +2,23 @@
 Optimade support.
 """
 
+from __future__ import annotations
+
 import logging
 import sys
 from collections import namedtuple
 from os.path import join
-from typing import Dict, Union, List, Optional
 from urllib.parse import urlparse
 
 import requests
-from retrying import retry
+from tqdm import tqdm
 
 from pymatgen.core.periodic_table import DummySpecies
 from pymatgen.core.structure import Structure
 from pymatgen.util.provenance import StructureNL
-from pymatgen.util.sequence import PBar
+
+# from retrying import retry
+
 
 # TODO: importing optimade-python-tool's data structures will make more sense
 Provider = namedtuple("Provider", ["name", "base_url", "description", "homepage", "prefix"])
@@ -28,15 +31,19 @@ _logger.setLevel(logging.WARNING)
 
 class OptimadeRester:
     """
-    Class to call OPTIMADE-compliant APIs, see optimade.org
+    Class to call OPTIMADE-compliant APIs, see https://optimade.org and [1].
+
+    This class is ready to use but considered in-development and subject to change.
+
+    [1] Andersen, C.W., *et al*.
+        OPTIMADE, an API for exchanging materials data.
+        Sci Data 8, 217 (2021). https://doi.org/10.1038/s41597-021-00974-z
 
-    This class is ready to use but considered in-development and subject to change
-    until the OPTIMADE paper is published.
     """
 
     # regenerate on-demand from official providers.json using OptimadeRester.refresh_aliases()
     # these aliases are provided as a convenient shortcut for users of the OptimadeRester class
-    aliases = {
+    aliases: dict[str, str] = {
         "aflow": "http://aflow.org/API/optimade/",
         "cod": "https://www.crystallography.net/cod/optimade",
         "mcloud.2dstructures": "https://aiida.materialscloud.org/2dstructures/optimade",
@@ -51,13 +58,18 @@ class OptimadeRester:
         "mcloud.tc-applicability": "https://aiida.materialscloud.org/tc-applicability/optimade",
         "mcloud.threedd": "https://aiida.materialscloud.org/3dd/optimade",
         "mp": "https://optimade.materialsproject.org",
+        "mpds": "https://api.mpds.io",
+        "nmd": "https://nomad-lab.eu/prod/rae/optimade/",
         "odbx": "https://optimade.odbx.science",
         "omdb.omdb_production": "http://optimade.openmaterialsdb.se",
         "oqmd": "http://oqmd.org/optimade/",
         "tcod": "https://www.crystallography.net/tcod/optimade",
     }
 
-    def __init__(self, aliases_or_resource_urls: Optional[Union[str, List[str]]] = None, timeout=5):
+    # The set of OPTIMADE fields that are required to define a `pymatgen.core.Structure`
+    mandatory_response_fields: set[str] = {"lattice_vectors", "cartesian_site_positions", "species", "species_at_sites"}
+
+    def __init__(self, aliases_or_resource_urls: str | list[str] | None = None, timeout: int = 5):
         """
         OPTIMADE is an effort to provide a standardized interface to retrieve information
         from many different materials science databases.
@@ -67,7 +79,7 @@ class OptimadeRester:
         as a way to quickly search an endpoint in a way familiar to users of pymatgen without
         needing to know the full OPTIMADE specification.
 
-        For advanced usage, please see the OPTIMADE documentation at optimate.org and
+        For advanced usage, please see the OPTIMADE documentation at optimade.org and
         consider calling the APIs directly.
 
         For convenience, known OPTIMADE endpoints have been given aliases in pymatgen to save
@@ -139,7 +151,7 @@ class OptimadeRester:
         description = f"OptimadeRester connected to:\n{provider_text}"
         return description
 
-    @retry(stop_max_attempt_number=3, wait_random_min=1000, wait_random_max=2000)
+    # @retry(stop_max_attempt_number=3, wait_random_min=1000, wait_random_max=2000)
     def _get_json(self, url):
         """
         Retrieves JSON, will attempt to (politely) try again on failure subject to a
@@ -149,7 +161,11 @@ class OptimadeRester:
 
     @staticmethod
     def _build_filter(
-        elements=None, nelements=None, nsites=None, chemical_formula_anonymous=None, chemical_formula_hill=None
+        elements: str | list[str] | None = None,
+        nelements: int = None,
+        nsites: int = None,
+        chemical_formula_anonymous: str = None,
+        chemical_formula_hill: str = None,
     ):
         """
         Convenience method to build an OPTIMADE filter.
@@ -184,8 +200,13 @@ class OptimadeRester:
         return " AND ".join(filters)
 
     def get_structures(
-        self, elements=None, nelements=None, nsites=None, chemical_formula_anonymous=None, chemical_formula_hill=None,
-    ) -> Dict[str, Dict[str, Structure]]:
+        self,
+        elements: list[str] | str | None = None,
+        nelements: int = None,
+        nsites: int = None,
+        chemical_formula_anonymous: str = None,
+        chemical_formula_hill: str = None,
+    ) -> dict[str, dict[str, Structure]]:
         """
         Retrieve Structures from OPTIMADE providers.
 
@@ -213,8 +234,14 @@ class OptimadeRester:
         return self.get_structures_with_filter(optimade_filter)
 
     def get_snls(
-        self, elements=None, nelements=None, nsites=None, chemical_formula_anonymous=None, chemical_formula_hill=None,
-    ) -> Dict[str, Dict[str, StructureNL]]:
+        self,
+        elements: list[str] | str | None = None,
+        nelements: int = None,
+        nsites: int = None,
+        chemical_formula_anonymous: str = None,
+        chemical_formula_hill: str = None,
+        additional_response_fields: str | list[str] | set[str] | None = None,
+    ) -> dict[str, dict[str, StructureNL]]:
         """
         Retrieve StructureNL from OPTIMADE providers.
 
@@ -231,6 +258,8 @@ class OptimadeRester:
             nsites: Number of sites, e.g. 4 or [2, 5] for the range >=2 and <=5
             chemical_formula_anonymous: Anonymous chemical formula
             chemical_formula_hill: Chemical formula following Hill convention
+            additional_response_fields: Any additional fields desired from the OPTIMADE API,
+            these will be stored under the `'_optimade'` key in each `StructureNL.data` dictionary.
 
         Returns: Dict of (Dict of StructureNLs keyed by that database's id system) keyed by provider
         """
@@ -243,9 +272,9 @@ class OptimadeRester:
             chemical_formula_hill=chemical_formula_hill,
         )
 
-        return self.get_snls_with_filter(optimade_filter)
+        return self.get_snls_with_filter(optimade_filter, additional_response_fields=additional_response_fields)
 
-    def get_structures_with_filter(self, optimade_filter: str) -> Dict[str, Dict[str, Structure]]:
+    def get_structures_with_filter(self, optimade_filter: str) -> dict[str, dict[str, Structure]]:
         """
         Get structures satisfying a given OPTIMADE filter.
 
@@ -263,7 +292,11 @@ class OptimadeRester:
 
         return all_structures
 
-    def get_snls_with_filter(self, optimade_filter: str) -> Dict[str, Dict[str, StructureNL]]:
+    def get_snls_with_filter(
+        self,
+        optimade_filter: str,
+        additional_response_fields: str | list[str] | set[str] | None = None,
+    ) -> dict[str, dict[str, StructureNL]]:
         """
         Get structures satisfying a given OPTIMADE filter.
 
@@ -275,11 +308,11 @@ class OptimadeRester:
 
         all_snls = {}
 
+        fields = self._handle_response_fields(additional_response_fields)
+
         for identifier, resource in self.resources.items():
 
-            fields = "response_fields=lattice_vectors,cartesian_site_positions,species,species_at_sites"
-
-            url = join(resource, f"v1/structures?filter={optimade_filter}&fields={fields}")
+            url = join(resource, f"v1/structures?filter={optimade_filter}&response_fields={fields}")
 
             try:
 
@@ -287,7 +320,7 @@ class OptimadeRester:
 
                 structures = self._get_snls_from_resource(json, url, identifier)
 
-                pbar = PBar(total=json["meta"].get("data_returned", 0), desc=identifier, initial=len(structures))
+                pbar = tqdm(total=json["meta"].get("data_returned", 0), desc=identifier, initial=len(structures))
 
                 # TODO: check spec for `more_data_available` boolean, may simplify this conditional
                 if ("links" in json) and ("next" in json["links"]) and (json["links"]["next"]):
@@ -315,7 +348,7 @@ class OptimadeRester:
         return all_snls
 
     @staticmethod
-    def _get_snls_from_resource(json, url, identifier) -> Dict[str, StructureNL]:
+    def _get_snls_from_resource(json, url, identifier) -> dict[str, StructureNL]:
 
         snls = {}
 
@@ -347,7 +380,12 @@ class OptimadeRester:
                     coords=data["attributes"]["cartesian_site_positions"],
                     coords_are_cartesian=True,
                 )
-                namespaced_data = {k: v for k, v in data.items() if k.startswith("_")}
+                # Grab any custom fields or non-mandatory fields if they were requested
+                namespaced_data = {
+                    k: v
+                    for k, v in data["attributes"].items()
+                    if k.startswith("_") or k not in {"lattice_vectors", "species", "cartesian_site_positions"}
+                }
 
                 # TODO: follow `references` to add reference information here
                 snl = StructureNL(
@@ -370,7 +408,12 @@ class OptimadeRester:
                         coords=data["attributes"]["cartesian_site_positions"],
                         coords_are_cartesian=True,
                     )
-                    namespaced_data = {k: v for k, v in data["attributes"].items() if k.startswith("_")}
+                    # Grab any custom fields or non-mandatory fields if they were requested
+                    namespaced_data = {
+                        k: v
+                        for k, v in data["attributes"].items()
+                        if k.startswith("_") or k not in {"lattice_vectors", "species", "cartesian_site_positions"}
+                    }
 
                     # TODO: follow `references` to add reference information here
                     snl = StructureNL(
@@ -391,7 +434,7 @@ class OptimadeRester:
 
         return snls
 
-    def _validate_provider(self, provider_url) -> Optional[Provider]:
+    def _validate_provider(self, provider_url) -> Provider | None:
         """
         Checks that a given URL is indeed an OPTIMADE provider,
         returning None if it is not a provider, or the provider
@@ -434,7 +477,7 @@ class OptimadeRester:
             _logger.warning(f"Failed to extract required information from {url}: {exc}")
             return None
 
-    def _parse_provider(self, provider, provider_url) -> Dict[str, Provider]:
+    def _parse_provider(self, provider, provider_url) -> dict[str, Provider]:
         """
         Used internally to update the list of providers or to
         check a given URL is valid.
@@ -484,6 +527,22 @@ class OptimadeRester:
 
         return _parse_provider_link(provider, provider_link_json)
 
+    def _handle_response_fields(self, additional_response_fields: str | list[str] | set[str] | None = None) -> str:
+        """
+        Used internally to handle the mandatory and additional response fields.
+
+        Args:
+            additional_response_fields: A set of additional fields to request.
+
+        Returns:
+            A string of comma-separated OPTIMADE response fields.
+        """
+        if isinstance(additional_response_fields, str):
+            additional_response_fields = [additional_response_fields]
+        if not additional_response_fields:
+            additional_response_fields = set()
+        return ",".join(set(additional_response_fields).union(self.mandatory_response_fields))
+
     def refresh_aliases(self, providers_url="https://providers.optimade.org/providers.json"):
         """
         Updates available OPTIMADE structure resources based on the current list of OPTIMADE
