@@ -1,10 +1,9 @@
-# coding: utf-8
 # Copyright (c) Pymatgen Development Team.
 # Distributed under the terms of the MIT License.
 
 """
 This module defines the FeffInputSet abstract base class and a concrete
-implementation for the Materials Project.  The basic concept behind an input
+implementation for the Materials Project. The basic concept behind an input
 set is to specify a scheme to generate a consistent set of Feff inputs from a
 structure without further user intervention. This ensures comparability across
 runs.
@@ -53,7 +52,6 @@ class AbstractFeffInputSet(MSONable, metaclass=abc.ABCMeta):
         """
         Returns header to be used in feff.inp file from a pymatgen structure
         """
-        pass
 
     @property
     @abc.abstractmethod
@@ -64,7 +62,6 @@ class AbstractFeffInputSet(MSONable, metaclass=abc.ABCMeta):
         Returns:
             Atoms object.
         """
-        pass
 
     @property
     @abc.abstractmethod
@@ -80,7 +77,6 @@ class AbstractFeffInputSet(MSONable, metaclass=abc.ABCMeta):
         """
         Returns POTENTIAL section used in feff.inp from a structure.
         """
-        pass
 
     def all_input(self):
         """
@@ -199,7 +195,7 @@ class FEFFDictSet(AbstractFeffInputSet):
         """
         if "RECIPROCAL" in self.config_dict:
             if self.small_system:
-                self.config_dict["CIF"] = "{}.cif".format(self.structure.formula.replace(" ", ""))
+                self.config_dict["CIF"] = f"{self.structure.formula.replace(' ', '')}.cif"
                 self.config_dict["TARGET"] = self.atoms.center_index + 1
                 self.config_dict["COREHOLE"] = "RPA"
                 logger.warning("Setting COREHOLE = RPA for K-space calculation")
@@ -242,7 +238,7 @@ class FEFFDictSet(AbstractFeffInputSet):
 
     def __str__(self):
         output = [self.spectrum]
-        output.extend(["%s = %s" % (k, str(v)) for k, v in self.config_dict.items()])
+        output.extend([f"{k} = {v}" for k, v in self.config_dict.items()])
         output.append("")
         return "\n".join(output)
 
