@@ -1,4 +1,3 @@
-# coding: utf-8
 # Copyright (c) Pymatgen Development Team.
 # Distributed under the terms of the MIT License.
 
@@ -14,14 +13,14 @@ import scipy.constants as cst
 from monty.io import zopen
 
 from pymatgen.core.composition import Composition
+from pymatgen.core.operations import SymmOp
 from pymatgen.core.periodic_table import Element
 from pymatgen.core.structure import Molecule
-from pymatgen.core.operations import SymmOp
 from pymatgen.core.units import Ha_to_eV
 from pymatgen.electronic_structure.core import Spin
 from pymatgen.util.coord import get_angle
 
-__author__ = "Shyue Ping Ong, Germain  Salvato-Vallverdu, Xin Chen"
+__author__ = "Shyue Ping Ong, Germain Salvato-Vallverdu, Xin Chen"
 __copyright__ = "Copyright 2013, The Materials Virtual Lab"
 __version__ = "0.1"
 __maintainer__ = "Shyue Ping Ong"
@@ -111,7 +110,7 @@ class GaussianInput:
         """
         Args:
             mol: Input molecule. It can either be a Molecule object,
-                a string giving the geometry in a format supported by Guassian,
+                a string giving the geometry in a format supported by Gaussian,
                 or ``None``. If the molecule is ``None``, you will need to use
                 read it in from a checkpoint. Consider adding ``CHK`` to the
                 ``link0_parameters``.
@@ -130,7 +129,7 @@ class GaussianInput:
             route_parameters: Additional route parameters as a dict. For example,
                 {'SP':"", "SCF":"Tight"}
             input_parameters: Additional input parameters for run as a dict. Used
-                for example, in PCM calculations.  E.g., {"EPS":12}
+                for example, in PCM calculations. E.g., {"EPS":12}
             link0_parameters: Link0 parameters as a dict. E.g., {"%mem": "1000MW"}
             dieze_tag: # preceding the route line. E.g. "#p"
             gen_basis: allows a user-specified basis set to be used in a Gaussian
@@ -388,39 +387,37 @@ class GaussianInput:
         outputvar = []
         for i, site in enumerate(self._mol):
             if i == 0:
-                output.append("{}".format(site.specie))
+                output.append(f"{site.specie}")
             elif i == 1:
                 nn = self._find_nn_pos_before_site(i)
                 bondlength = self._mol.get_distance(i, nn[0])
-                output.append("{} {} B{}".format(self._mol[i].specie, nn[0] + 1, i))
-                outputvar.append("B{}={:.6f}".format(i, bondlength))
+                output.append(f"{self._mol[i].specie} {nn[0] + 1} B{i}")
+                outputvar.append(f"B{i}={bondlength:.6f}")
             elif i == 2:
                 nn = self._find_nn_pos_before_site(i)
                 bondlength = self._mol.get_distance(i, nn[0])
                 angle = self._mol.get_angle(i, nn[0], nn[1])
-                output.append("{} {} B{} {} A{}".format(self._mol[i].specie, nn[0] + 1, i, nn[1] + 1, i))
-                outputvar.append("B{}={:.6f}".format(i, bondlength))
-                outputvar.append("A{}={:.6f}".format(i, angle))
+                output.append(f"{self._mol[i].specie} {nn[0] + 1} B{i} {nn[1] + 1} A{i}")
+                outputvar.append(f"B{i}={bondlength:.6f}")
+                outputvar.append(f"A{i}={angle:.6f}")
             else:
                 nn = self._find_nn_pos_before_site(i)
                 bondlength = self._mol.get_distance(i, nn[0])
                 angle = self._mol.get_angle(i, nn[0], nn[1])
                 dih = self._mol.get_dihedral(i, nn[0], nn[1], nn[2])
-                output.append(
-                    "{} {} B{} {} A{} {} D{}".format(self._mol[i].specie, nn[0] + 1, i, nn[1] + 1, i, nn[2] + 1, i)
-                )
-                outputvar.append("B{}={:.6f}".format(i, bondlength))
-                outputvar.append("A{}={:.6f}".format(i, angle))
-                outputvar.append("D{}={:.6f}".format(i, dih))
+                output.append(f"{self._mol[i].specie} {nn[0] + 1} B{i} {nn[1] + 1} A{i} {nn[2] + 1} D{i}")
+                outputvar.append(f"B{i}={bondlength:.6f}")
+                outputvar.append(f"A{i}={angle:.6f}")
+                outputvar.append(f"D{i}={dih:.6f}")
         return "\n".join(output) + "\n\n" + "\n".join(outputvar)
 
     def get_cart_coords(self):
         """
-        Return the cartesian coordinates of the molecule
+        Return the Cartesian coordinates of the molecule
         """
 
         def to_s(x):
-            return "%0.6f" % x
+            return f"{x:0.6f}"
 
         outs = []
         for i, site in enumerate(self._mol):
@@ -434,7 +431,7 @@ class GaussianInput:
         """
         Return GaussianInput string
 
-        Option: whe cart_coords sets to True return the cartesian coordinates
+        Option: when cart_coords is set to True return the Cartesian coordinates
                 instead of the z-matrix
 
         """
@@ -447,9 +444,9 @@ class GaussianInput:
                     para_str.append(par)
                 elif isinstance(val, dict):
                     val_str = para_dict_to_string(val, joiner=",")
-                    para_str.append("{}=({})".format(par, val_str))
+                    para_str.append(f"{par}=({val_str})")
                 else:
-                    para_str.append("{}={}".format(par, val))
+                    para_str.append(f"{par}={val}")
             return joiner.join(para_str)
 
         output = []
@@ -461,25 +458,19 @@ class GaussianInput:
         bset_str = "" if self.basis_set is None else self.basis_set.strip()
 
         if func_str != "" and bset_str != "":
-            func_bset_str = " {}/{}".format(func_str, bset_str)
+            func_bset_str = f" {func_str}/{bset_str}"
         else:
             # don't use the slash if either or both are set as empty
-            func_bset_str = " {}{}".format(func_str, bset_str).rstrip()
+            func_bset_str = f" {func_str}{bset_str}".rstrip()
 
-        output.append(
-            "{diez}{func_bset} {route}".format(
-                diez=self.dieze_tag,
-                func_bset=func_bset_str,
-                route=para_dict_to_string(self.route_parameters),
-            )
-        )
+        output.append(f"{self.dieze_tag}{func_bset_str} {para_dict_to_string(self.route_parameters)}")
         output.append("")
         output.append(self.title)
         output.append("")
 
-        charge_str = "" if self.charge is None else "%d" % self.charge
-        multip_str = "" if self.spin_multiplicity is None else " %d" % self.spin_multiplicity
-        output.append("{}{}".format(charge_str, multip_str))
+        charge_str = "" if self.charge is None else f"{self.charge:.0f}"
+        multip_str = "" if self.spin_multiplicity is None else f" {self.spin_multiplicity:.0f}"
+        output.append(f"{charge_str}{multip_str}")
 
         if isinstance(self._mol, Molecule):
             if cart_coords is True:
@@ -490,7 +481,7 @@ class GaussianInput:
             output.append(str(self._mol))
         output.append("")
         if self.gen_basis is not None:
-            output.append("{:s}\n".format(self.gen_basis))
+            output.append(f"{self.gen_basis}\n")
         output.append(para_dict_to_string(self.input_parameters, "\n"))
         output.append("\n")
         return "\n".join(output)
@@ -509,8 +500,8 @@ class GaussianInput:
         :return: MSONable dict
         """
         return {
-            "@module": self.__class__.__module__,
-            "@class": self.__class__.__name__,
+            "@module": type(self).__module__,
+            "@class": type(self).__name__,
             "molecule": self.molecule.as_dict(),
             "functional": self.functional,
             "basis_set": self.basis_set,
@@ -584,7 +575,7 @@ class GaussianOutput:
 
     .. attribute:: cart_forces
 
-        All cartesian forces from the calculation.
+        All Cartesian forces from the calculation.
 
     .. attribute:: frequencies
 
@@ -970,7 +961,7 @@ class GaussianOutput:
                                     "Density Matrix:" in line or mo_coeff_patt.search(line)
                                 ):
                                     end_mo = True
-                                    warnings.warn("POP=regular case, matrix " "coefficients not complete")
+                                    warnings.warn("POP=regular case, matrix coefficients not complete")
                             f.readline()
 
                         self.eigenvectors = mat_mo
@@ -1089,7 +1080,7 @@ class GaussianOutput:
                             terminated = True
                     elif error_patt.search(line):
                         error_defs = {
-                            "! Non-Optimized Parameters !": "Optimization " "error",
+                            "! Non-Optimized Parameters !": "Optimization error",
                             "Convergence failure": "SCF convergence error",
                         }
                         m = error_patt.search(line)
@@ -1194,7 +1185,7 @@ class GaussianOutput:
 
         if energy_patt.search(line):
             m = energy_patt.search(line)
-            self.pcm["{} energy".format(m.group(1))] = float(m.group(2))
+            self.pcm[f"{m.group(1)} energy"] = float(m.group(2))
         elif total_patt.search(line):
             m = total_patt.search(line)
             self.pcm["Total energy"] = float(m.group(1))
@@ -1204,7 +1195,7 @@ class GaussianOutput:
 
     def as_dict(self):
         """
-        Json-serializable dict representation.
+        JSON-serializable dict representation.
         """
         structure = self.final_structure
         d = {
@@ -1247,8 +1238,8 @@ class GaussianOutput:
         }
 
         d["output"] = vout
-        d["@module"] = self.__class__.__module__
-        d["@class"] = self.__class__.__name__
+        d["@module"] = type(self).__module__
+        d["@class"] = type(self).__name__
 
         return d
 
@@ -1323,7 +1314,7 @@ class GaussianOutput:
         Get a matplotlib plot of the potential energy surface.
 
         Args:
-            coords: internal coordinate name to use as abcissa.
+            coords: internal coordinate name to use as abscissa.
         """
         from pymatgen.util.plotting import pretty_plot
 
@@ -1387,7 +1378,7 @@ class GaussianOutput:
 
     def get_spectre_plot(self, sigma=0.05, step=0.01):
         """
-        Get a matplotlib plot of the UV-visible xas. Transition are plotted
+        Get a matplotlib plot of the UV-visible xas. Transitions are plotted
         as vertical lines and as a sum of normal functions with sigma with. The
         broadening is applied in energy and the xas is plotted as a function
         of the wavelength.
@@ -1410,8 +1401,8 @@ class GaussianOutput:
 
         transitions = self.read_excitation_energies()
 
-        minval = min([val[0] for val in transitions]) - 5.0 * sigma
-        maxval = max([val[0] for val in transitions]) + 5.0 * sigma
+        minval = min(val[0] for val in transitions) - 5.0 * sigma
+        maxval = max(val[0] for val in transitions) + 5.0 * sigma
         npts = int((maxval - minval) / step) + 1
 
         eneval = np.linspace(minval, maxval, npts)  # in eV
