@@ -1,4 +1,3 @@
-# coding: utf-8
 # Copyright (c) Pymatgen Development Team.
 # Distributed under the terms of the MIT License
 
@@ -7,17 +6,19 @@ Module for reading Lobster output files. For more information
 on LOBSTER see www.cohp.de.
 """
 
+from __future__ import annotations
+
 import itertools
 import os
 import warnings
-from collections import OrderedDict
-from typing import Any, Dict, List, Optional
+from typing import Any, Dict, List
 
 import numpy as np
 import spglib
 from monty.io import zopen
 from monty.json import MSONable
 from monty.serialization import loadfn
+
 from pymatgen.core.structure import Structure
 from pymatgen.io.vasp.inputs import Incar, Kpoints, Potcar
 from pymatgen.symmetry.bandstructure import HighSymmKpath
@@ -109,16 +110,16 @@ class Lobsterin(dict, MSONable):
         # check for duplicates
         listkey = [key.lower() for key in settingsdict.keys()]
         if len(listkey) != len(list(set(listkey))):
-            raise IOError("There are duplicates for the keywords! The program will stop here.")
+            raise OSError("There are duplicates for the keywords! The program will stop here.")
         self.update(settingsdict)
 
     def __setitem__(self, key, val):
         """
-        Add parameter-val pair to Lobsterin.  Warns if parameter is not in list of
+        Add parameter-val pair to Lobsterin. Warns if parameter is not in list of
         valid lobsterintags. Also cleans the parameter and val by stripping
         leading and trailing white spaces. Similar to INCAR class.
         """
-        # due to the missing case sensitivity of lobster, the following code is neccessary
+        # due to the missing case sensitivity of lobster, the following code is necessary
         found = False
         for key_here in self.keys():
             if key.strip().lower() == key_here.lower():
@@ -210,7 +211,7 @@ class Lobsterin(dict, MSONable):
         get number of nbands
         """
         if self.get("basisfunctions") is None:
-            raise IOError("No basis functions are provided. The program cannot calculate nbands.")
+            raise OSError("No basis functions are provided. The program cannot calculate nbands.")
 
         basis_functions = []  # type: List[str]
         for string_basis in self["basisfunctions"]:
@@ -278,8 +279,8 @@ class Lobsterin(dict, MSONable):
         :return: MSONable dict
         """
         d = dict(self)
-        d["@module"] = self.__class__.__module__
-        d["@class"] = self.__class__.__name__
+        d["@module"] = type(self).__module__
+        d["@class"] = type(self).__name__
         return d
 
     @classmethod
@@ -349,7 +350,7 @@ class Lobsterin(dict, MSONable):
         AtomTypes = structure.symbol_set
 
         if set(AtomTypes) != set(AtomTypes_Potcar):
-            raise IOError("Your POSCAR does not correspond to your POTCAR!")
+            raise OSError("Your POSCAR does not correspond to your POTCAR!")
         BASIS = loadfn(address_basis_file)["BASIS"]
 
         basis_functions = []
@@ -379,7 +380,7 @@ class Lobsterin(dict, MSONable):
         Args:
             structure: Structure object
             potcar_symbols: list of the potcar symbols
-            address_basis_file_min: path to file with the minium required basis by the POTCAR
+            address_basis_file_min: path to file with the minimum required basis by the POTCAR
             address_basis_file_max: path to file with the largest possible basis of the POTCAR
 
         Returns: List of dictionaries that can be used to create new Lobsterin objects in
@@ -501,9 +502,9 @@ class Lobsterin(dict, MSONable):
             newlist = [list(gp) for gp in list(grid)]
             mapping = []
             for gp in newlist:
-                minusgp = [-k for k in gp]
-                if minusgp in newlist and minusgp not in [[0, 0, 0]]:
-                    mapping.append(newlist.index(minusgp))
+                minus_gp = [-k for k in gp]
+                if minus_gp in newlist and minus_gp not in [[0, 0, 0]]:
+                    mapping.append(newlist.index(minus_gp))
                 else:
                     mapping.append(newlist.index(gp))
 
@@ -566,11 +567,11 @@ class Lobsterin(dict, MSONable):
         with zopen(lobsterin, "rt") as f:
             data = f.read().split("\n")
         if len(data) == 0:
-            raise IOError("lobsterin file contains no data.")
+            raise OSError("lobsterin file contains no data.")
         Lobsterindict = {}  # type: Dict
 
         for datum in data:
-            # will remove all commments to avoid complications
+            # will remove all comments to avoid complications
             raw_datum = datum.split("!")[0]
             raw_datum = raw_datum.split("//")[0]
             raw_datum = raw_datum.split("#")[0]
@@ -612,11 +613,13 @@ class Lobsterin(dict, MSONable):
         potcar = Potcar.from_file(POTCAR_input)
         for pot in potcar:
             if pot.potential_type != "PAW":
-                raise IOError("Lobster only works with PAW! Use different POTCARs")
+                raise OSError("Lobster only works with PAW! Use different POTCARs")
 
         # Warning about a bug in lobster-4.1.0
         with zopen(POTCAR_input, "r") as f:
             data = f.read()
+        if isinstance(data, bytes):
+            data = data.decode("utf-8")
         if "SHA256" in data or "COPYR" in data:
             warnings.warn(
                 "These POTCARs are not compatible with "
@@ -627,7 +630,7 @@ class Lobsterin(dict, MSONable):
             )
 
         if potcar.functional != "PBE":
-            raise IOError("We only have BASIS options for PBE so far")
+            raise OSError("We only have BASIS options for PBE so far")
 
         Potcar_names = [name["symbol"] for name in potcar.spec]
         return Potcar_names
@@ -637,8 +640,8 @@ class Lobsterin(dict, MSONable):
         cls,
         POSCAR_input: str = "POSCAR",
         INCAR_input: str = "INCAR",
-        POTCAR_input: Optional[str] = None,
-        dict_for_basis: Optional[dict] = None,
+        POTCAR_input: str | None = None,
+        dict_for_basis: dict | None = None,
         option: str = "standard",
     ):
         """
@@ -687,7 +690,7 @@ class Lobsterin(dict, MSONable):
         # this basis set covers most elements
         Lobsterindict["basisSet"] = "pbeVaspFit2015"
         # energies around e-fermi
-        Lobsterindict["COHPstartEnergy"] = -15.0
+        Lobsterindict["COHPstartEnergy"] = -35.0
         Lobsterindict["COHPendEnergy"] = 5.0
 
         if option in [
@@ -811,13 +814,12 @@ def get_all_possible_basis_combinations(min_basis: list, max_basis: list) -> lis
         max_basis: list of basis entries: e.g., ['Si 3p 3s ']
 
     Returns: all possible combinations of basis functions, e.g. [['Si 3p 3s']]
-
     """
     max_basis_lists = [x.split() for x in max_basis]
     min_basis_lists = [x.split() for x in min_basis]
 
     # get all possible basis functions
-    basis_dict = OrderedDict({})  # type:  Dict[Any, Any]
+    basis_dict: dict[str, dict] = {}
     for iel, el in enumerate(max_basis_lists):
         basis_dict[el[0]] = {"fixed": [], "variable": [], "combinations": []}
         for basis in el[1:]:
