@@ -1,4 +1,3 @@
-# coding: utf-8
 # Copyright (c) Pymatgen Development Team.
 # Distributed under the terms of the MIT License.
 
@@ -6,9 +5,11 @@
 Input sets for Qchem
 """
 
+from __future__ import annotations
+
 import logging
 import os
-from typing import Dict, List, Optional
+from typing import Literal
 
 from monty.io import zopen
 
@@ -35,17 +36,17 @@ class QChemDictSet(QCInput):
         basis_set: str,
         scf_algorithm: str,
         dft_rung: int = 4,
-        pcm_dielectric: Optional[float] = None,
-        smd_solvent: Optional[str] = None,
-        custom_smd: Optional[str] = None,
-        opt_variables: Optional[Dict[str, List]] = None,
-        scan_variables: Optional[Dict[str, List]] = None,
+        pcm_dielectric: float | None = None,
+        smd_solvent: str | None = None,
+        custom_smd: str | None = None,
+        opt_variables: dict[str, list] | None = None,
+        scan_variables: dict[str, list] | None = None,
         max_scf_cycles: int = 200,
         geom_opt_max_cycles: int = 200,
         plot_cubes: bool = False,
-        nbo_params: Optional[Dict] = None,
-        overwrite_inputs: Optional[Dict] = None,
-        vdw_mode: str = "atomic",
+        nbo_params: dict | None = None,
+        overwrite_inputs: dict | None = None,
+        vdw_mode: Literal["atomic", "sequential"] = "atomic",
     ):
         """
         Args:
@@ -65,7 +66,7 @@ class QChemDictSet(QCInput):
                 argument to {"method":"<NAME OF FUNCTIONAL>"}
 
                 **Note that the "rungs" in this argument do NOT correspond to rungs on "Jacob's
-                Ladder of Density Functional Approxmations"**
+                Ladder of Density Functional Approximations"**
             pcm_dielectric (float): Dielectric constant to use for PCM implicit solvation model. (Default: None)
             smd_solvent (str): Solvent to use for SMD implicit solvation model. (Default: None)
                 Examples include "water", "ethanol", "methanol", and "acetonitrile". Refer to the QChem
@@ -79,7 +80,7 @@ class QChemDictSet(QCInput):
                 electronegative halogenicity"
                 Refer to the QChem manual for further details.
             opt_variables (dict): A dictionary of opt sections, where each opt section is a key
-                and the corresponding values are a list of strings. Stings must be formatted
+                and the corresponding values are a list of strings. Strings must be formatted
                 as instructed by the QChem manual. The different opt sections are: CONSTRAINT, FIXED,
                 DUMMY, and CONNECT.
 
@@ -110,12 +111,11 @@ class QChemDictSet(QCInput):
                 the PCM "radii" setting to "read".**
 
                 **Note that all keys must be given as strings, even when they are numbers!**
-            vdw_mode (str): Method of specifying custom van der Waals radii. Applies only if
-                you are using overwrite_inputs to add a $van_der_waals section to the input.
-                Valid value are 'atomic' and 'sequential'. In 'atomic' mode (default), dict
-                keys represent the atomic number associated with each radius (e.g., '12' = carbon).
-                In 'sequential' mode, dict keys represent the sequential position of a single
-                specific atom in the input structure.
+            vdw_mode ('atomic' | 'sequential'): Method of specifying custom van der Waals radii. Applies
+                only if you are using overwrite_inputs to add a $van_der_waals section to the input.
+                In 'atomic' mode (default), dict keys represent the atomic number associated with each
+                radius (e.g., '12' = carbon). In 'sequential' mode, dict keys represent the sequential
+                position of a single specific atom in the input structure.
         """
         self.molecule = molecule
         self.job_type = job_type
@@ -202,7 +202,7 @@ class QChemDictSet(QCInput):
                 mysmx["solvent"] = self.smd_solvent
             myrem["solvent_method"] = "smd"
             myrem["ideriv"] = "1"
-            if self.smd_solvent == "custom" or self.smd_solvent == "other":
+            if self.smd_solvent in ("custom", "other"):
                 if self.custom_smd is None:
                     raise ValueError(
                         "A user-defined SMD requires passing custom_smd, a string"
@@ -216,8 +216,18 @@ class QChemDictSet(QCInput):
             myrem["plots"] = "true"
             myrem["make_cube_files"] = "true"
 
+        mynbo = self.nbo_params
         if self.nbo_params is not None:
             myrem["nbo"] = "true"
+            if "version" in self.nbo_params:
+                if self.nbo_params["version"] == 7:
+                    myrem["run_nbo6"] = "true"
+                else:
+                    raise RuntimeError("nbo params version should only be set to 7! Exiting...")
+            mynbo = {}
+            for key in self.nbo_params:
+                if key != "version":
+                    mynbo[key] = self.nbo_params[key]
 
         if self.overwrite_inputs:
             for sec, sec_dict in self.overwrite_inputs.items():
@@ -252,9 +262,11 @@ class QChemDictSet(QCInput):
                     for k, v in temp_plots.items():
                         myplots[k] = v
                 if sec == "nbo":
-                    temp_plots = lower_and_check_unique(sec_dict)
-                    for k, v in temp_plots.items():
-                        myplots[k] = v
+                    if mynbo is None:
+                        raise RuntimeError("Can't overwrite nbo params when NBO is not being run! Exiting...")
+                    temp_nbo = lower_and_check_unique(sec_dict)
+                    for k, v in temp_nbo.items():
+                        mynbo[k] = v
                 if sec == "opt":
                     temp_opts = lower_and_check_unique(sec_dict)
                     for k, v in temp_opts.items():
@@ -271,7 +283,7 @@ class QChemDictSet(QCInput):
             van_der_waals=myvdw,
             vdw_mode=self.vdw_mode,
             plots=myplots,
-            nbo=self.nbo_params,
+            nbo=mynbo,
         )
 
     def write(self, input_file: str):
@@ -280,7 +292,7 @@ class QChemDictSet(QCInput):
             input_file (str): Filename
         """
         self.write_file(input_file)
-        if self.smd_solvent == "custom" or self.smd_solvent == "other":
+        if self.smd_solvent in ("custom", "other"):
             with zopen(os.path.join(os.path.dirname(input_file), "solvent_data"), "wt") as f:
                 f.write(self.custom_smd)
 
@@ -296,14 +308,14 @@ class SinglePointSet(QChemDictSet):
         basis_set: str = "def2-tzvppd",
         scf_algorithm: str = "diis",
         dft_rung: int = 3,
-        pcm_dielectric: Optional[float] = None,
-        smd_solvent: Optional[str] = None,
-        custom_smd: Optional[str] = None,
+        pcm_dielectric: float | None = None,
+        smd_solvent: str | None = None,
+        custom_smd: str | None = None,
         max_scf_cycles: int = 200,
         plot_cubes: bool = False,
-        nbo_params: Optional[Dict] = None,
-        overwrite_inputs: Optional[Dict] = None,
-        vdw_mode: str = "atomic",
+        nbo_params: dict | None = None,
+        overwrite_inputs: dict | None = None,
+        vdw_mode: Literal["atomic", "sequential"] = "atomic",
     ):
         """
         Args:
@@ -324,7 +336,7 @@ class SinglePointSet(QChemDictSet):
                 argument to {"method":"<NAME OF FUNCTIONAL>"}
 
                 **Note that the "rungs" in this argument do NOT correspond to rungs on "Jacob's
-                Ladder of Density Functional Approxmations"**
+                Ladder of Density Functional Approximations"**
             pcm_dielectric (float): Dielectric constant to use for PCM implicit solvation model. (Default: None)
             smd_solvent (str): Solvent to use for SMD implicit solvation model. (Default: None)
                 Examples include "water", "ethanol", "methanol", and "acetonitrile". Refer to the QChem
@@ -354,12 +366,11 @@ class SinglePointSet(QChemDictSet):
                 the PCM "radii" setting to "read".**
 
                 **Note that all keys must be given as strings, even when they are numbers!**
-            vdw_mode (str): Method of specifying custom van der Waals radii. Applies only if
-                you are using overwrite_inputs to add a $van_der_waals section to the input.
-                Valid value are 'atomic' and 'sequential'. In 'atomic' mode (default), dict
-                keys represent the atomic number associated with each radius (e.g., '12' = carbon).
-                In 'sequential' mode, dict keys represent the sequential position of a single
-                specific atom in the input structure.
+            vdw_mode ('atomic' | 'sequential'): Method of specifying custom van der Waals radii. Applies
+                only if you are using overwrite_inputs to add a $van_der_waals section to the input.
+                In 'atomic' mode (default), dict keys represent the atomic number associated with each
+                radius (e.g., '12' = carbon). In 'sequential' mode, dict keys represent the sequential
+                position of a single specific atom in the input structure.
         """
         self.basis_set = basis_set
         self.scf_algorithm = scf_algorithm
@@ -392,16 +403,16 @@ class OptSet(QChemDictSet):
         basis_set: str = "def2-tzvppd",
         scf_algorithm: str = "diis",
         dft_rung: int = 3,
-        pcm_dielectric: Optional[float] = None,
-        smd_solvent: Optional[str] = None,
-        custom_smd: Optional[str] = None,
+        pcm_dielectric: float | None = None,
+        smd_solvent: str | None = None,
+        custom_smd: str | None = None,
         max_scf_cycles: int = 200,
         plot_cubes: bool = False,
-        nbo_params: Optional[Dict] = None,
-        opt_variables: Optional[Dict[str, List]] = None,
+        nbo_params: dict | None = None,
+        opt_variables: dict[str, list] | None = None,
         geom_opt_max_cycles: int = 200,
-        overwrite_inputs: Optional[Dict] = None,
-        vdw_mode: str = "atomic",
+        overwrite_inputs: dict | None = None,
+        vdw_mode: Literal["atomic", "sequential"] = "atomic",
     ):
         """
         Args:
@@ -422,7 +433,7 @@ class OptSet(QChemDictSet):
                 argument to {"method":"<NAME OF FUNCTIONAL>"}
 
                 **Note that the "rungs" in this argument do NOT correspond to rungs on "Jacob's
-                Ladder of Density Functional Approxmations"**
+                Ladder of Density Functional Approximations"**
             pcm_dielectric (float): Dielectric constant to use for PCM implicit solvation model. (Default: None)
             smd_solvent (str): Solvent to use for SMD implicit solvation model. (Default: None)
                 Examples include "water", "ethanol", "methanol", and "acetonitrile". Refer to the QChem
@@ -453,12 +464,11 @@ class OptSet(QChemDictSet):
                 the PCM "radii" setting to "read".**
 
                 **Note that all keys must be given as strings, even when they are numbers!**
-            vdw_mode (str): Method of specifying custom van der Waals radii. Applies only if
-                you are using overwrite_inputs to add a $van_der_waals section to the input.
-                Valid value are 'atomic' and 'sequential'. In 'atomic' mode (default), dict
-                keys represent the atomic number associated with each radius (e.g., '12' = carbon).
-                In 'sequential' mode, dict keys represent the sequential position of a single
-                specific atom in the input structure.
+            vdw_mode ('atomic' | 'sequential'): Method of specifying custom van der Waals radii. Applies
+                only if you are using overwrite_inputs to add a $van_der_waals section to the input.
+                In 'atomic' mode (default), dict keys represent the atomic number associated with each
+                radius (e.g., '12' = carbon). In 'sequential' mode, dict keys represent the sequential
+                position of a single specific atom in the input structure.
         """
         self.basis_set = basis_set
         self.scf_algorithm = scf_algorithm
@@ -494,15 +504,15 @@ class TransitionStateSet(QChemDictSet):
         basis_set: str = "def2-tzvppd",
         scf_algorithm: str = "diis",
         dft_rung: int = 3,
-        pcm_dielectric: Optional[float] = None,
-        smd_solvent: Optional[str] = None,
-        custom_smd: Optional[str] = None,
+        pcm_dielectric: float | None = None,
+        smd_solvent: str | None = None,
+        custom_smd: str | None = None,
         max_scf_cycles: int = 200,
         plot_cubes: bool = False,
-        nbo_params: Optional[Dict] = None,
-        opt_variables: Optional[Dict[str, List]] = None,
+        nbo_params: dict | None = None,
+        opt_variables: dict[str, list] | None = None,
         geom_opt_max_cycles: int = 200,
-        overwrite_inputs: Optional[Dict] = None,
+        overwrite_inputs: dict | None = None,
         vdw_mode="atomic",
     ):
         """
@@ -521,7 +531,7 @@ class TransitionStateSet(QChemDictSet):
                 argument to {"method":"<NAME OF FUNCTIONAL>"}
 
                 **Note that the "rungs" in this argument do NOT correspond to rungs on "Jacob's
-                Ladder of Density Functional Approxmations"**
+                Ladder of Density Functional Approximations"**
             pcm_dielectric (float): Dielectric constant to use for PCM implicit solvation model. (Default: None)
             smd_solvent (str): Solvent to use for SMD implicit solvation model. (Default: None)
                 Examples include "water", "ethanol", "methanol", and "acetonitrile". Refer to the QChem
@@ -552,12 +562,11 @@ class TransitionStateSet(QChemDictSet):
                 the PCM "radii" setting to "read".**
 
                 **Note that all keys must be given as strings, even when they are numbers!**
-            vdw_mode (str): Method of specifying custom van der Waals radii. Applies only if
-                you are using overwrite_inputs to add a $van_der_waals section to the input.
-                Valid value are 'atomic' and 'sequential'. In 'atomic' mode (default), dict
-                keys represent the atomic number associated with each radius (e.g., '12' = carbon).
-                In 'sequential' mode, dict keys represent the sequential position of a single
-                specific atom in the input structure.
+            vdw_mode ('atomic' | 'sequential'): Method of specifying custom van der Waals radii. Applies
+                only if you are using overwrite_inputs to add a $van_der_waals section to the input.
+                In 'atomic' mode (default), dict keys represent the atomic number associated with each
+                radius (e.g., '12' = carbon). In 'sequential' mode, dict keys represent the sequential
+                position of a single specific atom in the input structure.
         """
         self.basis_set = basis_set
         self.scf_algorithm = scf_algorithm
@@ -593,14 +602,14 @@ class ForceSet(QChemDictSet):
         basis_set: str = "def2-tzvppd",
         scf_algorithm: str = "diis",
         dft_rung: int = 3,
-        pcm_dielectric: Optional[float] = None,
-        smd_solvent: Optional[str] = None,
-        custom_smd: Optional[str] = None,
+        pcm_dielectric: float | None = None,
+        smd_solvent: str | None = None,
+        custom_smd: str | None = None,
         max_scf_cycles: int = 200,
         plot_cubes: bool = False,
-        nbo_params: Optional[Dict] = None,
-        overwrite_inputs: Optional[Dict] = None,
-        vdw_mode: str = "atomic",
+        nbo_params: dict | None = None,
+        overwrite_inputs: dict | None = None,
+        vdw_mode: Literal["atomic", "sequential"] = "atomic",
     ):
         """
         Args:
@@ -618,7 +627,7 @@ class ForceSet(QChemDictSet):
                 argument to {"method":"<NAME OF FUNCTIONAL>"}
 
                 **Note that the "rungs" in this argument do NOT correspond to rungs on "Jacob's
-                Ladder of Density Functional Approxmations"**
+                Ladder of Density Functional Approximations"**
             pcm_dielectric (float): Dielectric constant to use for PCM implicit solvation model. (Default: None)
             smd_solvent (str): Solvent to use for SMD implicit solvation model. (Default: None)
                 Examples include "water", "ethanol", "methanol", and "acetonitrile". Refer to the QChem
@@ -649,12 +658,11 @@ class ForceSet(QChemDictSet):
                 the PCM "radii" setting to "read".**
 
                 **Note that all keys must be given as strings, even when they are numbers!**
-            vdw_mode (str): Method of specifying custom van der Waals radii. Applies only if
-                you are using overwrite_inputs to add a $van_der_waals section to the input.
-                Valid value are 'atomic' and 'sequential'. In 'atomic' mode (default), dict
-                keys represent the atomic number associated with each radius (e.g., '12' = carbon).
-                In 'sequential' mode, dict keys represent the sequential position of a single
-                specific atom in the input structure.
+            vdw_mode ('atomic' | 'sequential'): Method of specifying custom van der Waals radii. Applies
+                only if you are using overwrite_inputs to add a $van_der_waals section to the input.
+                In 'atomic' mode (default), dict keys represent the atomic number associated with each
+                radius (e.g., '12' = carbon). In 'sequential' mode, dict keys represent the sequential
+                position of a single specific atom in the input structure.
         """
         self.basis_set = basis_set
         self.scf_algorithm = scf_algorithm
@@ -687,14 +695,14 @@ class FreqSet(QChemDictSet):
         basis_set: str = "def2-tzvppd",
         scf_algorithm: str = "diis",
         dft_rung: int = 3,
-        pcm_dielectric: Optional[float] = None,
-        smd_solvent: Optional[str] = None,
-        custom_smd: Optional[str] = None,
+        pcm_dielectric: float | None = None,
+        smd_solvent: str | None = None,
+        custom_smd: str | None = None,
         max_scf_cycles: int = 200,
         plot_cubes: bool = False,
-        nbo_params: Optional[Dict] = None,
-        overwrite_inputs: Optional[Dict] = None,
-        vdw_mode: str = "atomic",
+        nbo_params: dict | None = None,
+        overwrite_inputs: dict | None = None,
+        vdw_mode: Literal["atomic", "sequential"] = "atomic",
     ):
         """
         Args:
@@ -712,7 +720,7 @@ class FreqSet(QChemDictSet):
                 argument to {"method":"<NAME OF FUNCTIONAL>"}
 
                 **Note that the "rungs" in this argument do NOT correspond to rungs on "Jacob's
-                Ladder of Density Functional Approxmations"**
+                Ladder of Density Functional Approximations"**
             pcm_dielectric (float): Dielectric constant to use for PCM implicit solvation model. (Default: None)
             smd_solvent (str): Solvent to use for SMD implicit solvation model. (Default: None)
                 Examples include "water", "ethanol", "methanol", and "acetonitrile". Refer to the QChem
@@ -743,12 +751,11 @@ class FreqSet(QChemDictSet):
                 the PCM "radii" setting to "read".**
 
                 **Note that all keys must be given as strings, even when they are numbers!**
-            vdw_mode (str): Method of specifying custom van der Waals radii. Applies only if
-                you are using overwrite_inputs to add a $van_der_waals section to the input.
-                Valid value are 'atomic' and 'sequential'. In 'atomic' mode (default), dict
-                keys represent the atomic number associated with each radius (e.g., '12' = carbon).
-                In 'sequential' mode, dict keys represent the sequential position of a single
-                specific atom in the input structure.
+            vdw_mode ('atomic' | 'sequential'): Method of specifying custom van der Waals radii. Applies
+                only if you are using overwrite_inputs to add a $van_der_waals section to the input.
+                In 'atomic' mode (default), dict keys represent the atomic number associated with each
+                radius (e.g., '12' = carbon). In 'sequential' mode, dict keys represent the sequential
+                position of a single specific atom in the input structure.
         """
         self.basis_set = basis_set
         self.scf_algorithm = scf_algorithm
@@ -787,22 +794,22 @@ class PESScanSet(QChemDictSet):
         basis_set: str = "def2-tzvppd",
         scf_algorithm: str = "diis",
         dft_rung: int = 3,
-        pcm_dielectric: Optional[float] = None,
-        smd_solvent: Optional[str] = None,
-        custom_smd: Optional[str] = None,
+        pcm_dielectric: float | None = None,
+        smd_solvent: str | None = None,
+        custom_smd: str | None = None,
         max_scf_cycles: int = 200,
         plot_cubes: bool = False,
-        nbo_params: Optional[Dict] = None,
-        opt_variables: Optional[Dict[str, List]] = None,
-        scan_variables: Optional[Dict[str, List]] = None,
-        overwrite_inputs: Optional[Dict] = None,
-        vdw_mode: str = "atomic",
+        nbo_params: dict | None = None,
+        opt_variables: dict[str, list] | None = None,
+        scan_variables: dict[str, list] | None = None,
+        overwrite_inputs: dict | None = None,
+        vdw_mode: Literal["atomic", "sequential"] = "atomic",
     ):
         """
         Args:
             molecule (Pymatgen Molecule object)
             opt_variables (dict): A dictionary of opt sections, where each opt section is a key
-                and the corresponding values are a list of strings. Stings must be formatted
+                and the corresponding values are a list of strings. Strings must be formatted
                 as instructed by the QChem manual. The different opt sections are: CONSTRAINT, FIXED,
                 DUMMY, and CONNECT.
 
@@ -826,7 +833,7 @@ class PESScanSet(QChemDictSet):
                 argument to {"method":"<NAME OF FUNCTIONAL>"}
 
                 **Note that the "rungs" in this argument do NOT correspond to rungs on "Jacob's
-                Ladder of Density Functional Approxmations"**
+                Ladder of Density Functional Approximations"**
             pcm_dielectric (float): Dielectric constant to use for PCM implicit solvation model. (Default: None)
             smd_solvent (str): Solvent to use for SMD implicit solvation model. (Default: None)
                 Examples include "water", "ethanol", "methanol", and "acetonitrile". Refer to the QChem
@@ -857,10 +864,9 @@ class PESScanSet(QChemDictSet):
                 the PCM "radii" setting to "read".**
 
                 **Note that all keys must be given as strings, even when they are numbers!**
-            vdw_mode (str): Method of specifying custom van der Waals radii. Applies only if
-                you are using overwrite_inputs to add a $van_der_waals section to the input.
-                Valid value are 'atomic' and 'sequential'. In 'atomic' mode (default), dict
-                keys represent the atomic number associated with each radius (e.g., '12' = carbon).
+            vdw_mode ('atomic' | 'sequential'): Method of specifying custom van der Waals radii. Applies only if
+                you are using overwrite_inputs to add a $van_der_waals section to the input. In 'atomic' mode
+                (default), dict keys represent the atomic number associated with each radius (e.g., '12' = carbon).
                 In 'sequential' mode, dict keys represent the sequential position of a single
                 specific atom in the input structure.
         """
@@ -869,7 +875,7 @@ class PESScanSet(QChemDictSet):
         self.max_scf_cycles = max_scf_cycles
 
         if scan_variables is None:
-            raise ValueError("Cannot run a pes_scan job without some variable " "to scan over!")
+            raise ValueError("Cannot run a pes_scan job without some variable to scan over!")
 
         super().__init__(
             molecule=molecule,
