@@ -1,4 +1,3 @@
-# coding: utf-8
 # Copyright (c) Pymatgen Development Team.
 # Distributed under the terms of the MIT License.
 
@@ -6,7 +5,6 @@
 This module provides classes to define a phonon band structure.
 """
 
-import collections
 
 import numpy as np
 from monty.json import MSONable
@@ -98,18 +96,18 @@ class PhononBandStructure(MSONable):
                 internally). The second element containing the 3*len(structure)
                 phonon frequencies with non-analytical correction for that direction.
             eigendisplacements: the phonon eigendisplacements associated to the
-                frequencies in cartesian coordinates. A numpy array of complex
+                frequencies in Cartesian coordinates. A numpy array of complex
                 numbers with shape (3*len(structure), len(qpoints), len(structure), 3).
                 he First index of the array refers to the band, the second to the index
                 of the qpoint, the third to the atom in the structure and the fourth
-                to the cartesian coordinates.
+                to the Cartesian coordinates.
             nac_eigendisplacements: the phonon eigendisplacements associated to the
-                non-analytical frequencies in nac_frequencies in cartesian coordinates.
+                non-analytical frequencies in nac_frequencies in Cartesian coordinates.
                 A list of tuples. The first element of each tuple should be a list
                 defining the direction. The second element containing a numpy array of
                 complex numbers with shape (3*len(structure), len(structure), 3).
             labels_dict: (dict) of {} this links a qpoint (in frac coords or
-                cartesian coordinates depending on the coords) to a label.
+                Cartesian coordinates depending on the coords) to a label.
             coords_are_cartesian: Whether the qpoint coordinates are cartesian.
             structure: The crystal structure (as a pymatgen Structure object)
                 associated with the band structure. This is needed if we
@@ -252,8 +250,8 @@ class PhononBandStructure(MSONable):
         :return: MSONable dict
         """
         d = {
-            "@module": self.__class__.__module__,
-            "@class": self.__class__.__name__,
+            "@module": type(self).__module__,
+            "@class": type(self).__name__,
             "lattice_rec": self.lattice_rec.as_dict(),
             "qpoints": [],
         }
@@ -338,13 +336,13 @@ class PhononBandStructureSymmLine(PhononBandStructure):
                 non-analytical corrections at Gamma. If True frequenciens at Gamma from
                 diffent directions will be stored in naf. Default False.
             eigendisplacements: the phonon eigendisplacements associated to the
-                frequencies in cartesian coordinates. A numpy array of complex
+                frequencies in Cartesian coordinates. A numpy array of complex
                 numbers with shape (3*len(structure), len(qpoints), len(structure), 3).
                 he First index of the array refers to the band, the second to the index
                 of the qpoint, the third to the atom in the structure and the fourth
-                to the cartesian coordinates.
+                to the Cartesian coordinates.
             labels_dict: (dict) of {} this links a qpoint (in frac coords or
-                cartesian coordinates depending on the coords) to a label.
+                Cartesian coordinates depending on the coords) to a label.
             coords_are_cartesian: Whether the qpoint coordinates are cartesian.
             structure: The crystal structure (as a pymatgen Structure object)
                 associated with the band structure. This is needed if we
@@ -526,7 +524,7 @@ class PhononBandStructureSymmLine(PhononBandStructure):
         d["qpoints"] = qpoints
 
         # get labels
-        hsq_dict = collections.OrderedDict()
+        hsq_dict = {}
         for nq, q in enumerate(self.qpoints):
             if q.label is not None:
                 hsq_dict[nq] = q.label
