@@ -1,4 +1,3 @@
-# coding: utf-8
 # Copyright (c) Pymatgen Development Team.
 # Distributed under the terms of the MIT License.
 
@@ -23,7 +22,7 @@ from pymatgen.core.operations import MagSymmOp, SymmOp
 from pymatgen.symmetry.analyzer import SpacegroupAnalyzer
 
 try:
-    from seekpath import get_path  # type: ignore
+    from seekpath import get_path
 except ImportError:
     get_path = None
 
@@ -98,7 +97,7 @@ class KPathBase(metaclass=abc.ABCMeta):
     def get_kpoints(self, line_density=20, coords_are_cartesian=True):
         """
         Returns:
-        the kpoints along the paths in cartesian coordinates
+        the kpoints along the paths in Cartesian coordinates
         together with the labels for symmetry points -Wei.
         """
         list_k_points = []
@@ -192,7 +191,7 @@ class KPathSetyawanCurtarolo(KPathBase):
             elif "I" in spg_symbol:
                 self._kpath = self.bcc()
             else:
-                warn("Unexpected value for spg_symbol: %s" % spg_symbol)
+                warn(f"Unexpected value for spg_symbol: {spg_symbol}")
 
         elif lattice_type == "tetragonal":
             if "P" in spg_symbol:
@@ -205,7 +204,7 @@ class KPathSetyawanCurtarolo(KPathBase):
                 else:
                     self._kpath = self.bctet2(c, a)
             else:
-                warn("Unexpected value for spg_symbol: %s" % spg_symbol)
+                warn(f"Unexpected value for spg_symbol: {spg_symbol}")
 
         elif lattice_type == "orthorhombic":
             a = self._conv.lattice.abc[0]
@@ -216,9 +215,9 @@ class KPathSetyawanCurtarolo(KPathBase):
                 self._kpath = self.orc()
 
             elif "F" in spg_symbol:
-                if 1 / a ** 2 > 1 / b ** 2 + 1 / c ** 2:
+                if 1 / a**2 > 1 / b**2 + 1 / c**2:
                     self._kpath = self.orcf1(a, b, c)
-                elif 1 / a ** 2 < 1 / b ** 2 + 1 / c ** 2:
+                elif 1 / a**2 < 1 / b**2 + 1 / c**2:
                     self._kpath = self.orcf2(a, b, c)
                 else:
                     self._kpath = self.orcf3(a, b, c)
@@ -229,7 +228,7 @@ class KPathSetyawanCurtarolo(KPathBase):
             elif "C" in spg_symbol or "A" in spg_symbol:
                 self._kpath = self.orcc(a, b, c)
             else:
-                warn("Unexpected value for spg_symbol: %s" % spg_symbol)
+                warn(f"Unexpected value for spg_symbol: {spg_symbol}")
 
         elif lattice_type == "hexagonal":
             self._kpath = self.hex()
@@ -256,14 +255,14 @@ class KPathSetyawanCurtarolo(KPathBase):
                 if kgamma == 90:
                     self._kpath = self.mclc2(a, b, c, alpha * pi / 180)
                 if kgamma < 90:
-                    if b * cos(alpha * pi / 180) / c + b ** 2 * sin(alpha * pi / 180) ** 2 / a ** 2 < 1:
+                    if b * cos(alpha * pi / 180) / c + b**2 * sin(alpha * pi / 180) ** 2 / a**2 < 1:
                         self._kpath = self.mclc3(a, b, c, alpha * pi / 180)
-                    if b * cos(alpha * pi / 180) / c + b ** 2 * sin(alpha * pi / 180) ** 2 / a ** 2 == 1:
+                    if b * cos(alpha * pi / 180) / c + b**2 * sin(alpha * pi / 180) ** 2 / a**2 == 1:
                         self._kpath = self.mclc4(a, b, c, alpha * pi / 180)
-                    if b * cos(alpha * pi / 180) / c + b ** 2 * sin(alpha * pi / 180) ** 2 / a ** 2 > 1:
+                    if b * cos(alpha * pi / 180) / c + b**2 * sin(alpha * pi / 180) ** 2 / a**2 > 1:
                         self._kpath = self.mclc5(a, b, c, alpha * pi / 180)
             else:
-                warn("Unexpected value for spg_symbol: %s" % spg_symbol)
+                warn(f"Unexpected value for spg_symbol: {spg_symbol}")
 
         elif lattice_type == "triclinic":
             kalpha = self._rec_lattice.parameters[3]
@@ -279,7 +278,7 @@ class KPathSetyawanCurtarolo(KPathBase):
                 self._kpath = self.trib()
 
         else:
-            warn("Unknown lattice type %s" % lattice_type)
+            warn(f"Unknown lattice type {lattice_type}")
 
     @property
     def conventional(self):
@@ -377,7 +376,7 @@ class KPathSetyawanCurtarolo(KPathBase):
         BCT1 Path
         """
         self.name = "BCT1"
-        eta = (1 + c ** 2 / a ** 2) / 4.0
+        eta = (1 + c**2 / a**2) / 4.0
         kpoints = {
             "\\Gamma": np.array([0.0, 0.0, 0.0]),
             "M": np.array([-0.5, 0.5, 0.5]),
@@ -395,8 +394,8 @@ class KPathSetyawanCurtarolo(KPathBase):
         BCT2 Path
         """
         self.name = "BCT2"
-        eta = (1 + a ** 2 / c ** 2) / 4.0
-        zeta = a ** 2 / (2 * c ** 2)
+        eta = (1 + a**2 / c**2) / 4.0
+        zeta = a**2 / (2 * c**2)
         kpoints = {
             "\\Gamma": np.array([0.0, 0.0, 0.0]),
             "N": np.array([0.0, 0.5, 0.0]),
@@ -454,8 +453,8 @@ class KPathSetyawanCurtarolo(KPathBase):
         ORFC1 Path
         """
         self.name = "ORCF1"
-        zeta = (1 + a ** 2 / b ** 2 - a ** 2 / c ** 2) / 4
-        eta = (1 + a ** 2 / b ** 2 + a ** 2 / c ** 2) / 4
+        zeta = (1 + a**2 / b**2 - a**2 / c**2) / 4
+        eta = (1 + a**2 / b**2 + a**2 / c**2) / 4
 
         kpoints = {
             "\\Gamma": np.array([0.0, 0.0, 0.0]),
@@ -481,9 +480,9 @@ class KPathSetyawanCurtarolo(KPathBase):
         ORFC2 Path
         """
         self.name = "ORCF2"
-        phi = (1 + c ** 2 / b ** 2 - c ** 2 / a ** 2) / 4
-        eta = (1 + a ** 2 / b ** 2 - a ** 2 / c ** 2) / 4
-        delta = (1 + b ** 2 / a ** 2 - b ** 2 / c ** 2) / 4
+        phi = (1 + c**2 / b**2 - c**2 / a**2) / 4
+        eta = (1 + a**2 / b**2 - a**2 / c**2) / 4
+        delta = (1 + b**2 / a**2 - b**2 / c**2) / 4
         kpoints = {
             "\\Gamma": np.array([0.0, 0.0, 0.0]),
             "C": np.array([0.5, 0.5 - eta, 1 - eta]),
@@ -511,8 +510,8 @@ class KPathSetyawanCurtarolo(KPathBase):
         ORFC3 Path
         """
         self.name = "ORCF3"
-        zeta = (1 + a ** 2 / b ** 2 - a ** 2 / c ** 2) / 4
-        eta = (1 + a ** 2 / b ** 2 + a ** 2 / c ** 2) / 4
+        zeta = (1 + a**2 / b**2 - a**2 / c**2) / 4
+        eta = (1 + a**2 / b**2 + a**2 / c**2) / 4
         kpoints = {
             "\\Gamma": np.array([0.0, 0.0, 0.0]),
             "A": np.array([0.5, 0.5 + zeta, zeta]),
@@ -536,10 +535,10 @@ class KPathSetyawanCurtarolo(KPathBase):
         ORCI Path
         """
         self.name = "ORCI"
-        zeta = (1 + a ** 2 / c ** 2) / 4
-        eta = (1 + b ** 2 / c ** 2) / 4
-        delta = (b ** 2 - a ** 2) / (4 * c ** 2)
-        mu = (a ** 2 + b ** 2) / (4 * c ** 2)
+        zeta = (1 + a**2 / c**2) / 4
+        eta = (1 + b**2 / c**2) / 4
+        delta = (b**2 - a**2) / (4 * c**2)
+        mu = (a**2 + b**2) / (4 * c**2)
         kpoints = {
             "\\Gamma": np.array([0.0, 0.0, 0.0]),
             "L": np.array([-mu, mu, 0.5 - delta]),
@@ -567,7 +566,7 @@ class KPathSetyawanCurtarolo(KPathBase):
         ORCC Path
         """
         self.name = "ORCC"
-        zeta = (1 + a ** 2 / b ** 2) / 4
+        zeta = (1 + a**2 / b**2) / 4
         kpoints = {
             "\\Gamma": np.array([0.0, 0.0, 0.0]),
             "A": np.array([zeta, zeta, 0.5]),
@@ -707,7 +706,7 @@ class KPathSetyawanCurtarolo(KPathBase):
         self.name = "MCLC1"
         zeta = (2 - b * cos(alpha) / c) / (4 * sin(alpha) ** 2)
         eta = 0.5 + 2 * zeta * c * cos(alpha) / b
-        psi = 0.75 - a ** 2 / (4 * b ** 2 * sin(alpha) ** 2)
+        psi = 0.75 - a**2 / (4 * b**2 * sin(alpha) ** 2)
         phi = psi + (0.75 - psi) * b * cos(alpha) / c
         kpoints = {
             "\\Gamma": np.array([0.0, 0.0, 0.0]),
@@ -743,7 +742,7 @@ class KPathSetyawanCurtarolo(KPathBase):
         self.name = "MCLC2"
         zeta = (2 - b * cos(alpha) / c) / (4 * sin(alpha) ** 2)
         eta = 0.5 + 2 * zeta * c * cos(alpha) / b
-        psi = 0.75 - a ** 2 / (4 * b ** 2 * sin(alpha) ** 2)
+        psi = 0.75 - a**2 / (4 * b**2 * sin(alpha) ** 2)
         phi = psi + (0.75 - psi) * b * cos(alpha) / c
         kpoints = {
             "\\Gamma": np.array([0.0, 0.0, 0.0]),
@@ -776,8 +775,8 @@ class KPathSetyawanCurtarolo(KPathBase):
         MCLC3 Path
         """
         self.name = "MCLC3"
-        mu = (1 + b ** 2 / a ** 2) / 4.0
-        delta = b * c * cos(alpha) / (2 * a ** 2)
+        mu = (1 + b**2 / a**2) / 4.0
+        delta = b * c * cos(alpha) / (2 * a**2)
         zeta = mu - 0.25 + (1 - b * cos(alpha) / c) / (4 * sin(alpha) ** 2)
         eta = 0.5 + 2 * zeta * c * cos(alpha) / b
         phi = 1 + zeta - 2 * mu
@@ -813,8 +812,8 @@ class KPathSetyawanCurtarolo(KPathBase):
         MCLC4 Path
         """
         self.name = "MCLC4"
-        mu = (1 + b ** 2 / a ** 2) / 4.0
-        delta = b * c * cos(alpha) / (2 * a ** 2)
+        mu = (1 + b**2 / a**2) / 4.0
+        delta = b * c * cos(alpha) / (2 * a**2)
         zeta = mu - 0.25 + (1 - b * cos(alpha) / c) / (4 * sin(alpha) ** 2)
         eta = 0.5 + 2 * zeta * c * cos(alpha) / b
         phi = 1 + zeta - 2 * mu
@@ -850,12 +849,12 @@ class KPathSetyawanCurtarolo(KPathBase):
         MCLC5 Path
         """
         self.name = "MCLC5"
-        zeta = (b ** 2 / a ** 2 + (1 - b * cos(alpha) / c) / sin(alpha) ** 2) / 4
+        zeta = (b**2 / a**2 + (1 - b * cos(alpha) / c) / sin(alpha) ** 2) / 4
         eta = 0.5 + 2 * zeta * c * cos(alpha) / b
-        mu = eta / 2 + b ** 2 / (4 * a ** 2) - b * c * cos(alpha) / (2 * a ** 2)
+        mu = eta / 2 + b**2 / (4 * a**2) - b * c * cos(alpha) / (2 * a**2)
         nu = 2 * mu - zeta
-        rho = 1 - zeta * a ** 2 / b ** 2
-        omega = (4 * nu - 1 - b ** 2 * sin(alpha) ** 2 / a ** 2) * c / (2 * b * cos(alpha))
+        rho = 1 - zeta * a**2 / b**2
+        omega = (4 * nu - 1 - b**2 * sin(alpha) ** 2 / a**2) * c / (2 * b * cos(alpha))
         delta = zeta * c * cos(alpha) / b + omega / 2 - 0.25
         kpoints = {
             "\\Gamma": np.array([0.0, 0.0, 0.0]),
@@ -1072,13 +1071,13 @@ class KPathLatimerMunro(KPathBase):
     combination of lattice vectors which can produce a vector of
     lesser magnitude than the given set (this is required to
     obtain the correct Brillouin zone within the current
-    implementaiton). This is checked during initialization and a
+    implementation). This is checked during initialization and a
     warning is issued if the condition is not fulfilled.
     In the case of magnetic structures, care must also be taken to
     provide the magnetic primitive cell (i.e. that which reproduces
     the entire crystal, including the correct magnetic ordering,
     upon application of lattice translations). There is no way to
-    programatically check for this, so if the input structure is
+    programmatically check for this, so if the input structure is
     incorrect, the class will output the incorrect kpath without
     any warning being issued.
     """
@@ -1168,8 +1167,8 @@ class KPathLatimerMunro(KPathBase):
         # 1: Get lattices of real and reciprocal structures, and reciprocal
         # point group, and Brillouin zone (BZ)
 
-        V = self._latt.matrix.T  # fractional real space to cartesian real space
-        # fractional reciprocal space to cartesian reciprocal space
+        V = self._latt.matrix.T  # fractional real space to Cartesian real space
+        # fractional reciprocal space to Cartesian reciprocal space
         W = self._rec_lattice.matrix.T
         # fractional real space to fractional reciprocal space
         A = np.dot(np.linalg.inv(W), V)
@@ -1306,7 +1305,7 @@ class KPathLatimerMunro(KPathBase):
             orbit_cosines.append(
                 sorted(
                     sorted(
-                        [
+                        (
                             (
                                 j,
                                 np.round(
@@ -1317,7 +1316,7 @@ class KPathLatimerMunro(KPathBase):
                             )
                             for k in orbit
                             for j in range(26)
-                        ],
+                        ),
                         key=operator.itemgetter(0),
                     ),
                     key=operator.itemgetter(1),
@@ -1346,9 +1345,9 @@ class KPathLatimerMunro(KPathBase):
                     max_occurence = 0
                 else:
                     if "'" not in point_label:
-                        max_occurence = max([int(label[3:-1]) for label in existing_labels[1:]])
+                        max_occurence = max(int(label[3:-1]) for label in existing_labels[1:])
                     else:
-                        max_occurence = max([int(label[4:-1]) for label in existing_labels[1:]])
+                        max_occurence = max(int(label[4:-1]) for label in existing_labels[1:])
                 kpoints[point_label + "_{" + str(max_occurence + 1) + "}"] = key_points[point_ind]
                 reverse_kpoints[point_ind] = point_label + "_{" + str(max_occurence + 1) + "}"
 
@@ -1477,7 +1476,7 @@ class KPathLatimerMunro(KPathBase):
         face_center_inds = []
         bz_as_key_point_inds = []
 
-        # pymatgen gives BZ in cartesian coordinates; convert to fractional in
+        # pymatgen gives BZ in Cartesian coordinates; convert to fractional in
         # the primitive basis for reciprocal space
         for (i, facet) in enumerate(bz):
             for (j, vert) in enumerate(facet):
@@ -1676,7 +1675,7 @@ class KPathLatimerMunro(KPathBase):
 
     def _get_little_groups(self, key_points, key_points_inds_orbits, key_lines_inds_orbits):
 
-        little_groups_points = []  # elements are lists of indicies of recip_point_group. the
+        little_groups_points = []  # elements are lists of indices of recip_point_group. the
         # list little_groups_points[i] is the little group for the
         # orbit key_points_inds_orbits[i]
         for (i, orbit) in enumerate(key_points_inds_orbits):
@@ -1690,7 +1689,7 @@ class KPathLatimerMunro(KPathBase):
                 if check_gamma:
                     little_groups_points[i].append(j)
 
-        # elements are lists of indicies of recip_point_group. the list
+        # elements are lists of indices of recip_point_group. the list
         # little_groups_lines[i] is
         little_groups_lines = []
         # the little group for the orbit key_points_inds_lines[i]
@@ -1736,7 +1735,7 @@ class KPathLatimerMunro(KPathBase):
                 new_magmoms.append(magmom * magmom_axis)
 
         if found_scalar and not axis_specified:
-            warn("At least one magmom had a scalar value and magmom_axis was not specified." "Defaulted to z+ spinor.")
+            warn("At least one magmom had a scalar value and magmom_axis was not specified. Defaulted to z+ spinor.")
 
         struct.remove_site_property("magmom")
         struct.add_site_property("magmom", new_magmoms)
@@ -2204,7 +2203,7 @@ class KPathLatimerMunro(KPathBase):
         while len(orbit_labels_unsorted) < len(orbit_cosines_orig) + 1:
             pop_orbits = []
             pop_labels = []
-            max_cosine_value = max([orb_cos[0][1] for orb_cos in orbit_cosines_copy])
+            max_cosine_value = max(orb_cos[0][1] for orb_cos in orbit_cosines_copy)
             max_cosine_value_inds = [
                 j for j in range(len(orbit_cosines_copy)) if orbit_cosines_copy[j][0][1] == max_cosine_value
             ]
