@@ -1,4 +1,3 @@
-# coding: utf-8
 # Copyright (c) Pymatgen Development Team.
 # Distributed under the terms of the MIT License.
 
@@ -65,8 +64,9 @@ class RotationTransformation(AbstractTransformation):
         return s
 
     def __str__(self):
-        return "Rotation Transformation about axis " + "{} with angle = {:.4f} {}".format(
-            self.axis, self.angle, "radians" if self.angle_in_radians else "degrees"
+        return (
+            f"Rotation Transformation about axis {self.axis} with angle = "
+            f"{self.angle:.4f} {'radians' if self.angle_in_radians else 'degrees'}"
         )
 
     def __repr__(self):
@@ -197,7 +197,6 @@ class OxidationStateRemovalTransformation(AbstractTransformation):
         """
         No arg needed.
         """
-        pass
 
     def apply_transformation(self, structure):  # pylint: disable=R0201
         """
@@ -275,7 +274,7 @@ class SupercellTransformation(AbstractTransformation):
         return structure * self.scaling_matrix
 
     def __str__(self):
-        return "Supercell Transformation with scaling matrix " + "{}".format(self.scaling_matrix)
+        return "Supercell Transformation with scaling matrix " + f"{self.scaling_matrix}"
 
     def __repr__(self):
         return self.__str__()
@@ -477,9 +476,9 @@ class PartialRemoveSpecieTransformation(AbstractTransformation):
 
     def __str__(self):
         spec_str = [
-            "Species = {}".format(self.specie_to_remove),
-            "Fraction to remove = {}".format(self.fraction_to_remove),
-            "ALGO = {}".format(self.algo),
+            f"Species = {self.specie_to_remove}",
+            f"Fraction to remove = {self.fraction_to_remove}",
+            f"ALGO = {self.algo}",
         ]
         return "PartialRemoveSpecieTransformation : " + ", ".join(spec_str)
 
@@ -506,7 +505,7 @@ class OrderDisorderedStructureTransformation(AbstractTransformation):
     approximately 5,000,000 permutations per minute.
 
     Also, simple rounding of the occupancies are performed, with no attempt
-    made to achieve a target composition.  This is usually not a problem for
+    made to achieve a target composition. This is usually not a problem for
     most ordering problems, but there can be times where rounding errors may
     result in structures that do not have the desired composition.
     This second step will be implemented in the next iteration of the code.
@@ -578,7 +577,7 @@ class OrderDisorderedStructureTransformation(AbstractTransformation):
         if self.no_oxi_states:
             structure = Structure.from_sites(structure)
             for i, site in enumerate(structure):
-                structure[i] = {"%s0+" % k.symbol: v for k, v in site.species.items()}
+                structure[i] = {f"{k.symbol}0+": v for k, v in site.species.items()}
 
         equivalent_sites = []
         exemplars = []
@@ -609,12 +608,12 @@ class OrderDisorderedStructureTransformation(AbstractTransformation):
 
         m_list = []
         for g in equivalent_sites:
-            total_occupancy = sum([structure[i].species for i in g], Composition())
+            total_occupancy = sum((structure[i].species for i in g), Composition())
             total_occupancy = dict(total_occupancy.items())
             # round total occupancy to possible values
             for k, v in total_occupancy.items():
                 if abs(v - round(v)) > 0.25:
-                    raise ValueError("Occupancy fractions not consistent " "with size of unit cell")
+                    raise ValueError("Occupancy fractions not consistent with size of unit cell")
                 total_occupancy[k] = int(round(v))
             # start with an ordered structure
             initial_sp = max(total_occupancy.keys(), key=lambda x: abs(x.oxi_state))
@@ -710,7 +709,7 @@ class PrimitiveCellTransformation(AbstractTransformation):
         """
         Args:
             tolerance (float): Tolerance for each coordinate of a particular
-                site. For example, [0.5, 0, 0.5] in cartesian coordinates will be
+                site. For example, [0.5, 0, 0.5] in Cartesian coordinates will be
                 considered to be on the same coordinates as [0, 0, 0] for a
                 tolerance of 0.5. Defaults to 0.5.
         """
@@ -840,7 +839,7 @@ class PerturbStructureTransformation(AbstractTransformation):
         return s
 
     def __str__(self):
-        return "PerturbStructureTransformation : " + "Min_distance = {}".format(self.min_distance)
+        return "PerturbStructureTransformation : " + f"Min_distance = {self.min_distance}"
 
     def __repr__(self):
         return self.__str__()
@@ -886,7 +885,7 @@ class DeformStructureTransformation(AbstractTransformation):
         return self._deform.apply_to_structure(structure)
 
     def __str__(self):
-        return "DeformStructureTransformation : " + "Deformation = {}".format(str(self.deformation))
+        return f"DeformStructureTransformation : Deformation = {self.deformation}"
 
     def __repr__(self):
         return self.__str__()
@@ -1011,7 +1010,7 @@ class ChargedCellTransformation(AbstractTransformation):
         return s
 
     def __str__(self):
-        return "Structure with charge " + "{}".format(self.charge)
+        return f"Structure with charge {self.charge}"
 
     def __repr__(self):
         return self.__str__()
@@ -1087,7 +1086,7 @@ class ScaleToRelaxedTransformation(AbstractTransformation):
 
         params = list(structure.lattice.abc)
         params.extend(structure.lattice.angles)
-        new_lattice = Lattice.from_parameters(*[p * self.params_percent_change[i] for i, p in enumerate(params)])
+        new_lattice = Lattice.from_parameters(*(p * self.params_percent_change[i] for i, p in enumerate(params)))
         species, frac_coords = [], []
         for site in self.relaxed_structure:
             species.append(s_map[site.specie])
