@@ -1,4 +1,3 @@
-# coding: utf-8
 # Copyright (c) Pymatgen Development Team.
 # Distributed under the terms of the MIT License.
 
@@ -13,7 +12,6 @@ right away.
 import json
 import tempfile
 import unittest
-from io import open
 from pathlib import Path
 
 import numpy.testing as nptu
@@ -40,7 +38,7 @@ class PymatgenTest(unittest.TestCase):
         import warnings
 
         warnings.warn(
-            "It is recommended that you set the PMG_TEST_FILES_DIR environment variable explicity. "
+            "It is recommended that you set the PMG_TEST_FILES_DIR environment variable explicitly. "
             "Now using a fallback location based on relative path from this module."
         )
         TEST_FILES_DIR = MODULE_DIR / ".." / ".." / "test_files"
@@ -129,7 +127,7 @@ class PymatgenTest(unittest.TestCase):
         failed = []
         for l1, l2 in zip(lines1, lines2):
             if l1.strip() != l2.strip():
-                failed.append("%s != %s" % (l1, l2))
+                failed.append(f"{l1} != {l2}")
         return len(failed) == 0
 
     def serialize_with_pickle(self, objects, protocols=None, test_eq=True):
@@ -161,7 +159,6 @@ class PymatgenTest(unittest.TestCase):
             objects = [objects]
 
         if protocols is None:
-            # protocols = set([0, 1, 2] + [pickle.HIGHEST_PROTOCOL])
             protocols = [pickle.HIGHEST_PROTOCOL]
 
         # This list will contains the object deserialized with the different
@@ -177,14 +174,14 @@ class PymatgenTest(unittest.TestCase):
                 with open(tmpfile, mode) as fh:
                     pmg_pickle_dump(objects, fh, protocol=protocol)
             except Exception as exc:
-                errors.append("pickle.dump with protocol %s raised:\n%s" % (protocol, str(exc)))
+                errors.append(f"pickle.dump with protocol {protocol} raised:\n{exc}")
                 continue
 
             try:
                 with open(tmpfile, "rb") as fh:
                     new_objects = pmg_pickle_load(fh)
             except Exception as exc:
-                errors.append("pickle.load with protocol %s raised:\n%s" % (protocol, str(exc)))
+                errors.append(f"pickle.load with protocol {protocol} raised:\n{exc}")
                 continue
 
             # Test for equality
