@@ -1,4 +1,3 @@
-# coding: utf-8
 # Copyright (c) Pymatgen Development Team.
 # Distributed under the terms of the MIT License.
 
@@ -9,7 +8,7 @@ change until best practices are established.
 """
 
 from pathlib import Path
-from typing import Union, Sequence
+from typing import Sequence, Union
 
 import numpy as np
 
@@ -17,8 +16,8 @@ try:
     from numpy.typing import ArrayLike
 except ImportError:
     ArrayLike = Union[Sequence[float], Sequence[Sequence[float]], Sequence[np.ndarray], np.ndarray]  # type: ignore
-from pymatgen.core.periodic_table import Element, Species, DummySpecies
 from pymatgen.core.composition import Composition
+from pymatgen.core.periodic_table import DummySpecies, Element, Species
 
 VectorLike = Union[Sequence[float], np.ndarray]
 MatrixLike = Union[Sequence[Sequence[float]], Sequence[np.ndarray], np.ndarray]
